/*
			   ShipWars Universe & Objects
 */

#ifndef OBJECTS_H
#define OBJECTS_H

#include <limits.h>
#include <sys/types.h>
#include "eco.h"


typedef struct _xsw_vector_compoent_struct	xsw_vector_compoent_struct;
typedef struct _unv_head_struct		unv_head_struct;
typedef struct _xsw_ecoproduct_struct	xsw_ecoproduct_struct;
typedef struct _xsw_ecodata_struct	xsw_ecodata_struct;
typedef struct _xsw_score_struct	xsw_score_struct;
typedef struct _permission_struct	permission_struct;
typedef struct _xsw_weapon_struct	xsw_weapon_struct;
typedef struct _xsw_animation_struct	xsw_animation_struct;
typedef struct _xsw_object_struct	xsw_object_struct;


/*
 *	Garbage Object Name String:
 */
#define SW_OBJ_NAME_GARBAGE		"*Garbage*"

/*
 *	Default Object Permissions:
 *
 *	Valid values are from 5 to 0, where 0 is the highest permission
 *	level.
 */
#define SW_DEF_UID			5
#define SW_DEF_GID			5


/*
 *	String Maximums:
 *
 *      Includes the null terminating byte.
 */
#define SW_OBJ_NAME_MAX			33
#define SW_OBJ_PASSWORD_MAX		33
#define SW_OBJ_EMPIRE_MAX		9
#define SW_UNV_TITLE_MAX		128


/*
 *	Invalid Name Characters:
 *
 *	A string containing a list of characters (excluding the null
 *	terminating byte) not permitted in an object's name.
 */
#define SW_INVALID_NAME_CHARACTERS	";,=\n\r\t#"


/*
 *	Maximums:
 */
#define SW_OBJECTS_MAX			100000		/* Per universe */
#define SW_OBJ_WEAPONS_MAX		9		/* Per object */
#define SW_CREDITS_MAX			2147483647


/*
 *	Default Empire:
 */
#define SW_DEF_EMPIRE			"IND"


/*
 *	Default Shield Frequency:
 *
 *	Must be between SWR_FREQ_MIN and SWR_FREQ_MAX.
 */
#define SW_DEF_SHIELD_FREQ		180.20


/*
 *	Default Com Channel:
 *
 *	This value is an (int) that needs to be * 100 of the actual
 *	value between SWR_FREQ_MIN and SWR_FREQ_MAX..
 */
#define SW_DEF_COM_CHANNEL		31000


/*
 *	Tractor Beam Maximums:
 */
#define SW_TRACTOR_BEAM_MAX_OBJECTS	3
#define SW_TRACTOR_BEAM_MAX_RANGE	2.0	/* In real units */


/*
 *	Object Types & Type Names:
 */
#define SW_OBJ_TYPE_ERROR		-1
#define SW_OBJ_TYPE_NAME_ERROR		"Error"

#define SW_OBJ_TYPE_GARBAGE		0
#define SW_OBJ_TYPE_NAME_GARBAGE	"Garbage"

#define SW_OBJ_TYPE_STATIC		1	/* Non moving */
#define SW_OBJ_TYPE_NAME_STATIC		"Static"

#define SW_OBJ_TYPE_DYNAMIC		2	/* Moves */
#define SW_OBJ_TYPE_NAME_DYNAMIC	"Dynamic"

#define SW_OBJ_TYPE_CONTROLLED		3	/* AI controllable */
#define SW_OBJ_TYPE_NAME_CONTROLLED	"Controlled"

#define SW_OBJ_TYPE_PLAYER		4
#define SW_OBJ_TYPE_NAME_PLAYER		"Player"

#define SW_OBJ_TYPE_WEAPON		5	/* Pulse or projectile */
#define SW_OBJ_TYPE_NAME_WEAPON		"Weapon"

#define SW_OBJ_TYPE_STREAMWEAPON	6	/* Stream */
#define SW_OBJ_TYPE_NAME_STREAMWEAPON	"StreamWeapon"

#define SW_OBJ_TYPE_SPHEREWEAPON	7	/* Expanding sphere */
#define SW_OBJ_TYPE_NAME_SPHEREWEAPON	"SphereWeapon"

#define SW_OBJ_TYPE_HOME		10	/* Starbases, planets, etc */
#define SW_OBJ_TYPE_NAME_HOME		"Home"

#define SW_OBJ_TYPE_AREA		11	/* Nebulas, etc */
#define SW_OBJ_TYPE_NAME_AREA		"Area"

#define SW_OBJ_TYPE_ANIMATED		12	/* Explosions */
#define SW_OBJ_TYPE_NAME_ANIMATED	"Animated"

#define SW_OBJ_TYPE_WORMHOLE		13	/* Aka gates */
#define SW_OBJ_TYPE_NAME_WORMHOLE	"Wormhole"

#define SW_OBJ_TYPE_ELINK		14	/* External server link */
#define SW_OBJ_TYPE_NAME_ELINK		"Elink"


/*
 *	Location Type Codes:
 *
 *	Used by objects of type SW_OBJ_TYPE_AREA to determine it
 *	area type or by all other objects to determine what area type
 *	it is in.
 */
#define SW_LOC_TYPE_SPACE		0	/* Normal space */
#define SW_LOC_TYPE_NAME_SPACE		"Space"

#define SW_LOC_TYPE_SPACE_TERRITORY	1	/* Normal space that is claimed */
#define SW_LOC_TYPE_NAME_SPACE_TERRITORY	"SpaceTerritory"

#define SW_LOC_TYPE_NEBULA		2
#define SW_LOC_TYPE_NAME_NEBULA		"Nebula"


/*
 *	IFF:
 */
#define SW_IFF_UNKNOWN			0
#define SW_IFF_FRIENDLY			1
#define SW_IFF_HOSTILE			2

/*
 *	Engine states:
 */
#define SW_ENGINE_STATE_NONE		-1
#define SW_ENGINE_STATE_OFF		0
#define SW_ENGINE_STATE_STARTING	1
#define SW_ENGINE_STATE_ON		2

#define SW_ENGINE_STATE_STR_NONE	"None"
#define SW_ENGINE_STATE_STR_OFF		"Off"
#define SW_ENGINE_STATE_STR_STARTING	"Starting"
#define SW_ENGINE_STATE_STR_ON		"On"


/*
 *	Shield generator states:
 */
#define SHIELD_STATE_NONE		-1
#define SHIELD_STATE_DOWN		0
#define SHIELD_STATE_UP			1

#define SW_SHIELD_STATE_STR_NONE	"None"
#define SW_SHIELD_STATE_STR_DOWN	"Down"
#define SW_SHIELD_STATE_STR_UP		"Up"


/*
 *	Cloak States:
 */
#define CLOAK_STATE_NONE		-1
#define CLOAK_STATE_DOWN		0
#define CLOAK_STATE_UP			1

#define SW_CLOAK_STATE_STR_NONE		"None"
#define SW_CLOAK_STATE_STR_DOWN		"Down"
#define SW_CLOAK_STATE_STR_UP		"Up"


/*
 *	Damage Control States:
 */
#define DMGCTL_STATE_OFF		0
#define DMGCTL_STATE_ON			1

#define SW_DMGCTL_STATE_STR_OFF		"Off"
#define SW_DMGCTL_STATE_STR_ON		"On"


/*
 *	Weapon Emission Types:
 *
 *      These determine the firing style of the weapon.
 */
#define SW_WEP_EMISSION_STREAM		0	/* Supply = power left */
#define SW_WEP_EMISSION_PROJECTILE	1	/* Limited supply */
#define SW_WEP_EMISSION_PULSE		2	/* Supply = power left */

#define SW_WEP_EMISSION_STR_STREAM	"Stream"
#define SW_WEP_EMISSION_STR_PROJECTILE	"Projectile"
#define SW_WEP_EMISSION_STR_PULSE	"Pulse"


/*
 *	Weapon Flags:
 */
#define SW_WEP_FLAG_NO_LOCK		(1 << 0)	/* Does not lock on to target */
#define SW_WEP_FLAG_FIRE_CLOAKED	(1 << 1)	/* Can fire while cloaked */
#define SW_WEP_FLAG_NO_FIRE_SOUND	(1 << 2)	/* Quiet */
#define SW_WEP_FLAG_FIXED		(1 << 3)	/* No rotate */
#define SW_WEP_FLAG_PENETRATE_SHIELD	(1 << 4)	/* Always damages hull */

#define SW_WEP_FLAG_STR_NO_LOCK		"NoLock"
#define SW_WEP_FLAG_STR_FIRE_CLOAKED	"FireCloaked"
#define SW_WEP_FLAG_STR_NO_FIRE_SOUND	"NoFireSound"
#define SW_WEP_FLAG_STR_FIXED		"Fixed"
#define SW_WEP_FLAG_STR_PENETRATE_SHIELD	"PenetrateShield"

/*
 *	Lighting flags:
 *
 *	Vector lights, strobes, etc.
 *
 *	Set in member lighting of each object.
 */
#define SW_OBJ_LIGHT_VECTOR		(1 << 0)
#define SW_OBJ_LIGHT_STROBE		(1 << 1)
#define SW_OBJ_LIGHT_LUMINATION		(1 << 2)	/* Head lights */

#define SW_OBJ_LIGHT_STR_VECTOR		"Vector"
#define SW_OBJ_LIGHT_STR_STROBE		"Strobe"
#define SW_OBJ_LIGHT_STR_LUMINATION	"Lumination"

/*
 *	AI (controllable objects) AI flags:
 *
 *	Set in member ai_flags of each object.
 */
#define SW_AI_FOLLOW_FRIEND		(1 << 1)
#define SW_AI_FOLLOW_UNKNOWN		(1 << 2)
#define SW_AI_FOLLOW_HOSTILE		(1 << 3)
#define SW_AI_FOLLOW_ANY		(SW_AI_FOLLOW_FRIEND | \
					 SW_AI_FOLLOW_UNKNOWN | \
					 SW_AI_FOLLOW_HOSTILE)
#define SW_AI_FIRE_FRIEND		(1 << 8)
#define SW_AI_FIRE_UNKNOWN		(1 << 9)
#define SW_AI_FIRE_HOSTILE		(1 << 10)
#define SW_AI_FIRE_ANY			(SW_AI_FIRE_FRIEND | \
					 SW_AI_FIRE_UNKNOWN | \
					 SW_AI_FIRE_HOSTILE)

#define SW_AI_STR_FOLLOW_FRIEND		"FOLLOW_FRIEND"
#define SW_AI_STR_FOLLOW_UNKNOWN	"FOLLOW_UNKNOWN"
#define SW_AI_STR_FOLLOW_HOSTILE	"FOLLOW_HOSTILE"

#define SW_AI_STR_FIRE_FRIEND		"FIRE_FRIEND"
#define SW_AI_STR_FIRE_UNKNOWN		"FIRE_UNKNOWN"
#define SW_AI_STR_FIRE_HOSTILE		"FIRE_HOSTILE"


/*
 *	Universe Header:
 */
struct _unv_head_struct {

	char		title[SW_UNV_TITLE_MAX];
	char		version[SW_UNV_TITLE_MAX];

	char		ocsn[PATH_MAX + NAME_MAX];	/* OCS Names */
	char		imgrefs[PATH_MAX + NAME_MAX];	/* Image references */
	char		sndrefs[PATH_MAX + NAME_MAX];	/* Sound references */

	/* Unit conversions */
	double		ru_to_au;

	/* Lost and found owner (must be a player) */
	int		lost_found_owner;

	/* New player start position */
	long		player_start_sect_x,
			player_start_sect_y,
			player_start_sect_z;

	double		player_start_x,
			player_start_y,
			player_start_z;

	double		player_start_heading,
			player_start_pitch,
			player_start_bank;

	/* Guest start position */
	long		guest_start_sect_x,
			guest_start_sect_y,
			guest_start_sect_z;

	double		guest_start_x,
			guest_start_y,
			guest_start_z;

	double		guest_start_heading,
			guest_start_pitch,
			guest_start_bank;

};


/*
 *	XSW Object data types:
 */
typedef u_int64_t	sw_flags_t;	/* All purpose flags type */
typedef double		sw_credits_t;	/* Credits */
typedef double		sw_rmu_t;	/* Raw Materials */


/*
 *	Vector Compoent:
 */
struct _xsw_vector_compoent_struct {

	double i, j, k;

};

/*
 *	Economy Product:
 */
struct _xsw_ecoproduct_struct {

	char		name[ECO_PRODUCT_NAME_MAX];

	/* Cost of product per 1 unit */
	sw_credits_t	sell_price,	/* Price if being bought from customer */
			buy_price;	/* Price if being sold to customer */

	double 		amount,		/* How much in sellable stock */
			amount_max;	/* Maximum possible in stock */

};

/*
 *	Economy:
 */
struct _xsw_ecodata_struct {

	eco_flags_t	flags;		/* General flags */

	/* Tax coefficient value applied to unit buy or sell price
	 * For example, 1.2 would mean 20%, so if the unit buy price
	 * is 10 credits then with tax applied it would be 12 credits
	 */
	double		tax_general,
			tax_friend,
			tax_hostile;

	xsw_ecoproduct_struct	**product;
	int			total_products;

};


/*
 *	Score:
 */
struct _xsw_score_struct {

	sw_credits_t	credits;	/* Universal credits (money) */
	sw_rmu_t	rmu,		/* Raw material units */
			rmu_max;	/* Maximum raw material units */
	double		damage_given;
	double		damage_recieved;
	int		kills;

};


/*
 *	Permission:
 */
struct _permission_struct {

	int		uid,		/* User ID, 0 is highest */
			gid;		/* Group ID, 0 is highest */

};


/*
 *	Weapon:
 */
struct _xsw_weapon_struct {

	sw_flags_t	flags;		/* Any SW_WEP_FLAG_* */

	int		ocs_code;	/* Object create script to use to
					 * fire this weapon */
	int		emission_type;	/* One of SW_WEP_EMISSION_* */

	/* Quantity remaining and max, for emission_type =
	 * SW_WEP_EMISSION_PROJECTILE only */
	int		amount,
			max;

	double		power;		/* Amount of damage to be caused per
					 * unit hit with yield at 100% */

	long		range;		/* Length of stream weapons (in
					 * screen units), for emission_type =
					 * SW_WEP_EMISSION_PROJECTILE only */

	double		create_power;	/* Power needed to create unit fire */

	time_t		delay,          /* Interval between firings (in
					 * milliseconds) */
			last_used;      /* Last fired (in milliseconds) */

	/* Sound codes */
	int		use_sound_code,		/* Just before firing */
			fire_sound_code,	/* Fire sound */
			hit_sound_code,		/* When an object is hit */
			recover_sound_code;	/* When dead wep is recovered */

	char		name[SW_OBJ_NAME_MAX];

};


/*
 *	Animation:
 */
struct _xsw_animation_struct {

	time_t		interval;	/* Milliseconds between frames */
	time_t		last_interval;	/* Last millitime frame was inc */

	int		current_frame;	/* Current frame number */
	int		total_frames;	/* Total number of frames */

	int		cycle_count;	/* Times repeated */
	int 		cycle_times;	/* Total times to repeat */

};


/*
 *	ShipWars Object:
 */
struct _xsw_object_struct {

	int		type;		/* Type of object */

	sw_flags_t	ext_flags;	/* Proprietery flags used solely by
					 * the client, server, user, etc */

	char		name[SW_OBJ_NAME_MAX];
	char		*password;
	char		empire[SW_OBJ_EMPIRE_MAX];
	char		*description;
	char		*elink;		/* For elink objects */

	time_t		last_updated;	/* Last updated by client or
					 * server (in ms) */
	time_t		last_connected;	/* In seconds */

	int		loc_type;	/* Type of location object is in */
	int		imageset;	/* Image referance code */
	int		owner;		/* Object that owns this object */
	long		size;		/* Radius of object size in screen units */

	int		locked_on;	/* Scanning/locked on object, can be -1
					 * for unlocked */

	int		intercepting_object;	/* Object this object is
						 * intercepting, can be -1 for
						 * not intercepting */

	double		scanner_range;		/* In real units */

	long		sect_x,		/* Current sector */
			sect_y,
			sect_z;

	double		x,		/* Coordinate position in sector */
			y,
			z;

	double		heading,	/* In radians */
			pitch,		/* In radians (not implmented yet) */
			bank;		/* In radians (not implmented yet) */
	xsw_vector_compoent_struct attitude_vector_compoent;

	double		velocity;
	double		velocity_max;
	double		velocity_heading,	/* In radians */
			velocity_pitch,		/* In radians (not implmented yet) */
			velocity_bank;		/* In radians (not implmented yet) */
	xsw_vector_compoent_struct momentum_vector_compoent;

	double		thrust_dir;	/* In radians (for omni directional thrust) */
	double		thrust;		/* throttle * thrust_power */
	double		thrust_power;	/* Full power of thrust */
	double		throttle;	/* 0.0 to 1.0 */
	char		engine_state;	/* One of SW_ENGINE_STATE_* */
	double		turnrate;	/* In radians per cycle */

	sw_flags_t	lighting;	/* Vector lights, strobes, etc switches */

	double		hp,		/* Hull or density */
			hp_max;

	double		power,
			power_max;
	double		power_purity;	/* 0.0 to 1.0 */

	double		core_efficency;	/* In power units per cycle */

	double		antimatter,
			antimatter_max;

	int		shield_state;	/* One of SHIELD_STATE_* */
	double		shield_frequency;	/* In KHz */

	int		selected_weapon,
			total_weapons;	/* May not be greater than SW_OBJ_WEAPONS_MAX */

	time_t		birth_time;	/* In ms */
	time_t		lifespan;	/* In ms, 0 means never dies */

	int		creation_ocs;	/* The OCS used to create this
					 * object, 0 for none (used for
					 * weapons reclaiming) */

	int		cloak_state;	/* One of CLOAK_STATE_* */
	double		cloak_strength;	/* 0.0 to 1.0, when cloak is on,
					 * cloak_strength reduces visibility */
	double		visibility;	/* Normal visibility: 0.0 to 1.0 */
	double		cur_visibility;	/* Actual current visibility */

	double		shield_visibility;	/* When greater than 0.0,
						 * shields are visibable */

	int		damage_control;	/* One of DMGCTL_STATE_* */

	int		com_channel;	/* In * 100 units */

	time_t		auto_destruct_next;	/* Time to when auto destruct
						 * should occure in systime
						 * seconds (0 for none) */

	/* AI flags (for type XSW_OBJ_TYPE_CONTROLLED only) */
	sw_flags_t	ai_flags;


	/* Objects being tractored */
	int		*tractored_object;
	int		total_tractored_objects;


	/* Permissions */
	permission_struct	permission;

	/* Animation */
	xsw_animation_struct	animation;

	/* Score (dynamically allocated) */
	xsw_score_struct	*score;

	/* Weapons (dynamically allocated, limited to SW_OBJ_WEAPONS_MAX) */
	xsw_weapon_struct	**weapons;

	/* Economy (dynamically allocated) */
	xsw_ecodata_struct	*eco;

};


#endif /* OBJECTS_H */
