#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../include/string.h"
#include "../include/strexp.h"

#include "../include/osw-x.h"
#include "../include/widget.h"

#include "mon.h"
#include "conwin.h"


static void ConWinUnfocusPrompt(prompt_struct *prompt);
static void ConWinRefocusPrompts(
	con_win_struct *cw, prompt_struct *focus_prompt
);

static int ConWinConnectPBCB(void *data);
static int ConWinCancelPBCB(void *data);

int ConWinInit(
	con_win_struct *cw,
	void *client_data,
	int (*func_cb)(void *, char *, char *, char *, char *)
);
void ConWinDraw(con_win_struct *cw);
int ConWinManage(con_win_struct *cw, event_t *event);
void ConWinMap(con_win_struct *cw);
void ConWinUnmap(con_win_struct *cw);
void ConWinDestroy(con_win_struct *cw);


#define CW_DEF_WIDTH	380
#define CW_DEF_HEIGHT	165

#define CW_BTN_WIDTH	70
#define CW_BTN_HEIGHT	28


static void ConWinUnfocusPrompt(prompt_struct *prompt)
{
	if(prompt == NULL)
	    return;

	PromptUnmarkBuffer(prompt, PROMPT_POS_END);
	prompt->is_in_focus = False;

	if(prompt->map_state)
	    PromptDraw(prompt, PROMPT_DRAW_AMOUNT_TEXTONLY);
}

static void ConWinRefocusPrompts(
	con_win_struct *cw,
	prompt_struct *focus_prompt
)
{
	if(cw == NULL)
	    return;

	ConWinUnfocusPrompt(&cw->address_prompt);
	ConWinUnfocusPrompt(&cw->port_prompt);
	ConWinUnfocusPrompt(&cw->name_prompt);
	ConWinUnfocusPrompt(&cw->password_prompt);

	PromptMarkBuffer(focus_prompt, PROMPT_POS_END);
	focus_prompt->is_in_focus = True;

	if(focus_prompt->map_state)
	    PromptDraw(focus_prompt, PROMPT_DRAW_AMOUNT_TEXTONLY);
}


static int ConWinConnectPBCB(void *data)
{
	const char *s;
	char	address[HOST_NAME_MAX],
		port[80],
		name[80],
		password[80];
	con_win_struct *cw = (con_win_struct *)data;
	if(cw == NULL)
	    return(0);

	if(cw->func_cb != NULL)
	{
	    s = PromptGetS(&cw->address_prompt);
	    strncpy(
		address, (s != NULL) ? s : "",
		sizeof(address)
	    );
	    address[sizeof(address) - 1] = '\0';

	    s = PromptGetS(&cw->port_prompt);
	    strncpy(
		port, (s != NULL) ? s : "",
		sizeof(port)
	    );
	    port[sizeof(port) - 1] = '\0';

	    s = PromptGetS(&cw->name_prompt);
	    strncpy(
		name, (s != NULL) ? s : "",
		sizeof(name)
	    );
	    name[sizeof(name) - 1] = '\0';

	    s = PromptGetS(&cw->password_prompt);
	    strncpy(
		password, (s != NULL) ? s : "",
		sizeof(password)
	    );
	    password[sizeof(password) - 1] = '\0';

	    cw->func_cb(
		cw->client_data,
		address,
		port,
		name,
		password
	    );
	}

	ConWinUnmap(cw);

	return(0);
}

static int ConWinCancelPBCB(void *data)
{
	con_win_struct *cw = (con_win_struct *)data;
	if(cw == NULL)
	    return(0);
 
	ConWinUnmap(cw);

	return(0);
}


int ConWinInit(
	con_win_struct *cw,  
	void *client_data,
	int (*func_cb)(void *, char *, char *, char *, char *)
)
{
	win_t w, parent = osw_gui[0].root_win;
	win_attr_t wattr;
	char hotkeys[PBTN_MAX_HOTKEYS];

	if(cw == NULL)
	    return(-1);

	cw->map_state = False;
	cw->is_in_focus = False;
	cw->x = 0;
	cw->y = 0;
	cw->width = CW_DEF_WIDTH;
	cw->height = CW_DEF_HEIGHT;
	cw->visibility_state = VisibilityFullyObscured;

	cw->client_data = client_data;
	cw->func_cb = func_cb;

	/* Toplevel */
	OSWCreateWindow(
	    &w, parent,
	    cw->x, cw->y,
	    cw->width, cw->height
	);
	cw->toplevel = w;
	WidgetCenterWindow(w, WidgetCenterWindowToPointer);
	OSWGetWindowAttributes(w, &wattr);
	cw->x = wattr.x;
	cw->y = wattr.y;
	cw->width = wattr.width;
	cw->height = wattr.height;

	OSWSetWindowWMProperties(
	    w,
	    "Connect To Universe",	/* Title */
	    "Connect",			/* Icon title */
	    ((mon_image.monitor_icon_pm == 0) ?
		widget_global.std_icon_pm : mon_image.monitor_icon_pm),
	    False,		/* Let WM set coordinates? */
	    cw->x, cw->y,
	    cw->width, cw->height,
	    cw->width, cw->height,
	    OSW_WINDOW_FRAME_DIALOG,
	    0, NULL
	);
	OSWSetWindowInput(
	    w,
	    OSW_EVENTMASK_TOPLEVEL | ButtonPressMask | ButtonReleaseMask |
	    PointerMotionMask | ExposureMask |
	    EnterWindowMask | LeaveWindowMask
	);

	parent = w;

	/* Address prompt */
	PromptInit(
	    &cw->address_prompt, parent,
	    10, 10 + (30 * 0),
	    (int)cw->width - 130,
	    30,
	    PROMPT_STYLE_FLUSHED,
	    "Address:",
	    HOST_NAME_MAX, 0
	);

	/* Port prompt */
	PromptInit(
	    &cw->port_prompt, parent,
	    (int)cw->width - 130 + 15,
	    10 + (30 * 0),
	    130 - 25,
	    30,
	    PROMPT_STYLE_FLUSHED,
	    "Port:",
	    40, 0
	);

	/* Login name prompt */
	PromptInit(
	    &cw->name_prompt, parent,
	    10,
	    10 + (30 * 1) + 5,
	    (int)cw->width - 20,
	    30,
	    PROMPT_STYLE_FLUSHED,
	    "    Name:",
	    80, 0
	);

	/* Login password prompt */
	PromptInit(
	    &cw->password_prompt, parent,
	    10,
	    10 + (30 * 2) + 5, 
	    (int)cw->width - 20, 
	    30,
	    PROMPT_STYLE_FLUSHED,
	    "Password:",
	    80, 0
	);

	/* Link prompts togeather */
	cw->address_prompt.next = &cw->port_prompt;
	cw->address_prompt.prev = &cw->password_prompt;

	cw->port_prompt.next = &cw->name_prompt;
	cw->port_prompt.prev = &cw->address_prompt;

	cw->name_prompt.next = &cw->password_prompt;
	cw->name_prompt.prev = &cw->port_prompt;

	cw->password_prompt.next = &cw->address_prompt;
	cw->password_prompt.prev = &cw->name_prompt;


	/* Connect button */
	PBtnInit(
	    &cw->connect_btn, parent,
	    10,
	    (int)cw->height - CW_BTN_HEIGHT - 10,
	    CW_BTN_WIDTH, CW_BTN_HEIGHT,
	    "Connect",
	    PBTN_TALIGN_CENTER,
	    NULL,
	    cw, ConWinConnectPBCB
	);
	hotkeys[0] = '\n';
	hotkeys[1] = '\0';
	PBtnSetHotKeys(&cw->connect_btn, hotkeys);

	/* Cancel button */
	PBtnInit(
	    &cw->cancel_btn, parent,
	    (int)cw->width - CW_BTN_WIDTH - 10,
	    (int)cw->height - CW_BTN_HEIGHT - 10,
	    CW_BTN_WIDTH, CW_BTN_HEIGHT,
	    "Cancel",
	    PBTN_TALIGN_CENTER,
	    NULL,
	    cw, ConWinCancelPBCB
	);
	hotkeys[0] = 0x1b;
	hotkeys[1] = '\0';
	PBtnSetHotKeys(&cw->cancel_btn, hotkeys);

	return(0);
}


void ConWinDraw(con_win_struct *cw)
{
	int y;
	unsigned int width, height;
	font_t *prev_font;
	pixmap_t pixmap;
	win_t w;

	if(cw == NULL)
	    return;

	/* Map as needed */
	if(!cw->map_state)
	{
	    OSWMapRaised(cw->toplevel);
	    cw->map_state = True;

	    PromptMap(&cw->address_prompt);
	    PromptMap(&cw->port_prompt);
	    PromptMap(&cw->name_prompt);
	    PromptMap(&cw->password_prompt);

	    PBtnMap(&cw->connect_btn);
	    PBtnMap(&cw->cancel_btn);
	}

	/* Recreate buffers as needed */
	if(cw->toplevel_buf == 0)
	{
	    OSWGetWindowGeometry(
		cw->toplevel, NULL, NULL, &width, &height, NULL
	    );
	    if(OSWCreatePixmap(
		&cw->toplevel_buf, width, height
	    ))
		return;
	}

	prev_font = OSWQueryCurrentFont();


	/* Redraw toplevel */
	if(True)
	{
	    w = cw->toplevel;
	    pixmap = cw->toplevel_buf;
	    OSWGetWindowGeometry(
		w, NULL, NULL, &width, &height, NULL
	    );

	    if(widget_global.force_mono)
	    {
		OSWClearPixmap(
		    pixmap,
		    width, height,
		    osw_gui[0].black_pix
		);
	    }
	    else
	    {
		WidgetPutImageTile(
		    pixmap,
		    widget_global.std_bkg_img,
		    width, height
		);

		OSWSetFgPix(widget_global.surface_shadow_pix);
		y = (int)height - CW_BTN_HEIGHT - 25;
		OSWDrawLine(
		    pixmap,
		    0, y, width, y
		);

		OSWSetFgPix(widget_global.surface_highlight_pix);
		y = (int)height - CW_BTN_HEIGHT - 25 + 1;
		OSWDrawLine(
		    pixmap,
		    0, y, width, y
		);
	    }

	    OSWPutBufferToWindow(w, pixmap);
	}

	OSWSetFont(prev_font);
}


int ConWinManage(con_win_struct *cw, event_t *event)
{
	keycode_t keycode;
	win_t w;
	int events_handled = 0;

	if((cw == NULL) || (event == NULL))
	    return(events_handled);

	if(!cw->map_state && (event->type != MapNotify))
	    return(events_handled);

	w = event->xany.window;

	/* Manage events for monitor normally */
	switch(event->type)
	{
	  case KeyPress:
	    if(!cw->is_in_focus)
		break;

	    keycode = event->xkey.keycode;

	    break;

	  case KeyRelease:
	    if(!cw->is_in_focus)
		break;

	    keycode = event->xkey.keycode;

	    break;

	  case Expose:
	    if(w == cw->toplevel)
	    {
	        events_handled++;
	    }
	    break;

	  case FocusIn:
	    if(w == cw->toplevel)
	    {
		cw->is_in_focus = True;
		events_handled++;
		return(events_handled);
	    }
	    break;

	  case FocusOut:
	    if(w == cw->toplevel)
	    {
		cw->is_in_focus = False;
		events_handled++;
		return(events_handled);
	    }
	    break;

	  case MapNotify:
	    if(w == cw->toplevel)
	    {
		if(!cw->map_state)
		    ConWinMap(cw);

		events_handled++;
		return(events_handled);
	    }
	    break;

	  case UnmapNotify:
	    if(w == cw->toplevel)
	    {
		if(cw->map_state)
		    ConWinUnmap(cw);

		events_handled++;
		return(events_handled);
	    }
	    break;

	  case ClientMessage:
	    if(OSWIsEventDestroyWindow(cw->toplevel, event))
	    {
		ConWinUnmap(cw);

		events_handled++;
		return(events_handled);
	    }
	    break;
	}

	if(events_handled > 0)
	    ConWinDraw(cw);

	if(events_handled == 0)
	    events_handled += PBtnManage(&cw->connect_btn, event);
	if(events_handled == 0)
	    events_handled += PBtnManage(&cw->cancel_btn, event);

	if(events_handled == 0)
	    events_handled += PromptManage(&cw->address_prompt, event);
	if(events_handled == 0)
	    events_handled += PromptManage(&cw->port_prompt, event);
	if(events_handled == 0)
	    events_handled += PromptManage(&cw->name_prompt, event);
	if(events_handled == 0)
	    events_handled += PromptManage(&cw->password_prompt, event);

	return(events_handled);
}


void ConWinMap(con_win_struct *cw)
{
	if(cw == NULL)
	    return;

	cw->map_state = False;
	ConWinDraw(cw);

	ConWinRefocusPrompts(
	    cw,
	    &cw->address_prompt
	);
}

void ConWinUnmap(con_win_struct *cw)
{
	if(cw == NULL)
	    return;

	OSWUnmapWindow(cw->toplevel);
	cw->map_state = False;
	cw->is_in_focus = False;
	cw->visibility_state = VisibilityFullyObscured;

	OSWDestroyPixmap(&cw->toplevel_buf);
}


void ConWinDestroy(con_win_struct *cw)
{
	if(cw == NULL)
	    return;

	PBtnDestroy(&cw->connect_btn);
	PBtnDestroy(&cw->cancel_btn);

	PromptDestroy(&cw->address_prompt);
	PromptDestroy(&cw->port_prompt);
	PromptDestroy(&cw->name_prompt);
	PromptDestroy(&cw->password_prompt);

	OSWDestroyWindow(&cw->toplevel);
	OSWDestroyPixmap(&cw->toplevel_buf);

	cw->client_data = NULL;
	cw->func_cb = NULL;
}
