#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../include/unvmatch.h"

#include "net.h"
#include "sysparm.h"
#include "swserv.h"


int CmdBoot(int condescriptor, const char *arg);


#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)	(((s) != NULL) ? strdup(s) : NULL)

#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : 1)


#define THIS_CMD_NAME	"boot"


/*
 *      Boot off a player.
 */
int CmdBoot(int condescriptor, const char *arg)
{
	int con_num, con_obj_num, obj_num;
	const char *name;
	char	sndbuf[CS_DATA_MAX_LEN],
		name1[SW_OBJ_NAME_MAX + 80],
		name2[SW_OBJ_NAME_MAX + 80];
	xsw_object_struct *con_obj;
	sws_connection_struct *con = connection[condescriptor];

	con_obj_num = con->object_num;
	con_obj = xsw_object[con_obj_num];

	/* Print usage? */ 
	if((arg != NULL) ? (*arg == '\0') : 1)
	{
	    sprintf(
		sndbuf,
"Usage: `%s <player>'",
		THIS_CMD_NAME
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}

	/* Get name */
	name = arg;

	/* Permission check */
	if(con_obj->permission.uid > SWS_ACCESS_UID_BOOT)
	{
	    sprintf(
		sndbuf,
"%s: Permission denied: Requires access level %i.",
		THIS_CMD_NAME, SWS_ACCESS_UID_BOOT
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}

	/* Match the object_num to be booted */
	if(!strcasecmp(name, "me"))
	{
	    obj_num = con_obj_num;
	}
	else
	{
	    obj_num = MatchObjectByName(
		xsw_object, total_objects,
		name, SW_OBJ_TYPE_PLAYER
	    );
	}

	/* Valid match? */
	if(DBIsObjectGarbage(obj_num))
	{
	    sprintf(
		sndbuf,
"%s: No such player.",
		name
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}

	/* Is that object connected? */
	con_num = ConGetByObject(obj_num);
	if(con_num < 0)
	{
	    sprintf(
		sndbuf,
"%s: Not connected.",
		name
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}

	/* Get formal names of objects */
	strncpy(
	    name1, DBGetFormalNameStr(con_obj_num),
	    sizeof(name1)
	);
	name1[sizeof(name1) - 1] = '\0';

	strncpy(
	    name2, DBGetFormalNameStr(obj_num),
	    sizeof(name2)
	);
	name2[sizeof(name2) - 1] = '\0';


	/* Notify connection */
	sprintf(
	    sndbuf,
	    "Booted %s on connection %i.",
	    name2,
	    con_num
	);
	NetSendLiveMessage(condescriptor, sndbuf);

	/* Log */
	if(sysparm.log_general)
	{
	    char *s = (char *)malloc(
		STRLEN(name1) + STRLEN(name2) + 40 + 80
	    );
	    sprintf(
		s,
		"%s: Booted: %s on connection %i.",
		name1, name2, con_num
	    );
	    LogAppend(fname.log, s);
	    free(s);
	}

	/* Boot connection */
	NetCloseConnection(con_num);

	return(0);
}
