#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../include/string.h"
#include "../include/unvmatch.h"

#include "net.h"
#include "sysparm.h"
#include "swserv.h"


int CmdChown(int condescriptor, const char *arg);


#define THIS_CMD_NAME	"chown"


int CmdChown(int condescriptor, const char *arg)
{
	int tar_obj_num, con_obj_num, new_owner_obj_num;
	char	*s,
		sndbuf[CS_DATA_MAX_LEN],
		tar_obj_name[SW_OBJ_NAME_MAX + 80],
		con_obj_name[SW_OBJ_NAME_MAX + 80],
		new_owner_obj_name[SW_OBJ_NAME_MAX + 80],
		text[512 + (SW_OBJ_NAME_MAX * 3)];
	xsw_object_struct *tar_obj, *con_obj, *new_owner_obj;
	sws_connection_struct *con = connection[condescriptor];

	/* Get connection's object number (assumed valid) */
	con_obj_num = con->object_num;
	con_obj = xsw_object[con_obj_num];

	/* If argument is empty, then print usage */
	if((arg != NULL) ? (*arg == '\0') : 1)
	{
	    sprintf(
		sndbuf,
		"Usage: `%s <object>[=<owner>]'",
		THIS_CMD_NAME
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}


	/* Parse target object name */
	strncpy(tar_obj_name, arg, sizeof(tar_obj_name));
	tar_obj_name[sizeof(tar_obj_name) - 1] = '\0';
	s = strchr(tar_obj_name, '=');
	if(s != NULL) 
	    *s = '\0';
	strstrip(tar_obj_name);

	/* Match target object name */
	tar_obj_num = MatchObjectByName(
	    xsw_object, total_objects,
	    tar_obj_name, -1
	);
	if(DBIsObjectGarbage(tar_obj_num))
	{
	    sprintf(
		sndbuf,
		"%s: %s: No such object.",
		THIS_CMD_NAME,
		tar_obj_name
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}

	tar_obj = xsw_object[tar_obj_num];

	/* Players may not be chowned */
	if(tar_obj->type == SW_OBJ_TYPE_PLAYER)
	{
	    sprintf(
		sndbuf,
		"%s: Permission denied: Players always own themselves.",
		THIS_CMD_NAME
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}

	/* Parse new owner object name */
	s = strchr(arg, '=');
	if(s == NULL)
	    strncpy(
		new_owner_obj_name,
		con_obj->name,
		sizeof(new_owner_obj_name)
	    );
	else
	    strncpy(
		new_owner_obj_name,
		s + 1,
		sizeof(new_owner_obj_name)
	    );
	new_owner_obj_name[sizeof(new_owner_obj_name) - 1] = '\0';
	strstrip(new_owner_obj_name);

	/* Match new owner object */
	if(!strcasecmp(new_owner_obj_name, "me"))
	    new_owner_obj_num = con_obj_num;
	else
	    new_owner_obj_num = MatchObjectByName(
		xsw_object, total_objects,
		new_owner_obj_name, SW_OBJ_TYPE_PLAYER
	    );
	if(DBIsObjectGarbage(new_owner_obj_num))
	{
	    sprintf(
		sndbuf,
		"%s: No such player.",
		new_owner_obj_name
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);

	    return(-1);
	}

	new_owner_obj = xsw_object[new_owner_obj_num];


	/* Check if target object's current owner is the same
	 * as the new owner
	 */
	if(tar_obj->owner == new_owner_obj_num)
	{
	    sprintf(
		sndbuf,
		"%s: %s: Already owned by %s.",
		THIS_CMD_NAME,
		tar_obj_name,
		new_owner_obj_name
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(0);
	}


	/* Check permissions, allowed to chown? */
	if(con_obj->permission.uid > SWS_ACCESS_UID_CHOWN)
	{
	    sprintf(
		sndbuf,
		"%s: Permission denied: Requires access level %i.",
		THIS_CMD_NAME,
		SWS_ACCESS_UID_CHOWN
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1); 
	}

	/* If the target object owned by someone else, then check
	 * connection object is allowed to chown objects belonging
	 * to someone else
	 */
	if((con_obj->permission.uid > SWS_ACCESS_UID_CHOWNO) &&
	   (tar_obj->owner > -1) &&
	   (tar_obj->owner != con_obj_num)
	)
	{
	    sprintf(
		sndbuf,
		"%s: Permission denied: Requires access level %i.",
		THIS_CMD_NAME,
		SWS_ACCESS_UID_CHOWNO
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}


	/* Chown object */
	tar_obj->owner = new_owner_obj_num;


	/* Get formal names of the objects */
	strncpy(
	    con_obj_name,
	    DBGetFormalNameStr(con_obj_num),
	    sizeof(con_obj_name)
	);
	con_obj_name[sizeof(con_obj_name) - 1] = '\0';

	strncpy(
	    tar_obj_name,
	    DBGetFormalNameStr(tar_obj_num),
	    sizeof(tar_obj_name)
	);
	tar_obj_name[sizeof(tar_obj_name) - 1] = '\0';

	strncpy(
	    new_owner_obj_name,
	    DBGetFormalNameStr(new_owner_obj_num),
	    sizeof(new_owner_obj_name)
	);
	new_owner_obj_name[sizeof(new_owner_obj_name) - 1] = '\0';


	/* Log and print response */
	sprintf(
	    sndbuf,
	    "Chowned %s to %s.",
	    tar_obj_name,
	    new_owner_obj_name
	);
	NetSendLiveMessage(condescriptor, sndbuf);

	sprintf(
	    text,
	    "%s: Chowned %s to %s",
	    con_obj_name, tar_obj_name, new_owner_obj_name
	);
	if(sysparm.log_general)
	    LogAppend(fname.log, text);

	return(0);
}
