#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../include/string.h"
#include "../include/imgrefs.h"
#include "../include/unvmatch.h"

#include "net.h"
#include "sysparm.h"
#include "swserv.h"


int CmdCreate(int condescriptor, const char *arg);


#define THIS_CMD_NAME           "create"


int CmdCreate(int condescriptor, const char *arg)
{
	int type, con_obj_num, new_obj_num;
	char	*s,
		new_name[SW_OBJ_NAME_MAX],
		new_type_name[256],
		name1[SW_OBJ_NAME_MAX + 80],
		name2[SW_OBJ_NAME_MAX + 80],
		text[(2 * SW_OBJ_NAME_MAX) + 512],
		sndbuf[CS_DATA_MAX_LEN];
	xsw_object_struct *con_obj, *new_obj;
	sws_connection_struct *con = connection[condescriptor];

	/* Get connection's object number (assumed valid) */
	con_obj_num = con->object_num;
	con_obj = xsw_object[con_obj_num];

	/* Print usage? */
	if((arg != NULL) ? (*arg == '\0') : 1)
	{
	    sprintf(
		sndbuf,
		"Usage: `%s <name>[=<type>]'",
		THIS_CMD_NAME
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}

	/* Check if object's permission allows create object */
	if(con_obj->permission.uid > SWS_ACCESS_UID_CREATE)
	{
	    sprintf(
		sndbuf,
		"%s: Permission denied: Requires access level %i.",
		THIS_CMD_NAME,
		SWS_ACCESS_UID_CREATE
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}

	/* Parse name */
	strncpy(new_name, arg, sizeof(new_name));
	new_name[sizeof(new_name) - 1] = '\0';
	s = strchr(new_name, '=');
	if(s != NULL)
	    *s = '\0';
	strstrip(new_name);

	/* Parse type */
	s = strchr(arg, '=');
	strncpy(
	    new_type_name,
	    (s != NULL) ? s + 1 : SW_OBJ_TYPE_NAME_STATIC,
	    sizeof(new_type_name)
	);
	new_type_name[sizeof(new_type_name) - 1] = '\0';
	strstrip(new_type_name);

	/* Is the specified name invalid? */
	if(DBValidateObjectName(new_name)) 
	{
	    sprintf(
		sndbuf,
		"%s: Invalid name.",
		new_name
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-2);
	}

	/* Get the type */
	if(!strcasecmp(new_type_name, SW_OBJ_TYPE_NAME_STATIC))
	    type = SW_OBJ_TYPE_STATIC;
	else if(!strcasecmp(new_type_name, SW_OBJ_TYPE_NAME_DYNAMIC))
	    type = SW_OBJ_TYPE_DYNAMIC;
	else if(!strcasecmp(new_type_name, SW_OBJ_TYPE_NAME_CONTROLLED))
	    type = SW_OBJ_TYPE_CONTROLLED;
	else if(!strcasecmp(new_type_name, SW_OBJ_TYPE_NAME_PLAYER))
	    type = SW_OBJ_TYPE_PLAYER;
	else if(!strcasecmp(new_type_name, SW_OBJ_TYPE_NAME_WEAPON))
	    type = SW_OBJ_TYPE_WEAPON;
	else if(!strcasecmp(new_type_name, SW_OBJ_TYPE_NAME_STREAMWEAPON))
	    type = SW_OBJ_TYPE_STREAMWEAPON;
	else if(!strcasecmp(new_type_name, SW_OBJ_TYPE_NAME_SPHEREWEAPON))
	    type = SW_OBJ_TYPE_SPHEREWEAPON;
	else if(!strcasecmp(new_type_name, SW_OBJ_TYPE_NAME_HOME))
	    type = SW_OBJ_TYPE_HOME;
	else if(!strcasecmp(new_type_name, SW_OBJ_TYPE_NAME_AREA))
	    type = SW_OBJ_TYPE_AREA;
	else if(!strcasecmp(new_type_name, SW_OBJ_TYPE_NAME_ANIMATED))
	    type = SW_OBJ_TYPE_ANIMATED;
	else if(!strcasecmp(new_type_name, SW_OBJ_TYPE_NAME_WORMHOLE))
	    type = SW_OBJ_TYPE_WORMHOLE;
	else if(!strcasecmp(new_type_name, SW_OBJ_TYPE_NAME_ELINK))
	    type = SW_OBJ_TYPE_ELINK;
	else
	    type = SW_OBJ_TYPE_GARBAGE;

	/* Make sure that the type is valid and non-garbage */
	if(type <= SW_OBJ_TYPE_GARBAGE)
	{
	    sprintf(
		sndbuf,
		"%s: Invalid object type `%s'.",
		THIS_CMD_NAME,
		new_type_name
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-2);   
	}

	/* Trying to create a player? */
	if(type == SW_OBJ_TYPE_PLAYER)
	{
	    sprintf(
		sndbuf,
		"%s: Unable to create players, use `createplayer' instead.",
 		THIS_CMD_NAME
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-2);
	}

	/* Create the new object */
	new_obj_num = DBCreateObject(
	    IMGREF_DEFAULT_IMGREF,	/* ImgRef */
	    type,			/* Type */
	    con_obj_num,		/* Owner */
	    con_obj->sect_x, con_obj->sect_y, con_obj->sect_z,
	    con_obj->x, con_obj->y, con_obj->z,
	    con_obj->heading, con_obj->pitch, con_obj->bank
	);
	if(DBIsObjectGarbage(new_obj_num))
	{
	    sprintf(
		sndbuf,
		"%s: DBCreateObject(): Unable to create object.",
		THIS_CMD_NAME
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}

	new_obj = xsw_object[new_obj_num];

	strncpy(
	    new_obj->name, new_name,
	    sizeof(new_obj->name)
	);
	new_obj->name[sizeof(new_obj->name) - 1] = '\0';


	/* New object has been created */

	/* Notify all connections about the new object */
	NetSendCreateObject(-1, new_obj_num);
	NetSendObjectName(-1, new_obj_num);

	/* Print and log */
	strncpy(
	    name1, DBGetFormalNameStr(new_obj_num),
	    sizeof(name1)
	);
	name1[sizeof(name1) - 1] = '\0';
	strncpy(
	    name2, DBGetFormalNameStr(con_obj_num),
	    sizeof(name2)
	);
	name2[sizeof(name2) - 1] = '\0';

	sprintf(
	    sndbuf,
	    "Created new object %s.",
	    name1
	);
	NetSendLiveMessage(condescriptor, sndbuf);

	sprintf(
	    text,
	    "%s: Created new object: %s",
	    name2,
	    name1
	);
	if(sysparm.log_events)
	    LogAppend(fname.log, text);

	return(0);
}
