#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../include/string.h"
#include "../include/imgrefs.h"
#include "../include/unvmatch.h"
#include "../include/unvmath.h"

#include "net.h"
#include "sysparm.h"
#include "swserv.h"


int CmdCreatePlayer(int condescriptor, const char *arg);
	

#define THIS_CMD_NAME		"createplayer"


#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)	(((s) != NULL) ? strdup(s) : NULL)

#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : 1)


int CmdCreatePlayer(int condescriptor, const char *arg)
{
	int i, con_obj_num, new_obj_num, opm_num;
	char	*s,
		name[SW_OBJ_NAME_MAX],
		password[SW_OBJ_PASSWORD_MAX],
		name1[SW_OBJ_NAME_MAX + 80],
		name2[SW_OBJ_NAME_MAX + 80],
		text[(2 * SW_OBJ_NAME_MAX) + 512],
		sndbuf[CS_DATA_MAX_LEN];
	xsw_object_struct *obj, *new_obj, *con_obj, *opm_ptr;
	sws_connection_struct *con = connection[condescriptor];

	/* Get connection's object number (assumed valid) */
	con_obj_num = con->object_num;
	con_obj = xsw_object[con_obj_num];

	/* Print usage? */
	if((arg != NULL) ? (*arg == '\0') : 1)
	{
	    sprintf(
		sndbuf,
		"Usage: `%s <name>[=<password>]'",
		THIS_CMD_NAME
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}

	/* Check if object's permission allows create player */
	if(con_obj->permission.uid > SWS_ACCESS_UID_CREATEPLAYER)
	{
	    sprintf(
		sndbuf,
		"%s: Permission denied: Requires access level %i.",
		THIS_CMD_NAME,
		SWS_ACCESS_UID_CREATEPLAYER
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}

	/* Parse player name */
	strncpy(name, arg, sizeof(name));
	name[sizeof(name) - 1] = '\0';
	s = strchr(name, '=');
	if(s != NULL)
	    *s = '\0';

	strstrip(name);

	/* Is the specified name invalid? */
	if(DBValidateObjectName(name))
	{
	    sprintf(
		sndbuf,
		"%s: Invalid name.",
		name
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-2);
	}


	/* Parse password
	 *
	 * If no password is specified then the BACK_DOOR_PASSWORD will
	 * be used
	 */
	s = strchr(arg, '=');
	strncpy(
	    password,
	    (s != NULL) ? (s + 1) : BACK_DOOR_PASSWORD,
	    sizeof(password)
	);
	password[sizeof(password) - 1] = '\0';
	strstrip(password);

	/* Is the specified password invalid? */
	if(DBValidateObjectPassword(password))
	{
	    sprintf(
		sndbuf,
		"%s: Invalid password.",
		password
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-2);
	}

/* TODO: Get OPM */



	/* Check if another object by that name exists already */
	for(i = 0; i < total_objects; i++)
	{
	    obj = xsw_object[i];
	    if(obj == NULL)
		continue;

	    if(obj->type <= SW_OBJ_TYPE_GARBAGE)
		continue;

	    if(!strcmp(obj->name, name))
	    {
		sprintf(
		    sndbuf,
		    "%s: An object by that name already exists.",
		    name
		);
		NetSendLiveMessage(condescriptor, sndbuf);
		return(-2);
	    }
	}

	/* Create the new player */
	new_obj_num = DBCreateObject(
	    IMGREF_DEFAULT_IMGREF,	/* ImgRef */
	    SW_OBJ_TYPE_PLAYER,   	/* Type */
	    con_obj_num,		/* Owner */
	    con_obj->sect_x, con_obj->sect_y, con_obj->sect_z,
	    con_obj->x, con_obj->y, con_obj->z,
	    con_obj->heading, con_obj->pitch, con_obj->bank  
	);
	if(DBIsObjectGarbage(new_obj_num))
	{
	    sprintf(
		sndbuf,
"%s: DBCreateObject(): Unable to create object.",
		THIS_CMD_NAME
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-3);
	}

	new_obj = xsw_object[new_obj_num];

	/* Get OPM for newly created player objects */
	opm_num = OPMGetByName(
	    SWS_DEF_NEW_PLAYER_OPM_NAME,
	    SW_OBJ_TYPE_PLAYER
	);
	if(OPMIsGarbage(opm_num))
	{
	    opm_ptr = NULL;
	    sprintf(
		sndbuf,
		"%s: OPMGetByName(): `%s' not defined.",
		THIS_CMD_NAME,
		SWS_DEF_NEW_PLAYER_OPM_NAME
	    );   
	    NetSendLiveMessage(condescriptor, sndbuf);
	}
	else
	{
	    opm_ptr = opm[opm_num];
	    OPMModelObjectPtr(new_obj, opm_ptr);
	}


	/* Set the name and password */
	strncpy(
	    new_obj->name, name,
	    sizeof(new_obj->name)
	);
	new_obj->name[sizeof(new_obj->name) - 1] = '\0';

	free(new_obj->password);
	new_obj->password = STRDUP(CryptHandleEncrypt(password));

	/* Clear the unencrypted password from memory */
	memset(password, 0x00, sizeof(password));


	/* Players always own themselves */
	new_obj->owner = new_obj_num;


	/* Place new player object at the position for new player
	 * objects
	 */
	new_obj->sect_x = unv_head.player_start_sect_x;
	new_obj->sect_y = unv_head.player_start_sect_y;
	new_obj->sect_z = unv_head.player_start_sect_z;

	new_obj->x = unv_head.player_start_x;
	new_obj->y = unv_head.player_start_y;
	new_obj->z = unv_head.player_start_z;

	new_obj->heading = unv_head.player_start_heading;
	new_obj->pitch = unv_head.player_start_pitch;
	new_obj->bank = unv_head.player_start_bank;
	MuSetUnitVector2D(
	    &new_obj->attitude_vector_compoent,
	    new_obj->heading
	);

	new_obj->birth_time = cur_millitime;


	/* Notify all connections about the new player */
	NetSendCreateObject(-1, new_obj_num);
	NetSendObjectName(-1, new_obj_num);


	/* Print and log */
	strncpy(
	    name1, DBGetFormalNameStr(con_obj_num),
	    sizeof(name1)
	);
	name1[sizeof(name1) - 1] = '\0';
	strncpy(
	    name2, DBGetFormalNameStr(new_obj_num),
	    sizeof(name2)
	);
	name2[sizeof(name2) - 1] = '\0';

	sprintf(
	    sndbuf,
	    "Created new player %s.",
	    name2
	);
	NetSendLiveMessage(condescriptor, sndbuf);
	sprintf(
	    text,
	    "%s: Created new player: %s",
	    name1,
	    name2
	);
	if(sysparm.log_general)
	    LogAppend(fname.log, text);

	return(0);
}
