#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "../include/unvmatch.h"

#include "net.h"
#include "sysparm.h"
#include "swserv.h"


int CmdDisk(int condescriptor, const char *arg);


#define THIS_CMD_NAME	"disk"

#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)	(((s) != NULL) ? strdup(s) : NULL)

#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : 1)


int CmdDisk(int condescriptor, const char *arg)
{
	int con_obj_num;
	struct stat stat_buf;
	xsw_object_struct *con_obj;
	sws_connection_struct *con = connection[condescriptor];

	/* Get connection's object number (assumed valid) */
	con_obj_num = con->object_num;
	con_obj = xsw_object[con_obj_num];

	/* Permission check */
	if(con_obj->permission.uid > SWS_ACCESS_UID_DISK)
	{
	    char *s = (char *)malloc(
		STRLEN(THIS_CMD_NAME) + 40 + 80
	    );
	    sprintf(
		s,
"%s: Permission denied: Requires access level %i.",
		THIS_CMD_NAME,
		SWS_ACCESS_UID_DISK
	    );
	    NetSendLiveMessage(condescriptor, s);
	    free(s);
	    return(-1);
	}

#if 0
/* Do not call disk stats update since it would cause process to halt
 * for a short time, that would leave this function too opened for
 * abuse
 */
	SWServGetOSStats();
#endif

	/* No argument? */
	if((arg != NULL) ? (arg[0] == '\0') : 1)
	{
	    /* Print program files and disk stats */
	    char *s;

	    /* Device hosting program */
	    s = (char *)malloc(40 + 40 + 40 + 80);
	    sprintf(
		s,
"Host device space: %ld kb used  %ld kb total  %.0f%% ratio",
		os_stat.disk_used,
		os_stat.disk_total,
		(os_stat.disk_total > 0l) ?
		    ((float)os_stat.disk_used /
		        (float)os_stat.disk_total * 100.0) : 0.0
	    );
	    NetSendLiveMessage(condescriptor, s);
	    free(s);

#if 0
	    /* Universe input file */
	    if(!stat(fname.unv_in, &stat_buf))
	    {
		s = (char *)malloc(
		    STRLEN(fname.unv_in) + 40 + 80
		);
		sprintf(
		    s,
"Universe in: %s  %ld bytes",
		    fname.unv_in,
		    stat_buf.st_size
		);
		NetSendLiveMessage(condescriptor, s);
		free(s);
	    }
#endif

	    /* Universe output file */
	    if(!stat(fname.unv_out, &stat_buf))
	    {
		s = (char *)malloc(
		    STRLEN(fname.unv_out) + 40 + 80
		);
		sprintf(
		    s,
"Universe out: %s  %ld bytes",
		    fname.unv_out,
		    stat_buf.st_size
		);
		NetSendLiveMessage(condescriptor, s);
		free(s);
	    }

	    /* Log */    
	    if(!stat(fname.log, &stat_buf))
	    {
		s = (char *)malloc(
		    STRLEN(fname.log) + 40 + 80
		);
		sprintf(
		    s,
"Primary log: %s  %ld bytes",
		    fname.log,
		    stat_buf.st_size
		);
		NetSendLiveMessage(condescriptor, s);
		free(s);
	    }

	    /* Events export log */
	    if(!stat(fname.events_export, &stat_buf))
	    {
		s = (char *)malloc(
		    STRLEN(fname.events_export) + 40 + 80
		);
		sprintf(
		    s,
"Events export log: %s  %ld bytes",
		    fname.events_export,
		    stat_buf.st_size
		);
		NetSendLiveMessage(condescriptor, s);
		free(s);
	    }

	    return(0);
	}

	/* Handle argument */


 
	return(0);
}

