#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../include/string.h"
#include "../include/unvmatch.h"
#include "../include/unvmath.h"

#include "net.h"
#include "sysparm.h"
#include "swserv.h"


int CmdETA(int condescriptor, const char *arg);


#define THIS_CMD_NAME	"eta"  


int CmdETA(int condescriptor, const char *arg)
{
	int con_obj_num, dest_obj_num;
	double distance, velocity, eta;
	char sndbuf[CS_DATA_MAX_LEN];
	xsw_object_struct *con_obj, *dest_obj;
	sws_connection_struct *con = connection[condescriptor];

	/* Get connection's object number (assumed valid) */
	con_obj_num = con->object_num;
	con_obj = xsw_object[con_obj_num];

	/* Get destination object */
	dest_obj_num = con_obj->intercepting_object;
	if(DBIsObjectGarbage(dest_obj_num))
	    return(0);

	dest_obj = xsw_object[dest_obj_num];

	/* Cannot calculate if velocity is 0 */
	velocity = con_obj->velocity;
	if(velocity <= 0.0)
	    return(0);

	/* Check if in same sector */   
	if(!Mu3DInSameSectorPtr(
	    con_obj, dest_obj
	))
	{
	    sprintf(
		sndbuf,
		"%s: Destination not in current sector.",
		THIS_CMD_NAME
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(0);
	}

	/* Get distance between objects (in XSW real units) */
	distance = Mu3DDistance(
	    dest_obj->x - con_obj->x,
	    dest_obj->y - con_obj->y,
	    dest_obj->z - con_obj->z
	);

	/* Calculate eta in units of `cycles' */
	eta = distance / velocity;

	/* Convert ETC in `cycles' to milliseconds */
	eta = eta * (1000 / SW_CYCLE_LAPSE_MS) / 1000;

	sprintf(
	    sndbuf,
	    "ETA to %s in %s",
	    dest_obj->name,
	    StringFormatTimePeriod((time_t)eta)
	);
	NetSendLiveMessage(condescriptor, sndbuf);

	return(0);
}
