#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../include/string.h"
#include "../include/unvmatch.h"
#include "../include/unvmath.h"

#include "net.h"
#include "sysparm.h"
#include "swserv.h"


int CmdExamine(int condescriptor, const char *arg);


#define THIS_CMD_NAME	"examine"


int CmdExamine(int condescriptor, const char *arg)
{
	int i, con_obj_num, obj_num, wep_num;
	char		*s, *s2,
			sndbuf[CS_DATA_MAX_LEN],
			name[SW_OBJ_NAME_MAX],
			name1[SW_OBJ_NAME_MAX + 80],
			name2[SW_OBJ_NAME_MAX + 80],
			lifespan_str[80];
	xsw_object_struct *obj, *con_obj;
	sws_connection_struct *con = connection[condescriptor];


	/* Get the connection's object (assumed valid) */
	con_obj_num = con->object_num;
	con_obj = xsw_object[con_obj_num];

#if 0
	/* If no argument, print usage */
	if((arg != NULL) ? (*arg == '\0') : 1)
	{
	    sprintf(
		sndbuf,
		"Usage: `%s <object>'",
		THIS_CMD_NAME
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}
#endif

	/* Check if allowed to examine? */
	if(con_obj->permission.uid > SWS_ACCESS_UID_EXAMINE)
	{
	    sprintf(
		sndbuf,
		"%s: Permission denied: Requires access level %i.",
		THIS_CMD_NAME,
		SWS_ACCESS_UID_EXAMINE
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}

	/* Parse object name from argument */
	strncpy(name, arg, sizeof(name));
	name[sizeof(name) - 1] = '\0';
	s = strchr(name, '=');
	if(s != NULL)
	    *s = '\0';   
	strstrip(name);

	/* Match object by name, first check for special cases */
	/* Locked on object */
	if(*name == '\0')
	{
	    if(DBIsObjectGarbage(con_obj->locked_on))
		obj_num = con_obj_num;
	    else
		obj_num = con_obj->locked_on;
	}
	/* Connection's object */
	else if(!strcasecmp(name, "me"))
	{
	    obj_num = con->object_num;
	}
	/* All else match by name */
	else
	{
	    obj_num = MatchObjectByName(
		xsw_object, total_objects,
		name, -1
	    );
	}

	/* Make sure that the specified object is valid */
	if(DBIsObjectGarbage(obj_num))
	{
	    sprintf(
		sndbuf,
		"%s: No such object.",
		name
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}

	obj = xsw_object[obj_num];


	/* Check if allowed to examine other? */
	if((obj->owner != con_obj_num) &&
	   (con_obj->permission.uid > SWS_ACCESS_UID_EXAMINEO)
	)
	{
	    if(DBIsObjectGarbage(obj->owner))
	    {
		NetSendLiveMessage(
		    condescriptor,
THIS_CMD_NAME ": *No owner*"
		);
		return(-1);
	    }
	    else
	    {
		xsw_object_struct *owner_obj = xsw_object[obj->owner];
		sprintf(
		    sndbuf,
"%s: Owner: %s",
		    THIS_CMD_NAME,
		    owner_obj->name
		);
		NetSendLiveMessage(condescriptor, sndbuf);
		return(-1);
	    }
	}
	/* *********************************************** */
	/* Print information */


	/* Line 1: Name, type, uid, and credits */
	strncpy(
	    name1, DBGetFormalNameStr(obj_num),
	    sizeof(name1)
	);
	name1[sizeof(name1) - 1] = '\0';
	strncpy(
	    name2, DBGetFormalNameStr(obj->owner),
	    sizeof(name2)
	);
	name2[sizeof(name2) - 1] = '\0';

	if(obj->lifespan > 0l)
	    sprintf(
		lifespan_str,
		"Lifespan: %ld ms (%ld ms)",
		(obj->birth_time + obj->lifespan) - cur_millitime,
		obj->lifespan
	    );
	else
	    *lifespan_str = '\0';

	sprintf(
	    sndbuf,
"%s  Owner: %s  Type: %s  UID: %i  Empire: %s  %s",
	    name1,
	    name2,
	    DBGetTypeName(obj->type),
	    obj->permission.uid,
	    obj->empire,
	    lifespan_str
	);
	NetSendLiveMessage(condescriptor, sndbuf);

	/* Line 1.25: (optional for Elink objects) */
	if(obj->type == SW_OBJ_TYPE_ELINK)
	{
	    sprintf(
		sndbuf,
"External Link: %s",
		((obj->elink == NULL) ? "*unlinked*" : obj->elink)
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	}

	/* Line 1.5: (optional for AI/Controlled) */
	if(obj->type == SW_OBJ_TYPE_CONTROLLED)
	{
	    sprintf(
		sndbuf,
 "AI Flags: %s%s%s%s%s%s",
		((obj->ai_flags & SW_AI_FOLLOW_FRIEND) ?
		    SW_AI_STR_FOLLOW_FRIEND " " : ""
		),
		((obj->ai_flags & SW_AI_FOLLOW_UNKNOWN) ?
		    SW_AI_STR_FOLLOW_UNKNOWN " " : ""
		),
		((obj->ai_flags & SW_AI_FOLLOW_HOSTILE) ?
		    SW_AI_STR_FOLLOW_HOSTILE " " : ""
		),
		((obj->ai_flags & SW_AI_FIRE_FRIEND) ?
		    SW_AI_STR_FIRE_FRIEND " " : ""
		),
		((obj->ai_flags & SW_AI_FIRE_UNKNOWN) ?
		    SW_AI_STR_FIRE_UNKNOWN " " : ""
		),
		((obj->ai_flags & SW_AI_FIRE_HOSTILE) ?
		    SW_AI_STR_FIRE_HOSTILE " " : ""
		)
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	}

	/* Line 2: Sector, position, location type, and size */
	switch(obj->loc_type)
	{
	  case SW_LOC_TYPE_NEBULA:
	    s = SW_LOC_TYPE_NAME_NEBULA;
	    break;
	  case SW_LOC_TYPE_SPACE_TERRITORY:
	    s = SW_LOC_TYPE_NAME_SPACE_TERRITORY;
	    break;
	  default:
	    s = SW_LOC_TYPE_NAME_SPACE;
	    break;
	}
	sprintf(
	    sndbuf,
"Sector: %ld %ld %ld  Position: %.4f %.4f %.4f  Location Type: %s\
  Size: %ld su",
	    obj->sect_x, obj->sect_y, obj->sect_z,
	    obj->x, obj->y, obj->z,
	    s,
	    obj->size
	);
	NetSendLiveMessage(condescriptor, sndbuf);

	/* Line 3: Attitude, velocity vector */
	sprintf(
	    sndbuf,
"Attitude: %.2f' %.2f' %.2f'  Velocity vector: %.2f' %.2f' %.2f'\
  Turn Rate: %.5f",
	    RADTODEG(obj->heading),
	    RADTODEG(obj->pitch),
	    RADTODEG(obj->bank),
	    RADTODEG(obj->velocity_heading),
	    0.0 * PI,
	    0.0 * PI,
	    obj->turnrate
	);
	NetSendLiveMessage(condescriptor, sndbuf);

	/* Line 4: Velocity, thrust power, throttle, engine state */
	sprintf(sndbuf,
"Velocity: %.5f(%.5f)  Thrust power: %.5f  Throttle: %.0f%%\
  Engine state: %i",
	    obj->velocity, obj->velocity_max,
	    obj->thrust_power,
	    obj->throttle * 100.0,
	    obj->engine_state
	);
	NetSendLiveMessage(condescriptor, sndbuf);

	/* Line 5: ImgRef, scanner range, and turn rate */
	strncpy(
	    name1, DBGetFormalNameStr(obj->locked_on),
	    sizeof(name1)
	);
	name1[sizeof(name1) - 1] = '\0';
	strncpy(
	    name2, DBGetFormalNameStr(obj->intercepting_object),
	    sizeof(name2)
	);
	name2[sizeof(name2) - 1] = '\0';

	sprintf(
	    sndbuf,
"ImgRef: %i  Scanner Range: %.2f ru  Locked on: %s  Intercept: %s",
	    obj->imageset,
	    obj->scanner_range,
	    name1,
	    name2
	);
	NetSendLiveMessage(condescriptor, sndbuf);
		
	/* Line 6: Hull, power, purity, core efficiency */
	sprintf(
	    sndbuf,
"Hull: %.2f(%.2f)  Power: %.2f(%.2f)  Purity: %.3f%%\
  Core efficency: %.4f",
	    obj->hp, obj->hp_max,
	    obj->power, obj->power_max,
	    obj->power_purity * 100.0,
	    obj->core_efficency
	);
	NetSendLiveMessage(condescriptor, sndbuf);
	    
	/* Line 7: Antimatter, shields, cloak and visibility */
	switch(obj->shield_state)
	{
	  case SHIELD_STATE_UP:
	    s = "Up";
	    break;
	  case SHIELD_STATE_DOWN:
	    s = "Down";
	    break;
	  default:
	    s = "None";
	    break;
	}
	switch(obj->cloak_state)
	{
	  case CLOAK_STATE_UP:
	    s2 = "Up";
	    break;
	  case CLOAK_STATE_DOWN:
	    s2 = "Down";
	    break;
	  default:
	    s2 = "None";
	    break;
	}
	sprintf(sndbuf,
"AM: %.2f(%.2f)  Shields: %s  Cloak: %s  Cloak Strength: %.0f%%\
  Vis: %.0f%%",
	    obj->antimatter,
	    obj->antimatter_max,
	    s,
	    s2,
	    obj->cloak_strength * 100.0,
	    obj->visibility * 100.0
	);
	NetSendLiveMessage(condescriptor, sndbuf);
	    
	/* Print weapons */
	if(obj->weapons != NULL)
	{
	    xsw_weapon_struct *wep;

	    for(wep_num = 0; wep_num < obj->total_weapons; wep_num++)
	    {
		wep = obj->weapons[wep_num];
		if(wep == NULL)
		    continue;

		sprintf(
		    sndbuf,
"Wep %i [%s%s%s%s%s]:  OCS: %i  EmtType: %i  Amt: %i(%i)  Pwr: %.2f\
  Rng: %.2f  CPwr: %.2f  Delay: %ld ms  FireSndRef: %i",
		    wep_num,
                (wep->flags & SW_WEP_FLAG_NO_LOCK) ? "L" : "",
		(wep->flags & SW_WEP_FLAG_FIRE_CLOAKED) ? "C" : "",
		(wep->flags & SW_WEP_FLAG_NO_FIRE_SOUND) ? "S" : "",
		(wep->flags & SW_WEP_FLAG_FIXED) ? "F" : "",
                (wep->flags & SW_WEP_FLAG_PENETRATE_SHIELD) ? "P" : "",
		    wep->ocs_code,
		    wep->emission_type,
		    wep->amount,
		    wep->max,
		    wep->power,

		    (double)wep->range / 1000.0,
		    wep->create_power,
		    wep->delay,
		    wep->fire_sound_code
		);
		NetSendLiveMessage(condescriptor, sndbuf);
	    }
	}
 
	/* Scores */
	if(obj->score != NULL)  
	{
	    xsw_score_struct *score = obj->score;
	    sprintf(
		sndbuf,
"Credits: %.2f  RMU: %.2f(%.2f)  DmgGiv: %.2f  DmgRec: %.2f  Kills: %i",
		score->credits,
		score->rmu,
		score->rmu_max,
		score->damage_given,
		score->damage_recieved,
		score->kills
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	}

	/* Economy */
	if(obj->eco != NULL)
	{
	    xsw_ecodata_struct *eco = obj->eco;
	    xsw_ecoproduct_struct *product;

	    sprintf(
		sndbuf,
"Eco [%s%s%s%s%s]: TaxG: %.0f%%  TaxF: %.0f%%  TaxH: %.0f%%",
		((eco->flags & ECO_FLAG_OPEN) ? "O" : ""),
		((eco->flags & ECO_FLAG_BUY_OK) ? "B" : ""),
		((eco->flags & ECO_FLAG_SELL_OK) ? "S" : ""),
		((eco->flags & ECO_FLAG_TRADE_OK) ? "T" : ""),
		((eco->flags & ECO_FLAG_INTRODUCE_OK) ? "I" : ""),
		(eco->tax_general - 1) * 100,
		(eco->tax_friend - 1) * 100,
		(eco->tax_hostile - 1) * 100
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);

	    for(i = 0; i < eco->total_products; i++)
	    {
		product = eco->product[i];
		if(product == NULL)
		    continue;

		if(product->amount_max > 0.0)
		    sprintf(
			sndbuf,
"EcoProd `%s':  Buy: %.2f  Sell: %.2f  Amt: %.2f(%.2f)",
			product->name,
			product->buy_price,
			product->sell_price,
			product->amount,
			product->amount_max
		    );
		else
		    sprintf(
			sndbuf,
"EcoProd `%s':  Buy: %.2f  Sell: %.2f  Amt: *unlimited*",
			product->name,
			product->buy_price,
			product->sell_price
		    );
		NetSendLiveMessage(condescriptor, sndbuf);
	    }
	}

	/* Description */
	if((obj->description != NULL) ? (*obj->description != '\0') : 0)
	{
	    char desc[CS_DATA_MAX_LEN - 20];

	    strncpy(desc, obj->description, sizeof(desc));
	    desc[sizeof(desc) - 1] = '\0';

	    sprintf(
		sndbuf,
"Description: %s",
		desc
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	}


	return(0);
}
