#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fnmatch.h>

#include "../include/string.h"
#include "../include/unvmatch.h"

#include "net.h"
#include "sysparm.h"
#include "swserv.h"


static int CmdLastonMatch(const char *s, const char *name);
int CmdLaston(int condescriptor, const char *arg);


#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)	(((s) != NULL) ? strdup(s) : NULL)

#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : 1)


#define THIS_CMD_NAME	"laston"


static int CmdLastonMatch(const char *s, const char *name)
{
	int status;
	char	*s2 = STRDUP(s),
		*name2 = STRDUP(name);

	if((s2 == NULL) || (name2 == NULL))
	{
	    free(s2);
	    free(name2);
	    return(0);
	}

	strtoupper(s2);
	strtoupper(name2);

	status = fnmatch(s2, name2, 0);

	free(s2);
	free(name2);

	return(!status);
}

/*
 *	Get the last time a player has been connected.
 */
int CmdLaston(int condescriptor, const char *arg)
{
	int i, con_obj_num, nobj_matched;
	const char *name;
	xsw_object_struct *obj, *con_obj;
	sws_connection_struct *con = connection[condescriptor];

	con_obj_num = con->object_num;
	con_obj = xsw_object[con_obj_num];

	/* Print usage? */ 
	if((arg != NULL) ? (*arg == '\0') : 1)
	{
	    char *s = (char *)malloc(STRLEN(THIS_CMD_NAME) + 80);
	    sprintf(
		s,
"Usage: `%s <player>'",
		THIS_CMD_NAME
	    );
	    NetSendLiveMessage(condescriptor, s);
	    free(s);
	    return(-1);
	}

	/* Get name */
	name = arg;

	/* No permission check needed */

	/* Iterate through the objects list */
	nobj_matched = 0;
	for(i = 0; i < total_objects; i++)
	{
	    obj = xsw_object[i];
	    if(obj == NULL)
		continue;

	    if(obj->type != SW_OBJ_TYPE_PLAYER)
		continue;

	    /* This object's name matches the specified name? */
	    if(CmdLastonMatch(name, obj->name))
	    {
		/* Already connected? */
		if(ConGetByObject(i) > -1)
		{
		    char *s = (char *)malloc(STRLEN(obj->name) + 1024);
		    *s = '\0';
		    strcat(s, obj->name);
		    strcat(s, "                              ");
		    s[24] = '\0';
		    strcat(s, "Online");
		    NetSendLiveMessage(condescriptor, s);
		    free(s);
		}
		else
		{
		    const char *ts = (obj->last_connected > 0l) ?
			StringFormatTimePeriod(
			    cur_systime - obj->last_connected
			) : NULL;
                    char *s = (char *)malloc(STRLEN(obj->name) + STRLEN(ts) + 1024);
                    *s = '\0';
                    strcat(s, obj->name);
                    strcat(s, "                              ");
                    s[24] = '\0';                            
		    if(ts != NULL)
		    {
			strcat(s, ts);
			strcat(s, " ago");
		    }
		    else
		    {
			strcat(s, "Never connected");
		    }
                    NetSendLiveMessage(condescriptor, s);
                    free(s);
		}
		nobj_matched++;
	    }
	}

	/* No objects matched? */
	if(nobj_matched == 0)
	{
	    char *s = (char *)malloc(STRLEN(name) + 80);
	    sprintf(
		s,
"%s: No such player.",
		name
	    );
	    NetSendLiveMessage(condescriptor, s);
	    free(s);
	    return(-1);
	}

	return(0);
}
