#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef PLUGIN_SUPPORT
# include "plugins.h"
#endif
#include "net.h"
#include "sysparm.h"
#include "swserv.h"


int CmdMemory(int condescriptor, const char *arg);


#define THIS_CMD_NAME	"memory"
  
 
int CmdMemory(int condescriptor, const char *arg)
{
	int con_obj_num;
	long total;
	char sndbuf[CS_DATA_MAX_LEN];
	xsw_object_struct *con_obj;
	sws_connection_struct *con = connection[condescriptor];
	sws_memory_stats_struct m;

	/* Get connection's object number (assumed valid) */
	con_obj_num = con->object_num;
	con_obj = xsw_object[con_obj_num];

	/* Check if allowed to view memory */
	if(con_obj->permission.uid > SWS_ACCESS_UID_MEM)
	{
	    sprintf(
		sndbuf,
		"%s: Permission denied: Requires access level %i.",
		THIS_CMD_NAME,
		SWS_ACCESS_UID_MEM
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}


	/* Calculate memory being used */
	SWServGetMemoryStats(&m);


	/* Print memory stats */
	if((arg == NULL) ? 1 : (*arg == '\0'))
	{
	    sprintf(
		sndbuf,
"-        total       con       obj   rec_obj       opm       ocs   plugins"
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);

	    sprintf(
		sndbuf,
"Mem:\
 %9ld\
 %9ld\
 %9ld\
 %9ld\
 %9ld\
 %9ld\
 %9ld",
		m.total,
		m.con,
		m.obj,
		m.rec_obj,
		m.opm,
		m.ocs,
		m.plugins
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);

	    total = total_connections + total_objects +
		total_recycled_objects + total_opms + total_ocss
#ifdef PLUGIN_SUPPORT
		+ total_plugins
#endif	/* PLUGIN_SUPPORT */
	    ;

	    sprintf(
		sndbuf,
"Ent:\
 %9ld\
 %9d\
 %9d\
 %9d\
 %9d\
 %9d\
 %9d",
		total,

		total_connections,
		total_objects,
		total_recycled_objects,
		total_opms,
		total_ocss,
#ifdef PLUGIN_SUPPORT
		total_plugins
#else
		0
#endif
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);


	    /* Print usage at end */ 
	    sprintf(
		sndbuf,
"Usage: `%s [refresh|reload|reload_universe]'",
		THIS_CMD_NAME
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
 

	    return(0);
	}

 
	/* ********************************************************** */

	/* Reload memory */
	if(!strcasecmp(arg, "reload"))
	{
	    /* Permission check */
	    if(con_obj->permission.uid > SWS_ACCESS_UID_MEMOP)
	    {
		sprintf(
		    sndbuf,
		    "%s: Permission denied: Requires access level %i.",
		    THIS_CMD_NAME,
		    SWS_ACCESS_UID_MEMOP
		);
		NetSendLiveMessage(condescriptor, sndbuf);
		return(-1);
	    }

	    /* Set master reset to 2 so that the configuration will
	     * be reloaded.  The universe database file will NOT be
	     * reloaded.
	     */
	    master_reset = 2;
	}
	/* Refresh objects in memory */
	else if(!strcasecmp(arg, "refresh"))
	{
	    /* Permission check */
	    if(con_obj->permission.uid > SWS_ACCESS_UID_MEMOP)
	    {
		sprintf(
		    sndbuf,
		    "%s: Permission denied: Requires access level %i.",
		    THIS_CMD_NAME,
		    SWS_ACCESS_UID_MEMOP
		);
		NetSendLiveMessage(condescriptor, sndbuf);
		return(-1);
	    }

	    /* Set master reset to 1 so that the memory will be
	     * refreshed.
	     */
	    master_reset = 1;
	}
	/* Reload input universe file */
	else if(!strcasecmp(arg, "reload_universe"))
	{
	    /* Permission check */
	    if(con_obj->permission.uid > SWS_ACCESS_UID_MEMRELOADUNV)
	    {
		sprintf(
		    sndbuf,
		    "%s: Permission denied: Requires access level %i.",
		    THIS_CMD_NAME,
		    SWS_ACCESS_UID_MEMRELOADUNV
		);
		NetSendLiveMessage(condescriptor, sndbuf);
		return(-1);
	    }

	    /* Set master reset to 3 so that the universe database
	     * will be reloaded from the universe in file.
	     */
	    master_reset = 3;
	}
	/* Unsupported operation */
	else
	{
	    sprintf(
		sndbuf,
		"%s: %s: Unsupported operation.",
		THIS_CMD_NAME, arg
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	}


	return(0);
}
