#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../include/string.h"
#include "../include/unvmatch.h"

#include "net.h"
#include "sysparm.h"
#include "swserv.h" 


static void CmdNetstatPrintStandard(int condescriptor);
static void CmdNetstatPrintListening(int condescriptor);
static void CmdNetstatPrintPlayer(int condescriptor, int object_num);


#define THIS_CMD_NAME	"netstat"

#define SPACES		"                                 "


static void CmdNetstatPrintStandard(int condescriptor)
{
	int i;
	char	*s1, *s2,
		num_str[40],
		sndbuf[CS_DATA_MAX_LEN + SW_OBJ_NAME_MAX +
			SWS_HOST_NAME_MAX + 1024];
	sws_connection_struct *con;
	sws_aux_connection_struct *aux_con;

	/* Heading */
	NetSendLiveMessage(
	    condescriptor,
"Player                  \
Address                 \
RX          \
SX          \
ErrRX     \
ErrSX     \
State"
	);

	/* Iterate through connections list */
	for(i = 0; i < total_connections; i++)
	{
	    con = connection[i];
	    if(con == NULL)
		continue;

	    if(con->socket < 0)
		continue;

	    /* Player */
            s1 = sndbuf + 0;
            *s1 = '\0';
            if(DBIsObjectGarbage(con->object_num))
		s2 = "*Unknown*";
	    else
                s2 = DBGetFormalNameStr(con->object_num);
            strcat(s1, s2);
            strcat(s1, SPACES);

            /* Address */
            s1 = sndbuf + 24;
            *s1 = '\0';
            strcat(s1, con->conhost);
            strcat(s1, SPACES);

            /* RX (relative to client) */
            s1 = sndbuf + 48;
            *s1 = '\0';
            sprintf(num_str, "%ld", con->bytes_sent);
            strcat(s1, num_str);
            strcat(s1, SPACES);

            /* SX (relative to client) */
            s1 = sndbuf + 60;
            *s1 = '\0';
            sprintf(num_str, "%ld", con->bytes_received);
            strcat(s1, num_str);
            strcat(s1, SPACES);

	    /* ErrRX (relative to client) */
            s1 = sndbuf + 72;
            *s1 = '\0';
            sprintf(num_str, "%ld", con->errors_sent);
            strcat(s1, num_str);
            strcat(s1, SPACES);

	    /* ErrSX (relative to client) */
            s1 = sndbuf + 82;
            *s1 = '\0';
            sprintf(num_str, "%ld", con->errors_received);
            strcat(s1, num_str);
            strcat(s1, SPACES);

            /* State */
            s1 = sndbuf + 92;
            *s1 = '\0';
            if(con->object_num < 0)
                s2 = "NEGOTIATING";
            else
                s2 = "ESTABLISHED";
            strcat(s1, s2);

#if 0
	    switch(con->client_type)
	    {
	      default:
		s2 = "std";
		break;
	    }
	    strcat(s1, s2);
	    strcat(s1, SPACES);
#endif

	    NetSendLiveMessage(condescriptor, sndbuf);
	}

        /* Heading */
        NetSendLiveMessage(
            condescriptor,
"AUX Address             \
RX          \
SX          \
ErrRX     \
ErrSX     \
State"
        );

	/* Iterate through aux connections list */
	for(i = 0; i < total_aux_connections; i++)
	{
	    aux_con = aux_connection[i];
	    if(aux_con == NULL)
		continue;

	    if(aux_con->socket < 0)
		continue;

            /* Foreign Address */
            s1 = sndbuf + 0;
            *s1 = '\0';
            strcat(s1, aux_con->conhost);
            strcat(s1, SPACES);

            /* RX (relative to client) */
            s1 = sndbuf + 24;
            *s1 = '\0';
            sprintf(num_str, "%ld", aux_con->bytes_sent);
            strcat(s1, num_str);
            strcat(s1, SPACES);

            /* SX (relative to client) */
            s1 = sndbuf + 36;
            *s1 = '\0';
            sprintf(num_str, "%ld", aux_con->bytes_received);
            strcat(s1, num_str);
            strcat(s1, SPACES);

            /* ErrRX (relative to client) */
            s1 = sndbuf + 48;
            *s1 = '\0';
            sprintf(num_str, "%ld", aux_con->errors_sent);
            strcat(s1, num_str);
            strcat(s1, SPACES);

            /* ErrSX (relative to client) */
            s1 = sndbuf + 58;
            *s1 = '\0';
            sprintf(num_str, "%ld", aux_con->errors_received);
            strcat(s1, num_str);
            strcat(s1, SPACES);

	    /* State */
	    s1 = sndbuf + 68;
	    *s1 = '\0';
	    if(aux_con->logged_in)
		s2 = "AUTHENTICATED";
	    else
		s2 = "ANONYMOUS";
	    strcat(s1, s2);

	    NetSendLiveMessage(condescriptor, sndbuf);
	}
}


static void CmdNetstatPrintListening(int condescriptor)
{
	int i;
	char	*s1, *s2,
		num_str[80],
		sndbuf[CS_DATA_MAX_LEN + SW_OBJ_NAME_MAX + SWS_HOST_NAME_MAX];
	incoming_socket_struct *insock;

	/* Heading */
	NetSendLiveMessage(condescriptor,
"Type Port     Int"
	);

	/* Print each incoming socket */
	for(i = 0; i < total_incoming_sockets; i++)
	{
	    insock = incoming_socket[i];
	    if(insock == NULL)
		continue;

	    if(insock->socket < 0)
		continue;

	    /* Type */
	    s1 = &sndbuf[0];
	    *s1 = '\0';
	    switch(insock->type)
	    {
	      case INCOMING_SOCKET_TYPE_AUXSTATS:
		s2 = "aux";
		break;

	      default:	/* INCOMING_SOCKET_TYPE_STANDARD */
		s2 = "std";
		break;
	    }
	    strcat(s1, s2);
	    strcat(s1, SPACES);

	    /* Port */
	    s1 = &sndbuf[5];
	    *s1 = '\0';
	    sprintf(num_str, "%i", insock->port_num);
	    strcat(s1, num_str);
	    strcat(s1, SPACES);

	    /* Interval */
	    s1 = &sndbuf[14];
	    *s1 = '\0';
	    sprintf(num_str, "%ld", sysparm.int_new_connection_poll);
	    strcat(s1, num_str);
	    strcat(s1, SPACES);

	    s1 = &sndbuf[30];
	    *s1 = '\0';

	    NetSendLiveMessage(condescriptor, sndbuf);
	}
}

static void CmdNetstatPrintPlayer(int condescriptor, int object_num)
{
	int i, printed;
	char	*s1, *s2,
		num_str[80],
		sndbuf[CS_DATA_MAX_LEN + SW_OBJ_NAME_MAX + SWS_HOST_NAME_MAX];
	sws_connection_struct *con;

	/* Heading */
	NetSendLiveMessage(
	    condescriptor,
"Client RX           SX           Foreign Address                 State\
	  Player"
	);

	/* Get object's connection */
	printed = 0;
	for(i = 0; i < total_connections; i++)
	{
	    con = connection[i];
	    if(con == NULL)
		continue;

	    if(con->socket < 0)
		continue;

	    if(con->object_num != object_num)
		continue;

	    /* Client type */
	    s1 = &sndbuf[0];
	    *s1 = '\0';
	    switch(con->client_type)
	    {
	      default:
		s2 = "std";
		break;   
	    }
	    strcat(s1, s2);
	    strcat(s1, SPACES);

	    /* Recieved (relative to client) */
	    s1 = &sndbuf[7];
	    *s1 = '\0';
	    sprintf(num_str, "%ld", con->bytes_sent);
	    strcat(s1, num_str);
	    strcat(s1, SPACES);

	    /* Sent (relative to client) */
	    s1 = &sndbuf[20];
	    *s1 = '\0';
	    sprintf(num_str, "%ld", con->bytes_received);
	    strcat(s1, num_str);
	    strcat(s1, SPACES);

	    /* Foreign Address */
	    s1 = &sndbuf[33];
	    *s1 = '\0';
	    strcat(s1, con->conhost);
	    strcat(s1, SPACES);

	    /* State */
	    s1 = &sndbuf[65];
	    *s1 = '\0';
	    if(con->object_num < 0)
		s2 = "NEGOTIATING";
	    else
		s2 = "ESTABLISHED";
	    strcat(s1, s2);
	    strcat(s1, SPACES);

	    /* Player (assumed valid) */
	    s1 = &sndbuf[80];
	    *s1 = '\0';
	    s2 = DBGetFormalNameStr(object_num);
	    strcat(s1, s2);
	    strcat(s1, SPACES);

	    s1 = &sndbuf[130];
	    *s1 = '\0';

	    NetSendLiveMessage(condescriptor, sndbuf);

	    printed++;
	}

	/* Nothing printed? */
	if(!printed)
	    NetSendLiveMessage(condescriptor, "*** Not connected. ***");
}


/*
 *	Print network statistics.
 */
int CmdNetstat(int condescriptor, const char *arg)
{
	int con_obj_num, obj_num;
	char	name[SW_OBJ_NAME_MAX],
		larg[CS_DATA_MAX_LEN],
		sndbuf[CS_DATA_MAX_LEN + SW_OBJ_NAME_MAX];
	xsw_object_struct *con_obj;
	sws_connection_struct *con = connection[condescriptor];

	/* Get the connection's object (assumed valid) */
	con_obj_num = con->object_num;
	con_obj = xsw_object[con_obj_num];

	/* Check if object's permission allows save */
	if(con_obj->permission.uid > SWS_ACCESS_UID_NETSTAT)
	{
	    sprintf(
		sndbuf, 
		"%s: Permission denied: Requires access level %i.",
		THIS_CMD_NAME, SWS_ACCESS_UID_NETSTAT
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}

	/* Argument given? */
	if((arg != NULL) ? (*arg == '\0') : 1)
	{
	    /* No argument, print standard */        
	    CmdNetstatPrintStandard(condescriptor);

	    /* Print usage at end */
	    sprintf(
		sndbuf,
		"Usage: `%s [player] [-l] [-c]'",
		THIS_CMD_NAME
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	}
	else
	{
	    strncpy(larg, arg, CS_DATA_MAX_LEN);
	    strstrip(larg);

	    if(larg[0] == '-')
	    {
		if(larg[1] == 'c')
		{
		    /* Print connections */
		    CmdNetstatPrintStandard(condescriptor);
		}
		else if(larg[1] == 'l')
		{
		    /* Print listening sockets */
		    CmdNetstatPrintListening(condescriptor);
		}
		else
		{
		    /* Print usage */
		    sprintf(
			sndbuf,
			"Usage: `%s [player] [-l] [-c]'",
			THIS_CMD_NAME
		    );
		    NetSendLiveMessage(condescriptor, sndbuf);
		}
	    }
	    else
	    {
		/* Print netstats of player */

	        /* Get name of object */
		strncpy(name, larg, SW_OBJ_NAME_MAX);
		name[SW_OBJ_NAME_MAX - 1] = '\0';

		if(!strcasecmp(name, "me"))
		    obj_num = con->object_num;
		else
		    obj_num = MatchObjectByName(
		        xsw_object, total_objects,
			name, SW_OBJ_TYPE_PLAYER
		    );
		if(DBIsObjectGarbage(obj_num))
		{
		    sprintf(sndbuf,
			"%s: No such player.",
			name
		    );
		    NetSendLiveMessage(condescriptor, sndbuf);
		}
		else
		{
		    CmdNetstatPrintPlayer(condescriptor, obj_num);
		}
	    }
	}


	return(0);
}
