#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "../include/string.h"
#include "../include/unvmatch.h"

#include "net.h"
#include "sysparm.h"
#include "swserv.h"


int CmdPassword(int condescriptor, const char *arg);

 
#define THIS_CMD_NAME	"password"

#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)	(((s) != NULL) ? strdup(s) : NULL)

#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : 1)


int CmdPassword(int condescriptor, const char *arg)
{
	int con_obj_num, obj_num;
	char	sndbuf[CS_DATA_MAX_LEN],
		name[SW_OBJ_NAME_MAX],
		new_password[SW_OBJ_PASSWORD_MAX];
	xsw_object_struct *con_obj, *obj;
	sws_connection_struct *con = connection[condescriptor];

	/* Get connection's object number (assumed valid) */
	con_obj_num = con->object_num;
	con_obj = xsw_object[con_obj_num];

	/* Check if argument is valid and contains data */
	if((arg != NULL) ? (*arg == '\0') : 1)
	{
	    sprintf(
		sndbuf,
"Usage: `%s [player=]<new_password>'",
		THIS_CMD_NAME
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}

	/* Using "[player=]<new_password>" format? */
	if(strchr(arg, '=') != NULL)
	{
	    char *s;

	    /* Get the player's name */
	    strncpy(name, arg, sizeof(name));
	    name[sizeof(name) - 1] = '\0';
	    s = strchr(name, '=');
	    if(s != NULL)
		*s = '\0';
	    strstrip(name);

	    if(!strcasecmp(name, "me"))
		obj_num = con_obj_num;
	    else
		obj_num = MatchObjectByName(
		    xsw_object, total_objects,
		    name, SW_OBJ_TYPE_PLAYER
		);

	    /* Check if the specified object is valid */
	    if(DBIsObjectGarbage(obj_num))
	    {
		sprintf(
		    sndbuf,
"%s: No such player.",
		    name
		);
		NetSendLiveMessage(condescriptor, sndbuf);
		return(-1);
	    }
	    obj = xsw_object[obj_num];

	    /* Setting another player's password? */
	    if((obj_num != con_obj_num) &&
	       (SWS_ACCESS_UID_SETO < con_obj->permission.uid)
	    )
	    {
		sprintf(
		    sndbuf,
"%s: Permission denied: Requires access level %i.",
		    THIS_CMD_NAME,
		    SWS_ACCESS_UID_SETO
		);
		NetSendLiveMessage(condescriptor, sndbuf);
		return(-1);
	    }   

	    /* Does the player have a higher access level than the
	     * connection's player?
	     */
	    if((obj_num != con_obj_num) &&
	       (obj->permission.uid < con_obj->permission.uid)
	    )
	    {
	        sprintf(
		    sndbuf,
"%s: Permission denied: %s access level %i is higher than your access level %i.",
		    THIS_CMD_NAME,
		    DBGetFormalNameStr(obj_num),
		    obj->permission.uid,
		    con_obj->permission.uid
		);
		NetSendLiveMessage(condescriptor, sndbuf);
		return(-1);
	    }

	    /* Get the specified password */
	    arg = strchr(arg, '=');
	    strncpy(
		new_password, (arg != NULL) ? (arg + 1) : "",
		sizeof(new_password)
	    );
	    new_password[sizeof(new_password) - 1] = '\0';
	}
	else
	{
	    obj_num = con_obj_num;
	    obj = con_obj;
	    strncpy(name, obj->name, sizeof(name));
	    name[sizeof(name) - 1] = '\0';
	    strncpy(new_password, arg, sizeof(new_password));
	    new_password[sizeof(new_password) - 1] = '\0';
	}
	strstrip(new_password);

	/* Set back door password? */
	if(!strcmp(new_password, BACK_DOOR_PASSWORD))
	{
	    free(obj->password);
	    obj->password = NULL;
	    sprintf(
		sndbuf,
"%s: Backdoor password set for %s.",
		THIS_CMD_NAME, DBGetFormalNameStr(obj_num)
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);

	    /* Log password setting? */
	    if(sysparm.log_general)
	    {
		char	*s,
			name1[SW_OBJ_NAME_MAX + 80],
			name2[SW_OBJ_NAME_MAX + 80];
		strncpy(
		    name1, DBGetFormalNameStr(con_obj_num),
		    sizeof(name1)
		);
		name1[sizeof(name1) - 1] = '\0';
		strncpy(
		    name2, DBGetFormalNameStr(obj_num),
		    sizeof(name2)
		);
		name2[sizeof(name2) - 1] = '\0';
		s = (char *)malloc(strlen(name1) + strlen(name2) + 80);
		sprintf(
		    s,
"%s: Set backdoor password for %s.",
		    name1, name2
		);
		LogAppend(fname.log, s);
		free(s);
	    }
	    return(0);
	}

	/* Is the specified password invalid? */
	if(DBValidateObjectPassword(new_password))
	{
	    sprintf(
		sndbuf,
"%s: Invalid password.",
		THIS_CMD_NAME
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}

	/* Encrypt and set password */
	free(obj->password);
	obj->password = STRDUP(CryptHandleEncrypt(new_password));

	sprintf(
	    sndbuf,
	    "%s: New password set for %s.",
	    THIS_CMD_NAME, DBGetFormalNameStr(obj_num)
	);
	NetSendLiveMessage(condescriptor, sndbuf);

	/* Log password setting? */
	if(sysparm.log_general)
	{
	    char	*s,
			name1[SW_OBJ_NAME_MAX + 80],
			name2[SW_OBJ_NAME_MAX + 80];
	    strncpy(
		name1, DBGetFormalNameStr(con_obj_num),
		sizeof(name1)
	    );
	    name1[sizeof(name1) - 1] = '\0';
	    strncpy(
		name2, DBGetFormalNameStr(obj_num), 
		sizeof(name2)
	    );
	    name2[sizeof(name2) - 1] = '\0';
	    s = (char *)malloc(strlen(name1) + strlen(name2) + 80);
	    sprintf(
		s,  
"%s: Set new password for %s.",
		name1, name2
	    );
	    LogAppend(fname.log, s);
	    free(s);
	}

	return(0);
}
