#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "../include/string.h"
#include "../include/strexp.h"
#include "../include/unvmatch.h"
#include "../include/unvmath.h"
#include "../include/unvutil.h"

#include "net.h"
#include "sysparm.h"
#include "swserv.h"


#define THIS_CMD_NAME	"set"


static int ALLOW_SET_MAC(int condescriptor, int uid);
static int CmdOPMSet(int condescriptor, const char *arg);
static int CmdDoSet(int condescriptor, const char *arg);
int CmdSet(int condescriptor, const char *arg);


#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)	(((s) != NULL) ? strdup(s) : NULL)

#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : 1)


/*
 *	Macro to check if set is allowed by the given uid.
 */
static int ALLOW_SET_MAC(int condescriptor, int uid)
{
	char sndbuf[CS_DATA_MAX_LEN]; 

	if(uid > SWS_ACCESS_UID_SET)
	{
	    sprintf(
		sndbuf,
		"%s: Permission denied: Requires access level %i.",
		THIS_CMD_NAME,
		SWS_ACCESS_UID_SET
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(0);
	}
	else
	{
	    return(1);
	}
}



/*
 *	Remodel object to the values specified in an OPM.
 *
 *	Argument is of the format "<object>=$<opm>"
 */
static int CmdOPMSet(int condescriptor, const char *arg)
{
	char *s;
	int con_obj_num, obj_num, opm_num;
	xsw_object_struct *con_obj, *obj, *opm_ptr, *tmp_obj_buf;
	char *password;

	char	sndbuf[CS_DATA_MAX_LEN],
		opm_name[SW_OBJ_NAME_MAX],
		obj_name[SW_OBJ_NAME_MAX],
		tmp_name1[SW_OBJ_NAME_MAX + 80],
		tmp_name2[SW_OBJ_NAME_MAX + 80],
		text[(SW_OBJ_NAME_MAX * 2) + 256];

	/* Get the connection's object (assumed valid) */
	sws_connection_struct *con = connection[condescriptor];
	con_obj_num = con->object_num;
	con_obj = xsw_object[con_obj_num];


	/* Begin parsing */
	s = strchr(arg, '=');
	if(s == NULL)
	{


	    return(-1);
	}
	else
	{
	    /* Get OPM name */
	    strncpy(opm_name, s + 1, sizeof(opm_name));
	    opm_name[sizeof(opm_name) - 1] = '\0';
	    /* Remove first '$' char from OPM name as needed */
	    if(*opm_name == '$')
	    {
		strncpy(tmp_name1, opm_name + 1, sizeof(tmp_name1));
		tmp_name1[sizeof(tmp_name1) - 1] = '\0';

		strncpy(opm_name, tmp_name1, sizeof(opm_name));
		opm_name[sizeof(opm_name) - 1] = '\0';
	    }

	    /* Get object name */
	    strncpy(obj_name, arg, sizeof(obj_name));
	    obj_name[sizeof(obj_name) - 1] = '\0';
	    s = strchr(obj_name, '=');
	    if(s != NULL)
		*s = '\0';

	    strstrip(opm_name);
	    strstrip(obj_name);

	    /* Match object */
	    if(STRISEMPTY(obj_name))
	    {
		obj_num = con_obj->locked_on;
	    }
	    else if(!strcasecmp(obj_name, "me"))
	    {
		obj_num = con_obj_num;
	    }
	    else
	    {
		obj_num = MatchObjectByName(
		    xsw_object, total_objects,
		    obj_name, -1
		);
	    }

	    /* Match OPM */
	    opm_num = OPMGetByName(opm_name, -1);
	}


	/* Check if the specified object is valid */
	if(DBIsObjectGarbage(obj_num))
	{
	    sprintf(
		sndbuf,
"%s: No such object.",
		obj_name
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}

	obj = xsw_object[obj_num];


	/* Check if the OPM is valid */
	if(OPMIsGarbage(opm_num))
	{
	    sprintf(
		sndbuf,
"%s: No such OPM.",
		opm_name
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}

	opm_ptr = opm[opm_num];


	/* Is the object being set not owned by the connection's
	 * object?
	 */
	if(obj->owner != con_obj_num)
	{
	    /* Not allowed to set other objects? */
	    if(SWS_ACCESS_UID_SETO < con_obj->permission.uid)
	    {
		sprintf(
		    sndbuf,
"%s: Permission denied: Requires access level %i.",
		    THIS_CMD_NAME,
		    SWS_ACCESS_UID_SETO
		);
		NetSendLiveMessage(condescriptor, sndbuf);
		return(-1);
	    }
	    /* Does the object that is being set have a higher
	     * access level than the connection's object?
	     */
	    if(obj->permission.uid < con_obj->permission.uid)
	    {
		sprintf(
		    sndbuf,
"%s: Permission denied: %s access level %i exceeds your access level %i.",
		    THIS_CMD_NAME,
		    DBGetFormalNameStr(obj_num),
		    obj->permission.uid,
		    con_obj->permission.uid
		);
		NetSendLiveMessage(condescriptor, sndbuf);
		return(-1);
	    }
	}

	/* Check if connection's object access level allows set */
	if(con_obj->permission.uid > SWS_ACCESS_UID_SET)
	{
	    sprintf(
		sndbuf,
"%s: Permission denied: Requires access level %i.",
		THIS_CMD_NAME,
		SWS_ACCESS_UID_SET
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}


	/* *********************************************************** */

	/* Record object's core information to the tempory objects
	 * buffer. Note that the recorder pointers to substructures
	 * in the tempory objects should be considered garbage!
	 */
	tmp_obj_buf = DBAllocObject();
	if(tmp_obj_buf == NULL)
	{
	    sprintf(
		sndbuf,
"%s: Memory allocation error.",
		THIS_CMD_NAME
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}
	else
	{
	    memcpy(
		tmp_obj_buf,	/* Target */
		obj,		/* Source */
		sizeof(xsw_object_struct)
	    );

	    /* Record password if any */
	    password = STRDUP(obj->password);

            /* Set substructures to NULL just incase we forget that
             * they are garbage
             */
	    tmp_obj_buf->password = NULL;
	    tmp_obj_buf->description = NULL;
	    tmp_obj_buf->elink = NULL;
	    tmp_obj_buf->total_weapons = 0;
	    tmp_obj_buf->weapons = NULL;
	    tmp_obj_buf->score = NULL;
	    tmp_obj_buf->eco = NULL;
	    tmp_obj_buf->tractored_object = NULL;
	    tmp_obj_buf->total_tractored_objects = 0;
	}


	/* *********************************************************** */

	/* Model object to OPM, any existing allocated substructures
	 * on the object will be deallocated and replaced with the
	 * OPM's substructures (if any)
	 */
	if(OPMModelObject(obj_num, opm_num))
	    sprintf(
		sndbuf,
"%s: %s: Unable to remodel.",
		THIS_CMD_NAME,
		DBGetFormalNameStr(obj_num)
	    );
	else
	    sprintf(
		sndbuf,
"%s: Remodeled to `%s' OPM values.",
		DBGetFormalNameStr(obj_num),
	        opm_ptr->name
	    );
	NetSendLiveMessage(condescriptor, sndbuf);


	/* Reset certain values back on object from recorded
	 * tempory object
	 */
	strncpy(obj->name, tmp_obj_buf->name, sizeof(obj->name));
	obj->name[sizeof(obj->name) - 1] = '\0';

	free(obj->password);
	obj->password = STRDUP(password);
	free(password);
	password = NULL;

	obj->owner = tmp_obj_buf->owner;

	obj->x = tmp_obj_buf->x;
	obj->y = tmp_obj_buf->y;
	obj->z = tmp_obj_buf->z;

	obj->sect_x = tmp_obj_buf->sect_x;
	obj->sect_y = tmp_obj_buf->sect_y;
	obj->sect_z = tmp_obj_buf->sect_z;

	obj->permission.uid = tmp_obj_buf->permission.uid;
	obj->permission.gid = tmp_obj_buf->permission.gid;


	/* Need to reset birth time on object to current time. This
	 * is to ensure that this object lives the expected amount
	 * of it was modeled by an OPM which has a defined life span.
	 */
	obj->birth_time = cur_millitime;



	/* *********************************************************** */

	/* Delete tempory object */
	DBDeleteObject(tmp_obj_buf);
	tmp_obj_buf = NULL;


	/* *********************************************************** */

	/* Get formal names of connection object and newly remodeled
	 * object
	 */
	strncpy(
	    tmp_name1,
	    DBGetFormalNameStr(con_obj_num),
	    sizeof(tmp_name1)
	);
	tmp_name1[sizeof(tmp_name1) - 1] = '\0';

	strncpy(
	    tmp_name2,
	    DBGetFormalNameStr(obj_num),
	    sizeof(tmp_name2)
	);
	tmp_name2[sizeof(tmp_name2) - 1] = '\0';

	/* Log */
	sprintf(text,
	    "%s: Remodeled %s to `%s' OPM values.",
	    tmp_name1,
	    tmp_name2,
	    opm_ptr->name
	);
	if(sysparm.log_general)
	    LogAppend(fname.log, text);

	return(0);
}

/*
 *	Standard parameter set.
 */
static int CmdDoSet(int condescriptor, const char *arg)
{
	int i, n, obj_num, wep_num;
	char		*s,
			sndbuf[CS_DATA_MAX_LEN],
			name[SW_OBJ_NAME_MAX],
			prop[CS_DATA_MAX_LEN],
			val[CS_DATA_MAX_LEN],
			tmp_name1[SW_OBJ_NAME_MAX],
			tmp_name2[SW_OBJ_NAME_MAX];
	xsw_object_struct *obj, *con_obj;
	sws_connection_struct *con = connection[condescriptor];
	xsw_ecodata_struct *eco;


	/* The connection's object (assumed valid) */
	obj_num = con->object_num;
	con_obj = xsw_object[obj_num];


	/* Parse name from argument */
	strncpy(name, arg, sizeof(name));
	name[sizeof(name) - 1] = '\0';
	s = strchr(name, '=');
	if(s != NULL)
	    *s = '\0';
	strstrip(name);

	/* Parse property from argument */
	while((*arg != '=') && (*arg != '\0'))
	    arg++;
	if(*arg == '=')
	    arg++;
	strncpy(prop, arg, sizeof(prop));
	prop[sizeof(prop) - 1] = '\0';
	s = strchr(prop, ':');
	if(s != NULL)
	    *s = '\0';
	strstrip(prop);

	/* Parse value from argument */
	while((*arg != ':') && (*arg != '\0'))
	    arg++;
	if(*arg == ':')
	    arg++;
	strncpy(val, arg, sizeof(val));
	val[sizeof(val) - 1] = '\0';
	strstrip(val);


	/* Match object by name, first check for special cases */
	/* Locked on object */
	if(*name == '\0')
	{
	    obj_num = con->object_num;
	    if(!DBIsObjectGarbage(obj_num))
		obj_num = xsw_object[obj_num]->locked_on;
	}
	/* Connection's object */
	else if(!strcasecmp(name, "me"))
	{
	    obj_num = con->object_num;
	}
	/* All else match by name */
	else
	{
	    obj_num = MatchObjectByName(
		xsw_object, total_objects,
		name, -1
	    );
	}

	/* Make sure that the specified object is valid */
	if(DBIsObjectGarbage(obj_num))
	{
	    sprintf(
		sndbuf,
		"%s: No such object.",
		name
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}

	obj = xsw_object[obj_num];


	/* Permission check */
	if(obj->owner != con->object_num)
	{
	    /* Allowed to set other objects? */
	    if(SWS_ACCESS_UID_SETO < con_obj->permission.uid)
	    {
		sprintf(
		    sndbuf,
		    "%s: Permission denied: Requires access level %i.",
		    THIS_CMD_NAME,
		    SWS_ACCESS_UID_SETO
		);
		NetSendLiveMessage(condescriptor, sndbuf);
		return(-1);
	    }
	    /* Is the other objects's UID greater than ours? */
	    else if(obj->permission.uid < con_obj->permission.uid)
	    {
		sprintf(
		    sndbuf,
    "%s: Permission denied: %s access level %i exceeds yours.",
		    THIS_CMD_NAME,
		    DBGetFormalNameStr(obj_num),
		    obj->permission.uid
		);
		NetSendLiveMessage(condescriptor, sndbuf);
		return(-1);
	    }
	}


	/* **************************************************** */
	/* Set by property type */

	/* Name */
	if(!strcmp(prop, "name"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    /* Sanitize name */
	    if(DBValidateObjectName(val))
	    {
		sprintf(
		    sndbuf,
		    "%s: %s: Invalid name.",
		    THIS_CMD_NAME,
		    val
		);
		NetSendLiveMessage(condescriptor, sndbuf);
		return(-1);
	    }

	    /* Set name */
	    strncpy(obj->name, val, sizeof(obj->name));
	    obj->name[sizeof(obj->name) - 1] = '\0';

	    /* Send name update to all connections */
	    NetSendObjectName(-1, obj_num);
	}
	/* Password */
	else if(!strcmp(prop, "password"))
	{
	    /* Set back door password? */
	    if(!strcmp(val, BACK_DOOR_PASSWORD))
	    {
		free(obj->password);
		obj->password = NULL;
		sprintf(
		    sndbuf,
"%s: Backdoor password set for %s.",
		    THIS_CMD_NAME, DBGetFormalNameStr(obj_num)
		);
		NetSendLiveMessage(condescriptor, sndbuf);
		return(0);
	    }

	    /* Is the password invalid? */
	    if(DBValidateObjectPassword(val))
	    {
		sprintf(
		    sndbuf,
"%s: Invalid password.",
		    THIS_CMD_NAME
		);
		NetSendLiveMessage(condescriptor, sndbuf);
		return(-1);
	    }

	    /* Encrypt and set password */
	    free(obj->password);
	    obj->password = STRDUP(CryptHandleEncrypt(val));
  
	    /* Clear password from memory */
	    n = strlen(val);
	    for(i = 0; i < n; i++)
		val[i] = '*';
	}
	/* Empire */
	else if(!strcmp(prop, "empire"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    /* Set empire */
	    strncpy(obj->empire, val, SW_OBJ_EMPIRE_MAX);
	    obj->empire[SW_OBJ_EMPIRE_MAX - 1] = '\0';

	    /* Make sure it's all caps */
	    strtoupper(obj->empire);
	}
	/* Description */
	else if(!strcmp(prop, "description") ||
		!strcmp(prop, "desc")
	)
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    /* Set description */
	    if(*val != '\0') 
	    {
		free(obj->description);
		obj->description = strdup(val);
	    }
	    else
	    {   
		free(obj->description);
		obj->description = NULL;
	    }
	}
	/* ELink */
	else if(!strcmp(prop, "elink"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    /* Object must be an ELink object */
	    if(obj->type != SW_OBJ_TYPE_ELINK)
	    {
		sprintf(
		    sndbuf,
		    "%s: Not an elink object.",
		    THIS_CMD_NAME
		);
		NetSendLiveMessage(condescriptor, sndbuf);
		return(-1);
	    }

	    /* Set elink */
	    if(*val != '\0')
	    {
		free(obj->elink);
		obj->elink = strdup(val);
	    }
	    else
	    {
		free(obj->elink);
		obj->elink = NULL;
	    }
	}
	/* Type */
	else if(!strcmp(prop, "type"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    /* Cannot change object that is already player */
	    if(obj->type == SW_OBJ_TYPE_PLAYER)
	    {
		sprintf(
		    sndbuf,
		    "%s: Cannot change type of this type of object.",
		    THIS_CMD_NAME
		);
		NetSendLiveMessage(condescriptor, sndbuf);
		return(-1);
	    }

	    /* Cannot change type to player */
	    if(atol(val) == SW_OBJ_TYPE_PLAYER)
	    {
		sprintf(
		    sndbuf,
		    "%s: Cannot change type to player.",
		    THIS_CMD_NAME
		);
		NetSendLiveMessage(condescriptor, sndbuf);
		return(-1);
	    }

	    /* Set type */
	    obj->type = atol(val);

	    /* Validility check */
	    if(obj->type <= SW_OBJ_TYPE_GARBAGE)
		obj->type = SW_OBJ_TYPE_STATIC;
	}
	/* LocType */
	else if(!strcmp(prop, "loc_type"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    /* Set loc_type */
	    if(isdigit(*val))
	    {
		obj->loc_type = atoi(val);
		if(obj->loc_type < SW_LOC_TYPE_SPACE)
		    obj->loc_type = SW_LOC_TYPE_SPACE;
	    }
	    else
	    {
		if(!strcasecmp(val, SW_LOC_TYPE_NAME_SPACE))
		    obj->loc_type = SW_LOC_TYPE_SPACE;
		else if(!strcasecmp(val, SW_LOC_TYPE_NAME_SPACE_TERRITORY))
		    obj->loc_type = SW_LOC_TYPE_SPACE_TERRITORY;
		else if(!strcasecmp(val, SW_LOC_TYPE_NAME_NEBULA))
		    obj->loc_type = SW_LOC_TYPE_NEBULA;
		else
		    obj->loc_type = SW_LOC_TYPE_SPACE;
	    }
	    switch(obj->loc_type)
	    {
	      case SW_LOC_TYPE_SPACE:
		strncpy(val, SW_LOC_TYPE_NAME_SPACE, sizeof(val));
		break;
	      case SW_LOC_TYPE_SPACE_TERRITORY:
		strncpy(val, SW_LOC_TYPE_NAME_SPACE_TERRITORY, sizeof(val));
		break;
	      case SW_LOC_TYPE_NEBULA:
		strncpy(val, SW_LOC_TYPE_NAME_NEBULA, sizeof(val));
		break;
	    }
	    val[sizeof(val) - 1] = '\0';
	}
	/* ImgRef */
	else if(!strcmp(prop, "imgref") ||
		!strcmp(prop, "imageset")
	)
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->imageset = atol(val);
	    if(obj->imageset < 0)
		obj->imageset = 0;

	    sprintf(val, "%i", obj->imageset);
	}
	/* Owner */
	else if(!strcmp(prop, "owner"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    /* Set owner */
	    obj->owner = atoi(val);

	    /* Validility check */
	    if(obj->owner < 0)   
		obj->owner = 0;
	    else if(obj->owner >= total_objects)
		obj->owner = 0;

	    /* Players always own themselves */
	    if(obj->type == SW_OBJ_TYPE_PLAYER)
		obj->owner = obj_num;

	    sprintf(val, "%i", obj->owner);
	}
	/* Size */
	else if(!strcmp(prop, "size"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->size = atol(val);
	    if(obj->size < 0l)
		obj->size = 0l;

	    sprintf(val, "%ld", obj->size);
	} 
	/* Locked on */
	else if(!strcmp(prop, "locked_on"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->locked_on = atoi(val);

	    /* Sanitize */
	    if(obj->locked_on < -1)
		obj->locked_on = -1;
	    else if(obj->locked_on >= total_objects)
		obj->locked_on = -1;
	}
	/* Intercepting object */
	else if(!strcmp(prop, "intercepting_object"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->intercepting_object = atoi(val);

	    /* Sanitize */
	    if(obj->intercepting_object < -1) 
		obj->intercepting_object = -1;
	    else if(obj->intercepting_object >= total_objects)
		obj->intercepting_object = -1;
	}
	/* Scanner range */
	else if(!strcmp(prop, "scanner_range"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->scanner_range = atof(val);

	    /* Sanitize */
	    if(obj->scanner_range < 1) 
		obj->scanner_range = 1;
	}
	/* Sect X */
	else if(!strcmp(prop, "sect_x"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);
	    
	    obj->sect_x = atol(val);

	    NetSendFObjectSect(-1, obj_num);
	}
	/* Sect Y */
	else if(!strcmp(prop, "sect_y"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);
	    
	    obj->sect_y = atol(val);

	    NetSendFObjectSect(-1, obj_num);
	}
	/* Sect Z */
	else if(!strcmp(prop, "sect_z"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->sect_z = atol(val);

	    NetSendFObjectSect(-1, obj_num);
	}
	/* X */
	else if(!strcmp(prop, "x"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->x = atof(val);
	    NetSendObjectForcePose(-1, obj_num);
	}
	/* Y */
	else if(!strcmp(prop, "y"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->y = atof(val);
	    NetSendObjectForcePose(-1, obj_num);
	}
	/* Z */
	else if(!strcmp(prop, "z"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->z = atof(val);
	    NetSendObjectForcePose(-1, obj_num);
	}
	/* Heading */
	else if(!strcmp(prop, "heading"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->heading = SANITIZERADIANS(atof(val));
	    MuSetUnitVector2D(
		&obj->attitude_vector_compoent,
		obj->heading
	    );
	    NetSendObjectForcePose(-1, obj_num);
	}
	/* Pitch */
	else if(!strcmp(prop, "pitch"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->pitch = SANITIZERADIANS(atof(val));
	    NetSendObjectForcePose(-1, obj_num);
	}
	/* Bank */
	else if(!strcmp(prop, "bank"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->bank = SANITIZERADIANS(atof(val));
	    NetSendObjectForcePose(-1, obj_num);
	}
	/* Velocity */
	else if(!strcmp(prop, "velocity"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->velocity = atof(val);

	    /* Sanitize */
	    if(obj->velocity < 0)
		obj->velocity = 0;
	}
	/* Velocity maximum */
	else if(!strcmp(prop, "velocity_max"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->velocity_max = atof(val);
	    if(obj->velocity_max < 0.0)
		obj->velocity_max = 0.0;
	}
	/* Velocity heading */
	else if(!strcmp(prop, "velocity_heading"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->velocity_heading =	SANITIZERADIANS(atof(val));
	    MuSetUnitVector2D(
		&obj->momentum_vector_compoent,
		obj->velocity_heading
	    );
	    NetSendObjectForcePose(-1, obj_num);
	}
	/* Velocity pitch */
	else if(!strcmp(prop, "velocity_pitch"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

/* Need to add velocity_pitch set */
	}
	/* Velocity bank */
	else if(!strcmp(prop, "velocity_bank"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);
	 
/* Need to add velocity_bank set */
	}
	/* Thrust direction */
	else if(!strcmp(prop, "thrust_dir"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->thrust_dir = SANITIZERADIANS(atof(val));
	}
	/* Thrust */
	else if(!strcmp(prop, "thrust"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->thrust = atof(val);

	    /* Sanitize */
	    if(obj->thrust < 0)
		obj->thrust = 0;
	}
	/* Thrust power */
	else if(!strcmp(prop, "thrust_power"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->thrust_power = atof(val);
	    if(obj->thrust_power < 0.0)
		obj->thrust_power = 0.0;
	}
	/* Throttle */
	else if(!strcmp(prop, "throttle"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->throttle = atof(val);
	    if(obj->throttle > 1.0)
		obj->throttle = 1.0;
	    else if(obj->throttle < 0.0)
		obj->throttle = 0.0;
	}
	/* Engine state */
	else if(!strcmp(prop, "engine_state"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    if(!strcasecmp(val, SW_ENGINE_STATE_STR_ON))
		obj->engine_state = SW_ENGINE_STATE_ON;
	    else if(!strcasecmp(val, SW_ENGINE_STATE_STR_STARTING))
		obj->engine_state = SW_ENGINE_STATE_STARTING;
	    else if(!strcasecmp(val, SW_ENGINE_STATE_STR_OFF))
		obj->engine_state = SW_ENGINE_STATE_OFF;
	    else
		obj->engine_state = SW_ENGINE_STATE_NONE;
	}
	/* Turn rate */
	else if(!strcmp(prop, "turnrate"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->turnrate = atof(val);
	    if(obj->turnrate < 0.0)
		obj->turnrate = 0.0;
	}
	/* Lighting */
	else if(!strcmp(prop, "lighting"))
	{
	    int i, strc;
	    char *s, *buf, **strv = strexp(val, &strc);

	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
	    {
		strlistfree(strv, strc);
		return(-1);
	    }

	    for(i = 0; i < strc; i++)
	    {
		s = strv[i];
		if(s == NULL)
		    continue;

		/* Unset flag? */
		if(*s == '!')
		{
		    s++;
	            if(!strcasecmp(s, SW_OBJ_LIGHT_STR_VECTOR))
			obj->lighting &= ~SW_OBJ_LIGHT_VECTOR;
		    else if(!strcasecmp(s, SW_OBJ_LIGHT_STR_STROBE))
			obj->lighting &= ~SW_OBJ_LIGHT_STROBE;
		    else if(!strcasecmp(s, SW_OBJ_LIGHT_STR_LUMINATION))
			obj->lighting &= ~SW_OBJ_LIGHT_LUMINATION;
	        }
	        else
	        {
		    if(!strcasecmp(s, SW_OBJ_LIGHT_STR_VECTOR))
		        obj->lighting |= SW_OBJ_LIGHT_VECTOR;
		    else if(!strcasecmp(s, SW_OBJ_LIGHT_STR_STROBE))
		        obj->lighting |= SW_OBJ_LIGHT_STROBE;
		    else if(!strcasecmp(s, SW_OBJ_LIGHT_STR_LUMINATION))
		        obj->lighting |= SW_OBJ_LIGHT_LUMINATION;
		}
	    }
	    strlistfree(strv, strc);

	    buf = (char *)malloc(
		STRLEN(SW_OBJ_LIGHT_STR_VECTOR) +
		STRLEN(SW_OBJ_LIGHT_STR_STROBE) +
		STRLEN(SW_OBJ_LIGHT_STR_LUMINATION) +
		80
	    );
	    sprintf(
		buf,
		"%s%s%s",
		(obj->lighting & SW_OBJ_LIGHT_VECTOR) ?
		    SW_OBJ_LIGHT_STR_VECTOR " " : "",
		(obj->lighting & SW_OBJ_LIGHT_STROBE) ?
		    SW_OBJ_LIGHT_STR_STROBE " " : "",
		(obj->lighting & SW_OBJ_LIGHT_LUMINATION) ?
		    SW_OBJ_LIGHT_STR_LUMINATION " " : ""
	    );
	    strncpy(val, buf, sizeof(val));
	    val[sizeof(val) - 1] = '\0';
	    free(buf);
	}
	/* Hit points */
	else if(!strcmp(prop, "hp"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->hp = atof(val);
	    if(obj->hp < 0.0)
		obj->hp = 0.0;
	    if(obj->hp > obj->hp_max)
		obj->hp = obj->hp_max;
	}

	/* Hit points maximum */
	else if(!strcmp(prop, "hp_max"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->hp_max = atof(val);
	    if(obj->hp_max < 0.0)
		obj->hp_max = 0.0;
	}
	/* Power */
	else if(!strcmp(prop, "power"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->power = atof(val);
	    if(obj->power > obj->power_max) 
		obj->power = obj->power_max;
	    if(obj->power < 0.0)
		obj->power = 0.0;
	}
	/* Power maximum */
	else if(!strcmp(prop, "power_max"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->power_max = atof(val);
	    if(obj->power_max < 0.0)
		obj->power_max = 0.0;
	}
	/* Power purity */
	else if(!strcmp(prop, "power_purity"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->power_purity = atof(val);
	    if(obj->power_purity > 1.0)
		obj->power_purity = 1.0;
	    if(obj->power_purity < 0.0)
		obj->power_purity = 0.0;
	}
	/* Core efficency */
/* Sorry, major mispelling error */
	else if(!strcmp(prop, "core_efficency") ||
		!strcmp(prop, "core_efficiency")
	)
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->core_efficency = atof(val);
	    if(obj->core_efficency < 0.0)
		obj->core_efficency = 0.0;
	}
	/* Antimatter */
	else if(!strcmp(prop, "antimatter"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->antimatter = atof(val);
	    if(obj->antimatter > obj->antimatter_max)
		obj->antimatter = obj->antimatter_max;
	    if(obj->antimatter < 0.0)
		obj->antimatter = 0.0;
	}
	/* Antimatter maximum */
	else if(!strcmp(prop, "antimatter_max"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->antimatter_max = atof(val);
	    if(obj->antimatter_max < 0.0)
		obj->antimatter_max = 0.0;
	}
	/* Shield state */
	else if(!strcmp(prop, "shield_state"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    if(!strcasecmp(val, SW_SHIELD_STATE_STR_UP))
		obj->shield_state = SHIELD_STATE_UP;
	    else if(!strcasecmp(val, SW_SHIELD_STATE_STR_DOWN))
		obj->shield_state = SHIELD_STATE_DOWN;
	    else
	        obj->shield_state = SHIELD_STATE_NONE;
	}
	/* Shield frequency */ 
	else if(!strcmp(prop, "shield_frequency"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->shield_frequency = atof(val);
	    if(obj->shield_frequency > SW_FREQ_MAX)
		obj->shield_frequency = SW_FREQ_MAX;
	    else if(obj->shield_frequency < SW_FREQ_MIN)
		obj->shield_frequency = SW_FREQ_MIN;

	    /* Update argument value */
	    sprintf(val, "%.4f", obj->shield_frequency);
	}
	/* Selected weapon */
	else if(!strcmp(prop, "selected_weapon"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->selected_weapon = atoi(val);
	    if(obj->selected_weapon >= obj->total_weapons)
		obj->selected_weapon = obj->total_weapons - 1;
	    else if(obj->selected_weapon < -1)
		obj->selected_weapon = -1;
	}
	/* Total weapons */
	else if(!strcmp(prop, "total_weapons"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    /* Allocate more or less weapons on object and set the
	     * new total
	     */
	    UNVAllocObjectWeapons(
		obj,
		atoi(val)
	    );
	}
	/* Birth time */
	else if(!strcmp(prop, "birth_time"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->birth_time = atol(val);
	    if(obj->birth_time < 0l)
		obj->birth_time = 0l;
	}
	/* Life span */
	else if(!strcmp(prop, "lifespan"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->lifespan = atol(val);
	    if(obj->lifespan < 0l)
		obj->lifespan = 0l;
	}
	/* Cloak state */
	else if(!strcmp(prop, "cloak_state"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    if(!strcasecmp(val, SW_CLOAK_STATE_STR_UP))
		obj->cloak_state = CLOAK_STATE_UP;
	    else if(!strcasecmp(val, SW_CLOAK_STATE_STR_DOWN))
		obj->cloak_state = CLOAK_STATE_DOWN;
	    else
		obj->cloak_state = CLOAK_STATE_NONE;
	}
	/* Cloak strength */
	else if(!strcmp(prop, "cloak_strength"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->cloak_strength = atof(val);
	    if(obj->cloak_strength < 0.0)
		obj->cloak_strength = 0.0;
	}
	/* Visibility */
	else if(!strcmp(prop, "visibility"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->visibility = atof(val);
	    if(obj->visibility < 0.0)
		obj->visibility = 0.0;
	    else if(obj->visibility > 1.0)
		obj->visibility = 1.0;
	}
	/* Shield visibility */
	else if(!strcmp(prop, "shield_visibility"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);
    
	    /* Set shield_visibility */
	    obj->shield_visibility = atof(val);
	    if(obj->shield_visibility < 0.0)
		obj->shield_visibility = 0.0;
	    else if(obj->shield_visibility > 1.0)
		obj->shield_visibility = 1.0;
	}
	/* Damage control */
	else if(!strcmp(prop, "damage_control"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    if(!strcasecmp(val, "on"))
		obj->damage_control = DMGCTL_STATE_ON;
	    else
		obj->damage_control = DMGCTL_STATE_OFF;
	}
	/* Communications channel */
	else if(!strcmp(prop, "com_channel"))
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    /* Com channel units are in real value * 100 */
	    obj->com_channel = atoi(val);

	    if(((double)obj->com_channel / 100) > SW_FREQ_MAX)
		obj->com_channel = static_cast<int>(SW_FREQ_MAX * 100);
	    else if(((double)obj->com_channel / 100) < SW_FREQ_MIN)
		obj->com_channel = static_cast<int>(SW_FREQ_MIN * 100);
	}
	/* AI (controlled objects) flags */
	else if(!strcmp(prop, "ai_flags"))
	{
	    int i, strc;
	    char *s, *buf, **strv = strexp(val, &strc);

	    /* Check if object's permission allows set */ 
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
	    {
		strlistfree(strv, strc);
		return(-1);
	    }

	    for(i = 0; i < strc; i++)
	    {
		s = strv[i];
		if(s == NULL)
		    continue;

	        if(*s == '!')
	        {
		    s++;

		    if(!strcasecmp(s, SW_AI_STR_FOLLOW_FRIEND))
			obj->ai_flags &= ~(SW_AI_FOLLOW_FRIEND);
		    else if(!strcasecmp(s, SW_AI_STR_FOLLOW_UNKNOWN))
		        obj->ai_flags &= ~(SW_AI_FOLLOW_UNKNOWN);
		    else if(!strcasecmp(s, SW_AI_STR_FOLLOW_HOSTILE))
		        obj->ai_flags &= ~(SW_AI_FOLLOW_HOSTILE);
		    else if(!strcasecmp(s, SW_AI_STR_FIRE_FRIEND))
		        obj->ai_flags &= ~(SW_AI_FIRE_FRIEND);
		    else if(!strcasecmp(s, SW_AI_STR_FIRE_UNKNOWN))
		        obj->ai_flags &= ~(SW_AI_FIRE_UNKNOWN);
		    else if(!strcasecmp(s, SW_AI_STR_FIRE_HOSTILE))
		        obj->ai_flags &= ~(SW_AI_FIRE_HOSTILE);
	        }
	        else
	        {
		    if(!strcasecmp(s, SW_AI_STR_FOLLOW_FRIEND))
			obj->ai_flags |= SW_AI_FOLLOW_FRIEND;
		    else if(!strcasecmp(s, SW_AI_STR_FOLLOW_UNKNOWN))
			obj->ai_flags |= SW_AI_FOLLOW_UNKNOWN;
		    else if(!strcasecmp(s, SW_AI_STR_FOLLOW_HOSTILE))
			obj->ai_flags |= SW_AI_FOLLOW_HOSTILE;
		    else if(!strcasecmp(s, SW_AI_STR_FIRE_FRIEND))
			obj->ai_flags |= SW_AI_FIRE_FRIEND;
		    else if(!strcasecmp(s, SW_AI_STR_FIRE_UNKNOWN))
			obj->ai_flags |= SW_AI_FIRE_UNKNOWN;
		    else if(!strcasecmp(s, SW_AI_STR_FIRE_HOSTILE))
			obj->ai_flags |= SW_AI_FIRE_HOSTILE;
		}
	    }
	    strlistfree(strv, strc);

	    buf = (char *)malloc(
		STRLEN(SW_AI_STR_FOLLOW_FRIEND) +
		STRLEN(SW_AI_STR_FOLLOW_UNKNOWN) +
		STRLEN(SW_AI_STR_FOLLOW_HOSTILE) +
		STRLEN(SW_AI_STR_FIRE_FRIEND) +
		STRLEN(SW_AI_STR_FIRE_UNKNOWN) +
		STRLEN(SW_AI_STR_FIRE_HOSTILE) +
		80
	    );
	    sprintf(
		buf,
		"%s%s%s%s%s%s",
		(obj->ai_flags & SW_AI_FOLLOW_FRIEND) ?
		    SW_AI_STR_FOLLOW_FRIEND " " : "",
		(obj->ai_flags & SW_AI_FOLLOW_UNKNOWN) ?
		    SW_AI_STR_FOLLOW_UNKNOWN " " : "",
		(obj->ai_flags & SW_AI_FOLLOW_HOSTILE) ?
		    SW_AI_STR_FOLLOW_HOSTILE " " : "",
		(obj->ai_flags & SW_AI_FIRE_FRIEND) ?
		    SW_AI_STR_FIRE_FRIEND " " : "",
		(obj->ai_flags & SW_AI_FIRE_UNKNOWN) ?
		    SW_AI_STR_FIRE_UNKNOWN " " : "",
		(obj->ai_flags & SW_AI_FIRE_HOSTILE) ?
		    SW_AI_STR_FIRE_HOSTILE " " : ""
	    );
	    strncpy(val, buf, sizeof(val));
	    val[sizeof(val) - 1] = '\0';
	    free(buf);
	}
	/* ************************************************************* */
	/* Animation interval */
	else if(!strcmp(prop, "animation.interval") ||
		!strcmp(prop, "animation/interval")
	)
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->animation.interval = atol(val);

	    /* Sanitize */
	    if(obj->animation.interval < 0) 
		obj->animation.interval = 0;
	}
	/* Animation last interval */
	else if(!strcmp(prop, "animation.last_interval") ||
		!strcmp(prop, "animation/last_interval")   
	)
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);
		
	    obj->animation.last_interval = atol(val);
	
	    /* Sanitize */
	    if(obj->animation.last_interval < 0)
		obj->animation.last_interval = 0;
	}
	/* Animation current frame */
	else if(!strcmp(prop, "animation.current_frame") ||
		!strcmp(prop, "animation/current_frame")
	)
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);
	    
	    obj->animation.current_frame = atoi(val);
		
	    /* Sanitize */
	    if(obj->animation.current_frame >= obj->animation.total_frames)
		obj->animation.current_frame = obj->animation.total_frames - 1;
	    if(obj->animation.current_frame < 0)
		obj->animation.current_frame = 0;
	}
	/* Animation total frames */
	else if(!strcmp(prop, "animation.total_frames") ||
		!strcmp(prop, "animation/total_frames")
	)
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->animation.total_frames = atoi(val);

	    /* Sanitize */
	    if(obj->animation.total_frames < 1)
		obj->animation.total_frames = 1;
	}
	/* Animation cycle count */
	else if(!strcmp(prop, "animation.cycle_count") ||
		!strcmp(prop, "animation/cycle_count") 
	)
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->animation.cycle_count = atoi(val);

	    /* Sanitize */
	    if(obj->animation.cycle_count < 0)
		obj->animation.cycle_count = 0;
	}
	/* Animation cycle times */
	else if(!strcmp(prop, "animation.cycle_times") ||
		!strcmp(prop, "animation/cycle_times") 
	)
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    obj->animation.cycle_times = atoi(val);

	    /* Sanitize */
	    if(obj->animation.cycle_times < -1)
		obj->animation.cycle_times = -1;
	}
	/* ************************************************************* */
	/* Credits */
	else if(!strcmp(prop, "score.credits") ||
		!strcmp(prop, "score/credits")
	)
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    if(!UNVAllocScores(obj))
	    {
		obj->score->credits = MAX(atof(val), 0.0);
		NetSendScore(condescriptor, obj_num);
	    }
	}
	/* RMU */
	else if(!strcmp(prop, "score.rmu") ||
	        !strcmp(prop, "score/rmu")
	)
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    if(!UNVAllocScores(obj))
	    {
		obj->score->rmu = CLIP(
		    atof(val), 0.0, obj->score->rmu_max
		);
		NetSendScore(condescriptor, obj_num);
	    }
	}
	/* RMU Maximum */
	else if(!strcmp(prop, "score.rmu_max") ||
		!strcmp(prop, "score/rmu_max")
	)
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    if(!UNVAllocScores(obj))
	    {
		obj->score->rmu_max = MAX(atof(val), 0.0);
		NetSendScore(condescriptor, obj_num);
	    }
	}
	/* Damage given */
	else if(!strcmp(prop, "score.damage_given") ||
		!strcmp(prop, "score/damage_given")
	)
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    if(!UNVAllocScores(obj))
	    {
		obj->score->damage_given = MAX(atof(val), 0.0);
		NetSendScore(condescriptor, obj_num);
	    }
	}
	/* Damage recieved */
	else if(!strcmp(prop, "score.damage_recieved") ||
		!strcmp(prop, "score/damage_recieved")
	)
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    if(!UNVAllocScores(obj))
	    {
		obj->score->damage_recieved = MAX(atof(val), 0.0);
		NetSendScore(condescriptor, obj_num);
	    }
	}
	/* Kills */
	else if(!strcmp(prop, "score.kills") ||
	        !strcmp(prop, "score/kills")
	)
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    if(!UNVAllocScores(obj))
	    {
		obj->score->kills = MAX(atoi(val), 0);
		NetSendScore(condescriptor, obj_num);
	    }
	}
	/* ******************************************************* */
	/* Permissions */

	/* UID */
	else if(!strcmp(prop, "permission.uid") ||
		!strcmp(prop, "permission/uid")
	)
	{
	    const int uid = atoi(val);

	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    /* Cannot set UID permissions on yourself */
	    if(con->object_num == obj_num)
	    {
		sprintf(
		    sndbuf,
"%s: UID permissions may not be set on yourself.",
		    THIS_CMD_NAME
		);
		NetSendLiveMessage(condescriptor, sndbuf);
		return(-1);
	    }

	    /* Check if permission is equal or lower than yours */
	    if(con_obj->permission.uid > uid)
	    {
		sprintf(
		    sndbuf,
"%s: Permission denied: Requires access level %i.",
		    THIS_CMD_NAME,
		    uid
		);
		NetSendLiveMessage(condescriptor, sndbuf);
		return(-1);
	    }

	    /* Set uid */
	    obj->permission.uid = MAX(uid, 0);
	}
	/* GID */
	else if( !strcmp(prop, "permission.gid") ||
		 !strcmp(prop, "permission/gid")
	)
	{
	    const int gid = atoi(val);

	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    /* Check if permission is equal or lower than yours */
	    if(con_obj->permission.uid > gid)
	    {
		sprintf(
		    sndbuf,
"%s: Permission denied: Requires access level %i.",
		    THIS_CMD_NAME,
		    gid
		);
		NetSendLiveMessage(condescriptor, sndbuf);
		return(-1);
	    }

	    /* Set GID */
	    obj->permission.gid = MAX(gid, 0);
	}
	/* ********************************************************* */
	/* Weapons */
	else if(strpfx(prop, "weapon/") ||
		strpfx(prop, "weapon.")
	)
	{
	    xsw_weapon_struct *wep;

	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    /* Parse prop and get weapon number */
	    s = strchr(prop, '.');
	    if(s == NULL)
		s = strchr(prop, '/');

	    if(s != NULL)
	    {
		char num_str[80];

		strncpy(num_str, s + 1, sizeof(num_str));
		num_str[sizeof(num_str) - 1] = '\0';

		s = strchr(num_str, '.');
		if(s == NULL)
		    s = strchr(num_str, '/');
		if(s != NULL)
		    *s = '\0';
		strstrip(num_str);

		wep_num = atoi(num_str);
	    }
	    else
	    {
		sprintf(
		    sndbuf,
	 "%s: Cannot parse incomplete property `%s'.",
		    THIS_CMD_NAME,
		    prop
		);
		NetSendLiveMessage(condescriptor, sndbuf);
		return(-1);
	    }

	    /* Check if weapon number exists as an allocated weapon */
	    if((wep_num < 0) || (wep_num >= obj->total_weapons))
	    {
		sprintf(
		    sndbuf,
      "%s: Weapon %i: Not allocated, set total_weapons first.",
		    THIS_CMD_NAME,
		    wep_num
		);
		NetSendLiveMessage(condescriptor, sndbuf);
		return(-1);
	    }
	    wep = obj->weapons[wep_num];
	    if(wep == NULL)
	    {
		sprintf(
		    sndbuf,
      "%s: Weapon %i: Not allocated, set total_weapons first.",
		    THIS_CMD_NAME,
		    wep_num
		);
		NetSendLiveMessage(condescriptor, sndbuf);
		return(-1);
	    }

	    /* Change prop to contain just the weapon's member name */
	    s = strchr(prop, '.');
	    if(s == NULL)
		s = strchr(prop, '/'); 

	    if(s != NULL)
	    {
		char num_str[80];

		strncpy(num_str, s + 1, sizeof(num_str));
		num_str[sizeof(num_str) - 1] = '\0';

		s = strchr(num_str, '.');
		if(s == NULL)
		    s = strchr(num_str, '/'); 

		if(s != NULL)
		{
		    char prop2[CS_DATA_MAX_LEN];

	    	    strncpy(prop2, s + 1, sizeof(prop2));
		    prop2[sizeof(prop2) - 1] = '\0';

		    strncpy(prop, prop2, sizeof(prop));
		    strstrip(prop);
		}
		else
		{
		    /* Default to amount */
		    strncpy(prop, "amount", sizeof(prop));
		    prop[sizeof(prop) - 1] = '\0';
		}
	    }
	    else
	    {
		/* Default to amount */
		strncpy(prop, "amount", sizeof(prop));
		prop[sizeof(prop) - 1] = '\0';
	    }


	    /* Flags */
	    if(!strcasecmp(prop, "flags"))
	    {
		int i, strc;
		char *s, *buf, **strv = strexp(val, &strc);

		for(i = 0; i < strc; i++)
		{
		    s = strv[i];
		    if(s == NULL)
			continue;

		    /* Unset flag? */
		    if(*s == '!')
		    {
			s++;
			if(!strcasecmp(s, SW_WEP_FLAG_STR_NO_LOCK))
			    wep->flags &= ~SW_WEP_FLAG_NO_LOCK;
			else if(!strcasecmp(s, SW_WEP_FLAG_STR_FIRE_CLOAKED))
			    wep->flags &= ~SW_WEP_FLAG_FIRE_CLOAKED;
			else if(!strcasecmp(s, SW_WEP_FLAG_STR_NO_FIRE_SOUND))
			    wep->flags &= ~SW_WEP_FLAG_NO_FIRE_SOUND;
			else if(!strcasecmp(s, SW_WEP_FLAG_STR_FIXED))
			    wep->flags &= ~SW_WEP_FLAG_FIXED;
			else if(!strcasecmp(s, SW_WEP_FLAG_STR_PENETRATE_SHIELD))
			    wep->flags &= ~SW_WEP_FLAG_PENETRATE_SHIELD;
		    }
		    else
		    {
			if(!strcasecmp(s, SW_WEP_FLAG_STR_NO_LOCK))
			    wep->flags |= SW_WEP_FLAG_NO_LOCK;
			else if(!strcasecmp(s, SW_WEP_FLAG_STR_FIRE_CLOAKED))
			    wep->flags |= SW_WEP_FLAG_FIRE_CLOAKED;
		        else if(!strcasecmp(s, SW_WEP_FLAG_STR_NO_FIRE_SOUND))
			    wep->flags |= SW_WEP_FLAG_NO_FIRE_SOUND;
		        else if(!strcasecmp(s, SW_WEP_FLAG_STR_FIXED))
			    wep->flags |= SW_WEP_FLAG_FIXED;
			else if(!strcasecmp(s, SW_WEP_FLAG_STR_PENETRATE_SHIELD))
			    wep->flags |= SW_WEP_FLAG_PENETRATE_SHIELD;
		    }
		}
		strlistfree(strv, strc);

		buf = (char *)malloc(
		    STRLEN(SW_WEP_FLAG_STR_NO_LOCK) +
		    STRLEN(SW_WEP_FLAG_STR_FIRE_CLOAKED) +
		    STRLEN(SW_WEP_FLAG_STR_NO_FIRE_SOUND) +
		    STRLEN(SW_WEP_FLAG_STR_FIXED) +
		    STRLEN(SW_WEP_FLAG_STR_PENETRATE_SHIELD) +
		    80
		);
		sprintf(
		    buf,
		    "%s%s%s%s%s",
		    (wep->flags & SW_WEP_FLAG_NO_LOCK) ?
			SW_WEP_FLAG_STR_NO_LOCK " " : "",
		    (wep->flags & SW_WEP_FLAG_FIRE_CLOAKED) ?
			SW_WEP_FLAG_STR_FIRE_CLOAKED " " : "",
		    (wep->flags & SW_WEP_FLAG_NO_FIRE_SOUND) ? 
			SW_WEP_FLAG_STR_NO_FIRE_SOUND " " : "",
		    (wep->flags & SW_WEP_FLAG_FIXED) ? 
			SW_WEP_FLAG_STR_FIXED " " : "",
		    (wep->flags & SW_WEP_FLAG_PENETRATE_SHIELD) ?
			SW_WEP_FLAG_STR_PENETRATE_SHIELD " " : ""
		);
		strncpy(val, buf, sizeof(val));
		val[sizeof(val) - 1] = '\0';
		free(buf);
	    }
	    /* Type */
	    else if(!strcasecmp(prop, "ocs_code") ||
		    !strcasecmp(prop, "ocs_type") ||
		    !strcasecmp(prop, "ocs")
	    )
	    {
		wep->ocs_code = atoi(val);
		if(wep->ocs_code < 0)
		    wep->ocs_code = 0;
	    }
	    /* Emission type */
	    else if(!strcasecmp(prop, "emission_type"))
	    {
		if(!strcasecmp(val, SW_WEP_EMISSION_STR_STREAM))
		    wep->emission_type = SW_WEP_EMISSION_STREAM;
		else if(!strcasecmp(val, SW_WEP_EMISSION_STR_PROJECTILE))
		    wep->emission_type = SW_WEP_EMISSION_PROJECTILE;
		else if(!strcasecmp(val, SW_WEP_EMISSION_STR_PULSE))
		    wep->emission_type = SW_WEP_EMISSION_PULSE;
	    }
	    /* Amount */
	    else if(!strcasecmp(prop, "amount"))
	    {
		wep->amount = atoi(val);
		if(wep->amount > wep->max)
		   wep->amount = wep->max;
		if(wep->amount < 0)
		    wep->amount = 0;
	    }
	    /* Max */
	    else if(!strcasecmp(prop, "max"))
	    {
		wep->max = atoi(val);
		if(wep->max < 0)
		    wep->max = 0;
	    }
	    /* Power */
	    else if(!strcasecmp(prop, "power"))
	    {
		wep->power = atof(val);
		if(wep->power < 0.0)
		    wep->power = 0.0;
	    }
	    /* Range */
	    else if(!strcasecmp(prop, "range"))
	    {
		wep->range = atol(val);		/* In screen units */
		if(wep->range < 0l)
		    wep->range = 0l;
	    }
	    /* Create power */
	    else if(!strcasecmp(prop, "create_power"))
	    {
		wep->create_power = atof(val);
		if(wep->create_power < 0.0)
		    wep->create_power = 0.0;
	    }
	    /* Delay */
	    else if(!strcasecmp(prop, "delay"))
	    {
		wep->delay = atol(val);		/* Milliseconds */
		if(wep->delay < 1l)
		    wep->delay = 1l;
	    }
	    /* Last used */
	    else if(!strcasecmp(prop, "last_used"))
	    {
		wep->last_used = atol(val);	/* Milliseconds */
		if(wep->last_used < 0l)
		    wep->last_used = 0l;
	    }
	    /* Fire SndRef */
	    else if(!strcasecmp(prop, "fire_sndref") ||
		    !strcasecmp(prop, "fire_sound_code")
	    )
	    {
		wep->fire_sound_code = atoi(val);
		if(wep->fire_sound_code < 0)
		    wep->fire_sound_code = 0;
	    }

	    /* Schedual need to update weapon values */
	    next.need_weapon_values = 1;
	}
	/* *********************************************************** */
	/* Economy: Flags */
	else if(!strcmp(prop, "eco.flags") ||
		!strcmp(prop, "eco/flags")
	)
	{
	    char *buf;

	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    if(UNVAllocEco(obj))
		return(-1);

	    eco = obj->eco;

	    if(*val == '!')
	    {
		/* Remove flag */
		const char *s = val;
		while((*s == '!') || ISBLANK(*s))
		    s++;

		if(!strcasecmp(s, ECO_FLAG_NAME_OPEN))
		    eco->flags &= ~(ECO_FLAG_OPEN);
		else if(!strcasecmp(s, ECO_FLAG_NAME_BUY_OK))
		    eco->flags &= ~(ECO_FLAG_BUY_OK);
		else if(!strcasecmp(s, ECO_FLAG_NAME_SELL_OK))
		    eco->flags &= ~(ECO_FLAG_SELL_OK);
		else if(!strcasecmp(s, ECO_FLAG_NAME_TRADE_OK))
		    eco->flags &= ~(ECO_FLAG_TRADE_OK);
		else if(!strcasecmp(s, ECO_FLAG_NAME_INTRODUCE_OK))
		    eco->flags &= ~(ECO_FLAG_INTRODUCE_OK);
	    }
	    else
	    {
		/* Set flag */
		const char *s = val;

		if(!strcasecmp(s, ECO_FLAG_NAME_OPEN))
		    eco->flags |= ECO_FLAG_OPEN;
		else if(!strcasecmp(s, ECO_FLAG_NAME_BUY_OK))
		    eco->flags |= ECO_FLAG_BUY_OK;
		else if(!strcasecmp(s, ECO_FLAG_NAME_SELL_OK))
		    eco->flags |= ECO_FLAG_SELL_OK;
		else if(!strcasecmp(s, ECO_FLAG_NAME_TRADE_OK))
		    eco->flags |= ECO_FLAG_TRADE_OK;
		else if(!strcasecmp(s, ECO_FLAG_NAME_INTRODUCE_OK))
		    eco->flags |= ECO_FLAG_INTRODUCE_OK;
	    }

	    buf = (char *)malloc(
		STRLEN(ECO_FLAG_NAME_OPEN) +
		STRLEN(ECO_FLAG_NAME_BUY_OK) +
		STRLEN(ECO_FLAG_NAME_SELL_OK) +
		STRLEN(ECO_FLAG_NAME_TRADE_OK) +
		STRLEN(ECO_FLAG_NAME_INTRODUCE_OK) +
		80
	    );
	    sprintf(
		buf,
		"%s%s%s%s%s",
		(eco->flags & ECO_FLAG_OPEN) ?
		    ECO_FLAG_NAME_OPEN " " : "",
		(eco->flags & ECO_FLAG_BUY_OK) ?
		    ECO_FLAG_NAME_BUY_OK " " : "",
		(eco->flags & ECO_FLAG_SELL_OK) ?
		    ECO_FLAG_NAME_SELL_OK " " : "",  
		(eco->flags & ECO_FLAG_TRADE_OK) ?
		    ECO_FLAG_NAME_TRADE_OK " " : "",  
		(eco->flags & ECO_FLAG_INTRODUCE_OK) ?
		    ECO_FLAG_NAME_INTRODUCE_OK " " : ""
	    );
	    strncpy(val, buf, sizeof(val));
	    val[sizeof(val) - 1] = '\0';
	    free(buf);

	    NetSendEcoSetValues(condescriptor, obj_num);
	}
	/* Economy: Tax general */
	else if(!strcmp(prop, "eco.tax_general") ||
		!strcmp(prop, "eco/tax_general")
	)
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    if(UNVAllocEco(obj))
		return(-1);
	    else
		eco = obj->eco;


	    eco->tax_general = atof(val);
	    if(eco->tax_general < 0.0)
		eco->tax_general = 0.0;

	    NetSendEcoSetValues(condescriptor, obj_num);
	}
	/* Economy: Tax friend */
	else if(!strcmp(prop, "eco.tax_friend") ||
		!strcmp(prop, "eco/tax_friend")
	)
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    if(UNVAllocEco(obj))
		return(-1);
	    else
		eco = obj->eco;


	    eco->tax_friend = atof(val);
	    if(eco->tax_friend < 0.0)
		eco->tax_friend = 0.0;

	    NetSendEcoSetValues(condescriptor, obj_num);
	}
	/* Economy: Tax hostile */
	else if(!strcmp(prop, "eco.tax_hostile") ||
		!strcmp(prop, "eco/tax_hostile")
	)   
	{
	    /* Check if object's permission allows set */
	    if(!ALLOW_SET_MAC(condescriptor, con_obj->permission.uid))
		return(-1);

	    if(UNVAllocEco(obj))
		return(-1);
	    else
		eco = obj->eco;


	    eco->tax_hostile = atof(val);
	    if(eco->tax_hostile < 0.0)
		eco->tax_hostile = 0.0;

	    NetSendEcoSetValues(condescriptor, obj_num);
	}



	/* Unknown property */
	else
	{
	    sprintf(
		sndbuf,
		"%s: Unsupported property: `%s'.",
		THIS_CMD_NAME,
		prop
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}



	/* Print response and log */
	strncpy(
	    tmp_name1, 
	    DBGetFormalNameStr(con->object_num),
	    sizeof(tmp_name1)
	);
	tmp_name1[sizeof(tmp_name1) - 1] = '\0';
	
	strncpy(
	    tmp_name2,
	    DBGetFormalNameStr(obj_num),
	    sizeof(tmp_name2)
	);
	tmp_name2[sizeof(tmp_name2) - 1] = '\0';
 
	sprintf(
	    sndbuf,
	    "%s: %s: Property `%s' to value `%s'.",
	    THIS_CMD_NAME,
	    tmp_name2,
	    prop,
	    val
	);
	NetSendLiveMessage(condescriptor, sndbuf);

	/* Log property setting */
	sprintf(
	    sndbuf,
	    "%s: Set %s: Property: `%s'  Value: `%s'",
	    tmp_name1,
	    tmp_name2,
	    prop,
	    val
	);
	if(sysparm.log_general == 1)
	    LogAppend(fname.log, sndbuf);

	return(0);
}


int CmdSet(int condescriptor, const char *arg)
{
	char larg[CS_DATA_MAX_LEN];
	char sndbuf[CS_DATA_MAX_LEN];

	/* Copy argument */
	strncpy(larg, arg, sizeof(larg));
	larg[sizeof(larg) - 1] = '\0';
	strstrip(larg);

	/* Print usage? */
	if((*larg == '\0') || (*larg == '?'))
	{
	    sprintf(
		sndbuf,
		"Usage: `%s <object>=<property>:<value>'",
		THIS_CMD_NAME
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}

	/* If there is no ':' deliminator then set as an OPM */
	if(strchr(larg, ':') == NULL)
	{
	    return(CmdOPMSet(condescriptor, larg));
	}
	else
	{
	    return(CmdDoSet(condescriptor, larg));
	}
}
