#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>

#include "net.h"
#include "sysparm.h"
#include "swserv.h"


static void PRINT_USAGE(int condescriptor);
int CmdShutdown(int condescriptor, const char *arg);


#define THIS_CMD_NAME	"shutdown"


static void PRINT_USAGE(int condescriptor)
{
	char sndbuf[CS_DATA_MAX_LEN];
	sprintf(
	    sndbuf,
"Usage: `%s <seconds|now|abort>'",
	    THIS_CMD_NAME
	);
	NetSendLiveMessage(condescriptor, sndbuf);
}

/*
 *	Scheduals or aborts a server shutdown.
 */
int CmdShutdown(int condescriptor, const char *arg)
{
	int con_obj_num;
	time_t t = 0l;
	char sndbuf[CS_DATA_MAX_LEN];
	xsw_object_struct *con_obj;
	sws_connection_struct *con = connection[condescriptor];

	/* Get connection's object number (assumed valid) */
	con_obj_num = con->object_num;
	con_obj = xsw_object[con_obj_num];

	/* Check the connection's permission */
	if(con_obj->permission.uid > SWS_ACCESS_UID_SHUTDOWN)
	{
	    sprintf(
		sndbuf,
		"%s: Permission denied: Requires access level %i.",
		THIS_CMD_NAME, SWS_ACCESS_UID_SHUTDOWN
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}

	if((arg != NULL) ? (*arg == '\0') : 1)
	{
	    PRINT_USAGE(condescriptor);
	    return(0);
	}

	/* Specified argument is in seconds? */
	if(isdigit(*arg))
	{
	    t = atol(arg);
	}
	else
	{
	    while(*arg == '-')
		arg++;

	    /* Shutdown now? */
	    if(toupper(*arg) == 'N')
	    {
		t = 0l;
	    }
	    /* Abort shutdown? */
	    else if((toupper(*arg) == 'A') || (toupper(*arg) == 'C'))
	    {
		/* Shutdown in process? */
		if(next.shutdown > 0l)
		{
		    /* Cancel shutdown */
		    next.shutdown = 0l;

		    sprintf( 
			sndbuf,
"*** Server shutdown aborted. ***"
		    );
		    NetSendLiveMessage(-1, sndbuf);

		    if(sysparm.log_general)  
		    {
			char text[SW_OBJ_NAME_MAX + 80];
			sprintf(
			    text,
"%s: Aborted server shutdown.",
			    DBGetFormalNameStr(con_obj_num)
			);
			LogAppend(fname.log, text);
		    }
		}
		else
		{
		    sprintf(
			sndbuf,
"There is no server shutdown schedualed."
		    );
		    NetSendLiveMessage(condescriptor, sndbuf);
		}
		return(0);
	    }
	    else
	    {
		/* Unsupported argument */
		PRINT_USAGE(condescriptor);
		return(-2);
	    }
	}

	/* Shutdown already schedualed? */
	if(next.shutdown > 0l)
	{
            sprintf(
                sndbuf,
"There is already a server shutdown schedualed, type `%s abort' to abort.",
		THIS_CMD_NAME
	    );
            NetSendLiveMessage(condescriptor, sndbuf);
	}

	/* Log who requested shutdown */
	if(sysparm.log_general)
	{
	    char text[SW_OBJ_NAME_MAX + 80];
	    sprintf(
	        text,
"%s: Requesting server shutdown.",
	        DBGetFormalNameStr(con_obj_num)
	    );
	    LogAppend(fname.log, text);
	}

	/* Schedual shutdown */
	SWServShutdownStart(t);

	return(0);
}
