#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../include/string.h"

#include "net.h"
#include "sysparm.h"
#include "swserv.h"


int CmdSignup(int condescriptor, const char *arg);


#define THIS_CMD_NAME	"signup"


int CmdSignup(int condescriptor, const char *arg)
{
	int i, con_obj_num, opm_num;
	const char *name;
	char    *s,
		sndbuf[CS_DATA_MAX_LEN],
		player_name[SW_OBJ_NAME_MAX],
		vessel_name[SW_OBJ_NAME_MAX];
	xsw_object_struct *con_obj, *opm_ptr, *tar_obj;
	sws_connection_struct *con = connection[condescriptor];

	con_obj_num = con->object_num;
	con_obj = xsw_object[con_obj_num];

	/* Signup allowed? */
	if(!sysparm.allow_guest_signup)
	{
	    sprintf(
		sndbuf,
"%s signups are not allowed.",
		sysparm.guest_login_name
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}

	/* Print usage? */
	if((arg != NULL) ? (*arg == '\0') : 1)
	{
	    int nvessels = 0;

	    sprintf(
		sndbuf,
		"Usage: `%s <name>=<vessel_type>'",
		THIS_CMD_NAME
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);

	    /* Print list of vessels */
	    for(i = 0; i < total_opms; i++)
	    {
		opm_ptr = opm[i];
		if(opm_ptr == NULL)
		    continue;

		if(opm_ptr->type != SW_OBJ_TYPE_PLAYER)
		    continue;

		name = opm_ptr->name;

		/* Skip new player and guest opms */
		if(!strcmp(name, SWS_DEF_NEW_PLAYER_OPM_NAME) ||
		   !strcmp(name, sysparm.guest_login_name)
		)
		    continue;

		if(nvessels == 0)
		{
		    sprintf(
			sndbuf,
			"Available vessel types:"
		    );
		    NetSendLiveMessage(condescriptor, sndbuf);
		}
		sprintf(
		    sndbuf,
		    "    %s",
		    name
		);
		NetSendLiveMessage(condescriptor, sndbuf);
		nvessels++;
	    }
	    if(nvessels > 0)
	    {
		sprintf(
		    sndbuf,
		    "%i vessel%s listed",
		    nvessels,
		    (nvessels == 1) ? "" : "s"
		);
		NetSendLiveMessage(condescriptor, sndbuf);
	    }
	    return(-1);
	}

	/* Already signed up? */
	if(!(con->flags & CON_FLAG_IS_GUEST))
	{
	    sprintf(
		sndbuf,
"You are already signed up."
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}

	/* Parse player and vessel name */
	strncpy(player_name, arg, sizeof(player_name));
	player_name[sizeof(player_name) - 1] = '\0';
	s = strchr(player_name, '=');
	if(s != NULL)
	    *s = '\0';
	strstrip(player_name);

	while((*arg != '=') && (*arg != '\0'))
	    arg++;
	if(*arg == '=')
	    arg++;
	strncpy(vessel_name, arg, sizeof(vessel_name));
	vessel_name[sizeof(vessel_name) - 1] = '\0';
	s = strchr(vessel_name, '=');
	if(s != NULL)
	    *s = '\0';
	strstrip(vessel_name);


	/* Check if the name is valid for a player to have */
	if(DBValidateObjectName(player_name))
	{
	    sprintf(
		sndbuf,
"The specified name is not valid for a player's name."
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}
	/* The player name may also not have the guest name prefix */
	if(strpfx(player_name, sysparm.guest_login_name))
	{
            sprintf(
                sndbuf,
"The specified name is not valid for a player's name."
            );
            NetSendLiveMessage(condescriptor, sndbuf);
            return(-1);
	}

	/* Check if the player name already exists */
	for(i = 0; i < total_objects; i++)
	{
	    tar_obj = xsw_object[i];
	    if(tar_obj == NULL)
		continue;
	    if(tar_obj->type != SW_OBJ_TYPE_PLAYER)
		continue;

	    if(!strcmp(tar_obj->name, player_name))
	    {
		sprintf(
		    sndbuf,
"%s: An object by that name already exists.",
		    player_name
		);
		NetSendLiveMessage(condescriptor, sndbuf);
		return(-1);
	    }
	}

	/* Get the OPM */
	for(i = 0; i < total_opms; i++)
	{
	    opm_ptr = opm[i];
	    if(opm_ptr == NULL)
		continue;

	    if(opm_ptr->type != SW_OBJ_TYPE_PLAYER)
		continue;

	    name = opm_ptr->name;

	    /* Skip new player and guest opms */
	    if(!strcmp(name, SWS_DEF_NEW_PLAYER_OPM_NAME) ||
	       !strcmp(name, sysparm.guest_login_name)
	    )
		continue;

	    if(!strcmp(vessel_name, name))
		break;
	}
	if(i < total_opms)
	{
	    opm_num = i;
	}
	else
	{
	    sprintf(
		sndbuf,
"%s: No such vessel (type `%s' with no arguments for a list).",
		vessel_name,
		THIS_CMD_NAME
	    );
	    NetSendLiveMessage(condescriptor, sndbuf);
	    return(-1);
	}

	/* Got valid player name and vessel OPM
	 *
	 * Model the guest connection's object to the vessel OPM
	 */
	if(con_obj != NULL)
	{
	    int wep_num;
	    const double shield_frequency = con_obj->shield_frequency;
	    const int com_channel = con_obj->com_channel;

	    if(OPMModelObject(con_obj_num, opm_num))
	    {
		sprintf(
		    sndbuf,
"%s: Unable to model vessel `%s'.",
		    THIS_CMD_NAME, vessel_name
		);
		NetSendLiveMessage(condescriptor, sndbuf);
		return(-1);
	    }

	    con_obj->shield_frequency = shield_frequency;
	    con_obj->com_channel = com_channel;
	    con_obj->owner = con_obj_num;

	    /* Update object values to connections */
	    NetSendFObjectSect(-1, con_obj_num);
	    NetSendObjectForcePose(-1, con_obj_num);
	    NetSendObjectMaximums(-1, con_obj_num);
	    for(wep_num = 0; wep_num <= con_obj->total_weapons; wep_num++)
		NetSendWeaponValues(-1, con_obj_num, wep_num);
	    NetSendScore(-1, con_obj_num);
	}

	/* Successfully modeled the guest connection's object to
	 * the selected vessel OPM
	 *
	 * Set the connection as no longer being a guest connection
	 * and the player's anme
	 */
	con->flags &= ~CON_FLAG_IS_GUEST;

	strncpy(con_obj->name, player_name, sizeof(con_obj->name));
	con_obj->name[sizeof(con_obj->name) - 1] = '\0';
	NetSendObjectName(-1, con_obj_num);

	free(con_obj->password);
	con_obj->password = NULL;

	/* Notify connection about successful signup */
	sprintf(
	    sndbuf,
"Successfully signed up `%s' as vessel `%s'.",
	    player_name, vessel_name
	);
	NetSendLiveMessage(condescriptor, sndbuf);

	/* Log property setting */
	if(sysparm.log_general == 1)
	{
	    char s[256 + SW_OBJ_NAME_MAX + SW_OBJ_NAME_MAX];
	    sprintf(
		s,
"%s: Signed up as vessel `%s'",
		DBGetFormalNameStr(con_obj_num),
		vessel_name
	    );
	    LogAppend(fname.log, s);
	}

	return(0);
}
