#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "swserv.h"


int ExportConList(const char *path);
int ExportEvents(
	const char *path,
	int type, const char *data
);
int ExportScores(const char *path);


#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)	(((s) != NULL) ? strdup(s) : NULL)

#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : 1)


/*
 *	Exports list of connected players.
 */
int ExportConList(const char *path)
{
        FILE *fp;
        int i;
        sws_connection_struct *con;

        if(STRISEMPTY(path))
            return(0);

        fp = fopen(path, "wb");
        if(fp == NULL)
            return(-1);

        /* Header */
        fprintf(
	    fp,
"player;duration;residency;client\n"
        );

        /* Go through connections list */
        for(i = 0; i < total_connections; i++)
        {
            con = connection[i];
            if(con == NULL)
                continue;

            if(con->socket < 0)
                continue;

	    if(con->object_num < 0)
		continue;

            fprintf(
		fp,
"%s;%ld;%i;%i\n",
		DBGetFormalNameStr(con->object_num),
                cur_systime - con->contime,
                con->flags & CON_FLAG_IS_GUEST,
		con->client_type
            );
        }

        fclose(fp);

	return(0);
}

/*
 *	Exports a new event, a line appended to the events file.
 */
int ExportEvents(
	const char *path,
	int type, const char *data
)
{
	FILE *fp;

        if(STRISEMPTY(path))
	    return(0);

        fp = fopen(path, "ab");
        if(fp == NULL)
            return(-1);

	fprintf(
	    fp,
"%ld;%i;%s\n",
	    cur_systime,
	    type,
	    (data != NULL) ? data : ""
	);

	fclose(fp);

	return(0);
}


/*
 *	Exports scores of each player object that has scores.
 */
int ExportScores(const char *path)
{
	int i;
	FILE *fp;
	xsw_score_struct *score;	
	xsw_object_struct *obj;

	if(STRISEMPTY(path))
            return(0);

	fp = fopen(path, "wb");
	if(fp == NULL)
	    return(-1);

	/* Header */
	fprintf(
	    fp,
"player;empire;damage_given;damage_recieved;kills;credits;rmu;rmu_max;last_connected\n"
	);

	/* Iterate through objects list */
	for(i = 0; i < total_objects; i++)
	{
	    obj = xsw_object[i];
	    if(obj == NULL)
		continue;

	    /* Skip non-player objects */
	    if(obj->type != SW_OBJ_TYPE_PLAYER)
		continue;

	    score = obj->score;
	    if(score == NULL)
		continue;

            fprintf(
		fp,
 "%s;%s;%.2f;%.2f;%i;%.2f;%.2f;%.2f;%ld\n",
		obj->name,
		obj->empire,
		score->damage_given,
		score->damage_recieved,
		score->kills,
		score->credits,
		score->rmu,
		score->rmu_max,
		(ConGetByObject(i) > -1) ? 1l : obj->last_connected
	    );
	}

	fclose(fp);

	return(0);
}
