#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "swserv.h"
#include "../include/auxstatcodes.h"


static char *LogGetCurrentTimeString(time_t t);
void LogAppend(const char *filename, const char *msg);


#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)	(((s) != NULL) ? strdup(s) : NULL)

#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : 1)


/*
 *	Returns a formatted time string of the specified time.
 */
static char *LogGetCurrentTimeString(time_t t)
{
	static char s[256];
	size_t len;

	/* Get current time */
	struct tm *tm_ptr = localtime(&t);
	if(tm_ptr == NULL)
	    return("");

	/* Format time string */
	len = strftime(
	    s, sizeof(s),
	    "%Y %b %e %H:%M",
	    tm_ptr
	);
	if(len >= sizeof(s))
	    len = sizeof(s) - 1; 
	if(len < 0)
	    len = 0;
	s[len] = '\0';

	return(s);
}

/*
 *	Appends the specified message to the specified log file.
 *
 *	A string describing the current time will automatically be
 *	prefixed to the specified message.
 *
 *	The message will also be sent to any AUX connections.
 */
void LogAppend(const char *filename, const char *msg)
{
	time_t cur_sec = cur_systime;
	const char *time_s;
	FILE *fp;

	if(STRISEMPTY(msg))
	    return;

	fp = fopen(filename, "ab");
	if(fp == NULL)
	    return;

	/* Get current time string */
	time_s = LogGetCurrentTimeString(cur_sec);

	/* Print log line to file */
	fprintf(fp, "%s: %s\n", time_s, msg);

	/* Close file */
	fclose(fp);


	/* Any AUX connections to send to? */
	if(total_aux_connections > 0)
	{
	    int i;
	    sws_aux_connection_struct *aux_con;
	    size_t len = strlen(msg) + strlen(time_s) +
		strlen(STAT_PFX_MESSAGE) + 10;
	    char *buf = (char *)malloc(len);
	    if(buf == NULL)
		return;

	    /* Format the message for the AUX connections */
	    sprintf(
		buf,
		"%s: %s: %s\n",
		STAT_PFX_MESSAGE,
		time_s,
		msg
	    );
	    for(i = 0; i < total_aux_connections; i++)
	    {
	        aux_con = aux_connection[i];
	        if(aux_con == NULL)
		    continue;

	        if(!aux_con->logged_in || (aux_con->socket < 0))
		    continue;

	        AUXConSend(aux_con, buf);
	    }

	    free(buf);
	}
}
