#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../include/unvmatch.h"
#include "../include/swnetcodes.h"

#include "net.h"
#include "sysparm.h"
#include "swserv.h"


int NetHandleLiveMessage(int condescriptor, const char *mesg);
int NetSendLiveMessage(int condescriptor, const char *mesg);
int ConNotify(int condescriptor, const char *mesg);


#define ATOI(s)         (((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)         (((s) != NULL) ? atol(s) : 0)
#define ATOF(s)         (((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)       (((s) != NULL) ? strdup(s) : NULL)

#define MAX(a,b)        (((a) > (b)) ? (a) : (b))
#define MIN(a,b)        (((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)     (MIN(MAX((a),(l)),(h)))
#define STRLEN(s)       (((s) != NULL) ? strlen(s) : 0)
#define STRISEMPTY(s)   (((s) != NULL) ? (*(s) == '\0') : 1)


int NetHandleLiveMessage(int condescriptor, const char *mesg)
{
	int obj_num;
	char name[SW_OBJ_NAME_MAX], lmesg[CS_MESG_MAX], *sndbuf;
	sws_connection_struct *con;

        if(!ConIsLoggedIn(condescriptor))
            return(-1);

	con = connection[condescriptor];
        obj_num = con->object_num;
        if(DBIsObjectGarbage(obj_num))
	    return(-1);

        if(STRISEMPTY(mesg))
	    return(0);

        strncpy(lmesg, mesg, sizeof(lmesg));
        lmesg[sizeof(lmesg) - 1] = '\0';

	strncpy(name, DBGetFormalNameStr(obj_num), sizeof(name));
	name[sizeof(name) - 1] = '\0';

	sndbuf = (char *)malloc(
	    STRLEN(lmesg) + STRLEN(name) + 80
	);
	if(sndbuf == NULL)
	    return(-3);
        sprintf(
	    sndbuf,
            "%s: \"%s\"",
            name,
            lmesg
        );
        NetSendLiveMessage(-1, sndbuf);
        free(sndbuf);

	/* Log */
	if(sysparm.log_events)
	{
	    char *s = (char *)malloc(
		STRLEN(lmesg) + STRLEN(name) + 80
	    );
	    sprintf(
		s,
		"%s: Global Message: \"%s\"",
		name, lmesg
	    );
            LogAppend(fname.log, s);
	    free(s);
	}

        return(0);
}


int NetSendLiveMessage(int condescriptor, const char *mesg)
{
        char lmesg[CS_MESG_MAX], *sndbuf;

        if(mesg == NULL)
	    return(-1);

        strncpy(lmesg, mesg, sizeof(lmesg));
        lmesg[sizeof(lmesg) - 1] = '\0';

	sndbuf = (char *)malloc(
	    STRLEN(lmesg) + 80
	);
	if(sndbuf == NULL)
	    return(-3);

        /* CS_CODE_LIVEMESSAGE format:
         *
         *	mesg
         */
        sprintf(
	    sndbuf,
	    "%i %s\n",
	    CS_CODE_LIVEMESSAGE,
	    lmesg
        );
        NetDoSend(condescriptor, sndbuf);
	free(sndbuf);

	return(0);
}

int ConNotify(int condescriptor, const char *mesg)
{
	return(NetSendLiveMessage(condescriptor, mesg));
}
