#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../include/unvmatch.h"
#include "../include/swnetcodes.h"

#include "net.h"
#include "swserv.h"


int NetHandleEcoGetValues(int condescriptor, char *arg);

int NetSendEcoSetValues(int condescriptor, int obj_num);
int NetSendEcoSetProductValues(
	int condescriptor,
	int obj_num, int product_num
);

int NetHandleEcoBuy(int condescriptor, char *arg);
int NetHandleEcoSell(int condescriptor, char *arg);


int NetHandleEcoGetValues(int condescriptor, char *arg)
{
	int i, obj_num;
	xsw_object_struct *obj;

	/* SWEXTCMD_ECO_REQVALUES format:
	 *
	 *	obj_num
	 */
	sscanf(
	    arg,
	    "%i",
	    &obj_num
	);

	/* Is object valid? */
	if(DBIsObjectGarbage(obj_num))
	    obj = NULL;
	else
	    obj = xsw_object[obj_num];


	if((obj != NULL) ? (obj->eco != NULL) : 0)
	{
	    xsw_ecodata_struct *eco = obj->eco;

	    /* Send general eco info */
	    NetSendEcoSetValues(condescriptor, obj_num);

	    /* Send product info */
	    for(i = 0; i < eco->total_products; i++)
		NetSendEcoSetProductValues(
		    condescriptor, obj_num, i
		);

	    /* Send score for proprietor object */
	    NetSendScore(condescriptor, obj_num);
	}

	/* Send score for connection's object */
	NetSendScore(
	    condescriptor,
	    connection[condescriptor]->object_num
	);

	return(0);
}


int NetSendEcoSetValues(int condescriptor, int obj_num)
{
	char sndbuf[CS_DATA_MAX_LEN];
	xsw_object_struct *obj;
	xsw_ecodata_struct *eco;

	if(DBIsObjectGarbage(obj_num))
	    return(-1);
	else
	    obj = xsw_object[obj_num];

	/* Does object have economy data? */
	eco = obj->eco;
	if(eco == NULL)
	    return(0);

	/* SWEXTCMD_ECO_SETVALUES format:
	 *
	 *	obj_num
	 *	flags
	 *	tax_general tax_friend tax_hostile
	 */
	sprintf(
	    sndbuf,
"%i %i\
 %i %ld %.4f %.4f %.4f\n", 
	    CS_CODE_EXT,
	    SWEXTCMD_ECO_SETVALUES,
	    obj_num,
	    eco->flags,
	    eco->tax_general,
	    eco->tax_friend,
	    eco->tax_hostile
	);
	NetDoSend(condescriptor, sndbuf);

	return(0);
}

int NetSendEcoSetProductValues(
	int condescriptor,
	int obj_num,
	int product_num
)
{
	char sndbuf[CS_DATA_MAX_LEN];
	xsw_object_struct *obj;
	xsw_ecodata_struct *eco;
	xsw_ecoproduct_struct *product;

	if(DBIsObjectGarbage(obj_num))
	    return(-1);
	else
	    obj = xsw_object[obj_num];

	/* Does object have economy data? */
	eco = obj->eco;
	if(eco == NULL)
	    return(0);

	/* Is product_num valid? */
	if((product_num < 0) || (product_num >= eco->total_products))
	    return(-1);

	product = eco->product[product_num];
	if(product == NULL)
	    return(-1);

	/* SWEXTCMD_ECO_SETPRODUCTVALUES format:    
	 *
	 *	obj_num sell_price buy_price
	 *	product_amount product_max;product_name
	 */
	sprintf(
	    sndbuf,
"%i %i\
 %i %.4f %.4f %.4f %.4f;%s\n",
	    CS_CODE_EXT,
	    SWEXTCMD_ECO_SETPRODUCTVALUES,
	    obj_num,
	    product->sell_price,
	    product->buy_price,
	    product->amount,
	    product->amount_max,
	    product->name
	);
	NetDoSend(condescriptor, sndbuf);

	return(0); 
}


int NetHandleEcoBuy(int condescriptor, char *arg)
{
	char *s, larg[CS_DATA_MAX_LEN];
	int customer_obj, proprietor_obj;
	long magic_number;
	xsw_ecoproduct_struct product;

	strncpy(larg, arg, sizeof(larg));
	larg[sizeof(larg) - 1] = '\0';

	/* SWEXTCMD_ECO_BUY format:
	 *
	 *	customer_obj proprietor_obj
	 *	amount magic_number;product_name
	 */
	s = strchr(larg, CS_STRING_DELIMINATOR_CHAR);
	if(s == NULL)
	    return(-1);

	/* Clear product structure */
	memset(&product, 0x00, sizeof(xsw_ecoproduct_struct));

	strncpy(product.name, s + 1, sizeof(product.name));
	product.name[sizeof(product.name) - 1] = '\0';

	*s = '\0';

	sscanf(
	    larg,
"%i %i %lf %ld",
	    &customer_obj,
	    &proprietor_obj,
	    &product.amount,
	    &magic_number
	);

	/* Magic number must match */
	if(magic_number != ECO_EXCHANGE_MAGIC_NUMBER)
	    return(-3);

	/* Customer object must match that of connection */
	if(customer_obj != connection[condescriptor]->object_num)
	    return(-1);

	/* Do buying */
	EcoDoBuy(
	    condescriptor,
	    customer_obj,
	    proprietor_obj,
	    product
	);

	return(0);
}

int NetHandleEcoSell(int condescriptor, char *arg)
{
	char *s, larg[CS_DATA_MAX_LEN];
	int customer_obj, proprietor_obj;
	long magic_number;
	xsw_ecoproduct_struct product;
	
	strncpy(larg, arg, sizeof(larg));
	larg[ sizeof(larg) - 1] = '\0';

	/* SWEXTCMD_ECO_SELL format:
	 *
	 *	customer_obj proprietor_obj
	 *	amount magic_number;product_name
	 */
	s = strchr(larg, CS_STRING_DELIMINATOR_CHAR);
	if(s == NULL)
	    return(-1);

	/* Clear product structure */
	memset(&product, 0x00, sizeof(xsw_ecoproduct_struct));

	strncpy(product.name, s + 1, sizeof(product.name));
	product.name[sizeof(product.name) - 1] = '\0';

	*s = '\0';
	
	sscanf(
	    larg,
"%i %i %lf %ld",
	    &customer_obj,
	    &proprietor_obj,
	    &product.amount,
	    &magic_number
	);

	/* Magic number must match */
	if(magic_number != ECO_EXCHANGE_MAGIC_NUMBER)
	    return(-3);

	/* Customer object must match that of connection */
	if(customer_obj != connection[condescriptor]->object_num)
	    return(-1);

	/* Do sell */
	EcoDoSell(
	    condescriptor,
	    customer_obj,
	    proprietor_obj,
	    product
	);

	return(0);
}
