#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../include/unvmatch.h"
#include "../include/unvmath.h"
#include "../include/unvutil.h"
#include "../include/swnetcodes.h"

#include "net.h"
#include "sysparm.h"
#include "swserv.h"
#include "weapons.h"

#include "../include/swexporteventscodes.h"
#include "../include/swsoundcodes.h"                              


void NetFireWeaponSyncObjectValues(
        xsw_object_struct *obj, int obj_num,
        int wep_num
);
int NetHandleFireWeapon(int condescriptor, char *arg);
int NetSendFireWeapon(int condescriptor);


#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)	(((s) != NULL) ? strdup(s) : NULL)

#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : 1)


/*
 *	Sends the object values and the weapon values to all
 *	connections within scanner range.
 */
void NetFireWeaponSyncObjectValues(
	xsw_object_struct *obj, int obj_num,
	int wep_num
)
{
	int i, con_obj_num;
	xsw_object_struct *con_obj;
	sws_connection_struct *con;

	/* Check if marked hidden from connection */
	if(obj->ext_flags & SWS_OBJ_FLAG_HIDEFROMCON)
	    return;

	for(i = 0; i < total_connections; i++)
	{
	    con = connection[i];
	    if(con == NULL)
		continue;

	    con_obj_num = con->object_num;
	    if(DBIsObjectGarbage(con_obj_num))
		continue;

	    con_obj = xsw_object[con_obj_num];

	    /* Is this connection's object out of scanner range? */
	    if(!Mu3DInRangePtr(
		con_obj, obj,
		con_obj->scanner_range
	    ))
		continue;

	    /* This connection does not own this object? */
	    if(obj->owner != con_obj_num)
	    {
		/* Is this object completely not visible? */
		if(DBGetObjectVisibilityPtr(obj) <= 0.0)
		    continue;
	    }

	    /* Send the object and weapon values to this connection */
	    NetSendObjectMaximums(i, obj_num);
	    NetSendObjectValues(i, obj_num);
	    NetSendWeaponValues(i, obj_num, wep_num);
	}
}

/*
 *	Fire weapon.
 */
int NetHandleFireWeapon(int condescriptor, char *arg)
{
	int	obj_num, con_obj_num,
		sel_wep_num, objects_created;
	long sect_x, sect_y, sect_z;
	double	x, y, z,
		heading, pitch, bank,
		velocity,
		velocity_heading, velocity_pitch, velocity_bank,
		freq, yield;
	char loc_str[256];
	xsw_object_struct *con_obj;
	xsw_weapon_struct *sel_wep;
	sws_connection_struct *con;

	if(!ConIsLoggedIn(condescriptor))
	    return(-1);

	con = connection[condescriptor];
	con_obj_num = con->object_num;
	if(DBIsObjectGarbage(con_obj_num))
	    return(-1);

	con_obj = xsw_object[con_obj_num];
 

	/* SWEXTCMD_FIREWEAPON format:
	 *
	 *	obj_num
	 *	sect_x sect_y sect_z
	 *	x y z
	 *	heading pitch bank
	 *	velocity
	 *	velocity_heading velocity_pitch velocity_bank
	 *	freq yield
	 */
	sscanf(
	    arg,
"%i\
 %ld %ld %ld\
 %lf %lf %lf\
 %lf %lf %lf\
 %lf\
 %lf %lf %lf\
 %lf %lf",
	    &obj_num,
	    &sect_x, &sect_y, &sect_z,
	    &x, &y, &z,
	    &heading, &pitch, &bank,
	    &velocity,
	    &velocity_heading, &velocity_pitch, &velocity_bank,
	    &freq, &yield
	);

	/* Connection does not own object? */
	if(obj_num != con_obj_num)
	    return(-3);

	/* Update the position of the object that is firing the
	 * weapon
	 */
	con_obj->sect_x = sect_x;
	con_obj->sect_y = sect_y;
	con_obj->sect_z = sect_z;
	con_obj->x = x;
	con_obj->y = y;
	con_obj->z = z;
#if 0
/* Do not set object's attitude, the attitude is for the firing
 * direction of the weapon
 */
	con_obj->heading = heading;
	con_obj->pitch = pitch;
	con_obj->bank = bank;
	MuSetUnitVector2D(
	    &con_obj->attitude_vector_compoent,
	    con_obj->heading
	);
#endif
	con_obj->velocity = velocity;
	con_obj->velocity_heading = velocity_heading;
	con_obj->velocity_pitch = velocity_pitch;
	con_obj->velocity_bank = velocity_bank;
	MuSetUnitVector2D(
	    &con_obj->momentum_vector_compoent,
	    con_obj->velocity_heading
	);

	/* Sanitize weapon power yield */
	if(yield < 0.0)
	    yield = 0.0;
	else if(yield > 1.0)
	    yield = 1.0;


	/* Get the selected weapon */
	sel_wep_num = con_obj->selected_weapon;
	if((sel_wep_num < 0) || (sel_wep_num >= con_obj->total_weapons))
	    return(-1);

	sel_wep = con_obj->weapons[sel_wep_num];
	if(sel_wep == NULL)
	    return(-1);


	/* Did we wait the minimum delay before we can fire again? */ 
	if((sel_wep->delay + sel_wep->last_used) >=
	    cur_millitime
	)
	    return(0);

	/* Is the object cloaked and the weapon may not be used
	 * while cloaked?
	 */
	if((con_obj->cloak_state == CLOAK_STATE_UP) &&
           !(sel_wep->flags & SW_WEP_FLAG_FIRE_CLOAKED)
	)
	    return(0);
 
	 
	/* If weapon is expendable then there is any left */
	if(sel_wep->emission_type == SW_WEP_EMISSION_PROJECTILE)
	{
	    if(sel_wep->amount <= 0)
		return(0);
	}

	/* Does the object have enough power to fire weapon? */
	if(con_obj->power < (sel_wep->create_power * yield))
	    return(0); 


	/* Fire weapon by emission type */	    
	/* Stream weapons */
	if(sel_wep->emission_type == SW_WEP_EMISSION_STREAM)
	{
	    objects_created = WeaponsFire(
		sel_wep->ocs_code,		/* OCS */
		con_obj_num,			/* Owner */
		sel_wep->emission_type,		/* Emission type */
		SWS_MAX_OCS_COPPIES,		/* Create max */
		con_obj->x,
		con_obj->y,
		heading,
		pitch,
		sel_wep->power * yield,
		sel_wep->range,
		freq,
		sel_wep->flags
	    );
	}
	/* Projectile */
	else if(sel_wep->emission_type == SW_WEP_EMISSION_PROJECTILE)
	{
	    objects_created = WeaponsFire(
		sel_wep->ocs_code,		/* OCS */
		con_obj_num,			/* Owner */
		sel_wep->emission_type,		/* Emission type */
		sel_wep->amount,		/* Create max */
		con_obj->x,
		con_obj->y,
		heading,
		pitch,
		sel_wep->power * yield,
		sel_wep->range,
		freq,
		sel_wep->flags
	    );
	}
	/* All else assume pulse */
	else
	{
	    objects_created = WeaponsFire(
		sel_wep->ocs_code,		/* OCS */
		con_obj_num,			/* Owner */
		sel_wep->emission_type,		/* Emission type */
		SWS_MAX_OCS_COPPIES,		/* Create max */
		con_obj->x,
		con_obj->y,
		heading,
		pitch,
		sel_wep->power * yield,
		sel_wep->range,
		freq,
		sel_wep->flags
	    );
	}


	/* Were any objects created? */
	if(objects_created <= 0)
	    return(objects_created);


	/* Format location string */            
	UNVLocationFormatString(
	    loc_str,
	    &con_obj->sect_x, &con_obj->sect_y, &con_obj->sect_z,
	    &con_obj->x, &con_obj->y, &con_obj->z,
	    sizeof(loc_str)
	);


	/* Log weapons fire */
	if(sysparm.log_events)
	{
	    const char	*con_obj_name = DBGetFormalNameStr(con_obj_num),
			*wep_name = DBGetOCSOPMName(sel_wep->ocs_code);
	    char *s = (char *)malloc(
		STRLEN(con_obj_name) + STRLEN(wep_name) +
		40 + STRLEN(loc_str) + 128
	    );
	    sprintf(
		s,
"%s: Fired weapon `%s' (ocs %i) at %s",
		DBGetFormalNameStr(con_obj_num),
		DBGetOCSOPMName(sel_wep->ocs_code),
		sel_wep->ocs_code,
		loc_str
	    );
	    LogAppend(fname.log, s);
	    free(s);
	}

	/* Mark the last time it was fired */
	sel_wep->last_used = cur_millitime;


	/* Consume power that was needed to create this weapon */
	switch(sel_wep->emission_type)
	{
	  case SW_WEP_EMISSION_PROJECTILE:
	    con_obj->power -= sel_wep->create_power
		* objects_created * yield;

	    /* Decrease stock for projectiles */
	    sel_wep->amount -= objects_created;
	    if(sel_wep->amount < 0)
		sel_wep->amount = 0;

	    /* Play fire sound? */
	    if(!(sel_wep->flags & SW_WEP_FLAG_NO_FIRE_SOUND))
	        NetSendPlaySound(
		    condescriptor,
		    (sel_wep->fire_sound_code != 0) ?
			sel_wep->fire_sound_code :
			SOUND_CODE_FIRE_PROJECTILE,
		    1.0, 1.0
		);

	    /* Export projectile weapons fire */
	    if(1)
	    {
		char text[(3 * SW_OBJ_NAME_MAX) + 512];
		sprintf(text,
		    "%s fired %s at %s.",
		    DBGetFormalNameStr(con_obj_num),
		    DBGetOCSOPMName(sel_wep->ocs_code),
		    loc_str
		);
		ExportEvents(
		    fname.events_export,
		    EVST_TYPE_PROJECTILEWEAPON_FIRED,
		    text
		);
	    }
	    break;

	  case SW_WEP_EMISSION_PULSE:
	    con_obj->power -= sel_wep->create_power *
		objects_created * yield;

	    /* Play fire sound? */
	    if(!(sel_wep->flags & SW_WEP_FLAG_NO_FIRE_SOUND))
		NetSendPlaySound(
		    condescriptor,
		    (sel_wep->fire_sound_code != 0) ?
			sel_wep->fire_sound_code :
			SOUND_CODE_FIRE_PULSE,
		    1.0, 1.0
		);
	    break;

	  default:	/* SW_WEP_EMISSION_STREAM */
	    con_obj->power -= sel_wep->create_power *
		objects_created * yield;

	    /* Play fire sound */
	    if(!(sel_wep->flags & SW_WEP_FLAG_NO_FIRE_SOUND))
		NetSendPlaySound(
		    condescriptor,
		    (sel_wep->fire_sound_code != 0) ?
			sel_wep->fire_sound_code :
			SOUND_CODE_FIRE_STREAM,
		    1.0, 1.0
		);
	    break;
	}

	/* Sanitize power on object */
	if(con_obj->power < 0.0)
	    con_obj->power = 0.0;

	/* Report new object and weapons values to objects nearby */
	NetFireWeaponSyncObjectValues(
	    con_obj, con_obj_num,
	    sel_wep_num
	);

	return(objects_created);
}


int NetSendFireWeapon(int condescriptor)
{
	return(0);
}
