#include <stdio.h>

#include "../include/unvmatch.h"
#include "../include/unvmath.h"
#include "../include/swnetcodes.h"

#ifdef PLUGIN_SUPPORT
#include "plugins.h"
#endif
#include "net.h"
#include "sysparm.h"
#include "swserv.h"


int NetHandleHail(int condescriptor, char *arg);
int NetSendHail(
	int condescriptor,
	int src_obj, int tar_obj,
	double bearing, int channel
);


int NetHandleHail(int condescriptor, char *arg)
{
	int i, src_obj_num, tar_obj_num, channel, tar_con_num;
	double bearing;
	xsw_object_struct *src_obj;

	/* SWEXTCMD_HAIL format:
	 *
	 *	src_obj_num tar_obj_num
	 *	bearing channel
	 */
	sscanf(
	    arg,
	    "%i %i %lf %i",
	    &src_obj_num,
	    &tar_obj_num,
	    &bearing,
	    &channel
	);

	/* Connection must control source object */
	if(connection[condescriptor]->object_num != src_obj_num)
	    return(-3);

	/* Need valid source object pointer */
	if(DBIsObjectGarbage(src_obj_num))
	    return(-1);

	src_obj = xsw_object[src_obj_num];

	for(i = 0; i < total_objects; i++)
	{
	    /* Object not valid or not in communications range? */
	    if(!Mu3DInRange(
		xsw_object, total_objects,
		src_obj_num, i, sysparm.com_range
	    ))
		continue;

	    /* Is the target object specified? */
	    if((tar_obj_num > -1) && (tar_obj_num != i) &&
	       (src_obj_num != i)
	    )
	    {
		/* Skip since the hail is not ment for this object */
		continue;
	    }

	    /* Get the connection number of this object */
	    tar_con_num = ConGetByObject(i);
	    if(tar_con_num < 0)
		continue;

	    /* Get bearing, object_count to source object */
	    bearing = MuCoordinateDeltaVector(
		src_obj->x - xsw_object[i]->x,
		src_obj->y - xsw_object[i]->y
	    );

	    /* Send hail to that connection */
	    NetSendHail(
		tar_con_num,
		src_obj_num,
		tar_obj_num,
		bearing,
		channel
	    );
	}

#ifdef PLUGIN_SUPPORT
	/* Forward this hail to plugins */
	if(total_plugins > 0)
	{
	    int i;
	    for(i = 0; i < total_plugins; i++)
		PluginHailNotify(
		    i,
		    condescriptor,
		    src_obj_num,
		    tar_obj_num,
		    channel,
		    NULL	/* No message, since just a hail */
		);
	}
#endif

	return(0);
}


int NetSendHail(
	int condescriptor,
	int src_obj, int tar_obj,
	double bearing, int channel
)
{
	char sndbuf[CS_DATA_MAX_LEN];

	/*
	 *      SWEXTCMD_HAIL format:
	 *
	 *      src_obj, tar_obj, bearing, channel
	 */
	sprintf(sndbuf,
"%i %i\
 %i %i %.4f %i\n",
		CS_CODE_EXT,
		SWEXTCMD_HAIL,
		src_obj,
		tar_obj,
		bearing,
		channel
	);
	NetDoSend(condescriptor, sndbuf);

	return(0);
}
