#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../include/string.h"
#include "../include/imgrefs.h"
#include "../include/unvmatch.h"
#include "../include/swnetcodes.h"

#include "net.h"
#include "sysparm.h"
#include "swserv.h"


int NetHandleLogin(int condescriptor, char *arg);


#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)	(((s) != NULL) ? strdup(s) : NULL)

#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : 1)


int NetHandleLogin(int condescriptor, char *arg)
{
	int i, n, obj_num, client_type;
	char	*s, *s2,
		sndbuf[CS_DATA_MAX_LEN + 256],
		name[SW_OBJ_NAME_MAX],
		password[SW_OBJ_PASSWORD_MAX],
		guest_name[SW_OBJ_NAME_MAX + 256];
	sws_connection_struct *con = connection[condescriptor];
	xsw_object_struct *obj = NULL;

	/* Already logged in? */
	if(!DBIsObjectGarbage(con->object_num))
	{
#if 0
	    NetSendLiveMessage(
		condescriptor,
		"You are already logged in."
	    );
#endif
	    return(0);
	}


	/* Print help? */
	if((arg != NULL) ? (*arg == '\0') : 1)
	{
	    /* Print correct format to connection */
	    NetSendLiveMessage(
		condescriptor,
 "Login syntax incorrect, should be `<name>;<password>'"
	    );
	    return(-2);
	}

	/* Close this connection if it gave too many bad logins */
	if(con->badlogins >= sysparm.max_failed_logins)
	{
	    /* Close connection */
	    NetCloseConnection(condescriptor);

	    /* Log exessive bad logins as errors */
	    if(sysparm.log_errors)
	    {
		char *s = (char *)malloc(40 + 40 + 80);
		sprintf(
		    s,
"Connection %i: Sent %i or more bad logins.",
		    condescriptor,
		    sysparm.max_failed_logins
		);
		LogAppend(fname.log, s);
		free(s);
	    }

	    return(-1);
	}


	/* ********************************************************** */
	/* Check if at least one deliminator exists */
	s = strchr(arg, CS_STRING_DELIMINATOR_CHAR);
	if(s == NULL)
	{
	    /* Missing deliminator character, print usage response */
	    NetSendLiveMessage(
		condescriptor,
		"Login syntax incorrect, should be `<name>;<password>'"
	    );

	    /* Send a rerequest for login */
	    sprintf(sndbuf, "%i\n", CS_CODE_LOGIN);
	    NetDoSend(condescriptor, sndbuf);

	    /* Record a bad login */
	    con->badlogins++;

	    return(-1);
	}


	/* Begin parsing */
	s = arg;

	/* Parse name */
	if(s != NULL)
	{
	    strncpy(name, s, sizeof(name));
	    name[sizeof(name) - 1] = '\0';

	    /* Remove deliminator */
	    s2 = strchr(name, CS_STRING_DELIMINATOR_CHAR);
	    if(s2 != NULL)
		*s2 = '\0';

	    strstrip(name);

	    /* Seek to next deliminator */
	    s = strchr(s, CS_STRING_DELIMINATOR_CHAR);
	    if(s != NULL)
		s++;
	}
	else
	{
	    *name = '\0';
	}

	/* Parse password */
	if(s != NULL)
	{
	    strncpy(password, s, sizeof(password));
	    password[sizeof(password) - 1] = '\0';

	    /* Remove deliminator */
	    s2 = strchr(password, CS_STRING_DELIMINATOR_CHAR);
	    if(s2 != NULL) 
		*s2 = '\0';

	    strstrip(password);

	    /* Seek to next deliminator */
	    s = strchr(s, CS_STRING_DELIMINATOR_CHAR);
	    if(s != NULL)
		s++;
	}   
	else
	{
	    *password = '\0';
	}

	/* Parse client type code */
	if(s != NULL)
	{
	    char num_str[80];

	    strncpy(num_str, s, sizeof(num_str));
	    num_str[sizeof(num_str) - 1] = '\0';

	    /* Remove deliminator */
	    s2 = strchr(num_str, CS_STRING_DELIMINATOR_CHAR);
	    if(s2 != NULL)
		*s2 = '\0';

	    strstrip(num_str);
	    client_type = atoi(num_str);

	    /* Seek to next deliminator */
	    s = strchr(s, CS_STRING_DELIMINATOR_CHAR);
	    if(s != NULL)
		s++;
	}
	else
	{
	    client_type = 0;
	}


	/* Check if login is a guest login (case insensitive) */
	if(sysparm.allow_guest &&
	   !strcasecmp(name, sysparm.guest_login_name)
	)
	{
	    int guest_opm_num;

	    /* Check if there are too many guest connections */
	    if(NetConGuests() >= sysparm.max_guests)
	    {
		/* Warn bad or missing guest OPM message */
		sprintf(
		    sndbuf,
"The maximum capacity of %i %s connections has been reached.",
		    sysparm.max_guests,
		    sysparm.guest_login_name
		);
		NetSendSysMessage(
		    condescriptor,
		    CS_SYSMESG_CODE_LOGINFAIL,
		    sndbuf
		);

		/* Close the connection */
		NetCloseConnection(condescriptor);

		return(-1);
	    }

	    /* Create a new object for the guest */
	    obj_num = DBCreateObject(
		IMGREF_DEFAULT_IMGREF,	/* ImgRef */
		SW_OBJ_TYPE_PLAYER,	/* Type */
		-1,			/* Owner */
		0l, 0l, 0l,		/* Sector */
		0.0, 0.0, 0.0,		/* x, y, z */
		0.0 * PI, 0.0 * PI, 0.0 * PI
	    );
	    if(DBIsObjectGarbage(obj_num))
	    {
		/* Send failed allocation message to connection */
		sprintf(
		    sndbuf,
"Server is out of resources to allocate a %s connection.",
		    sysparm.guest_login_name
		);
		NetSendSysMessage(
		    condescriptor,
		    CS_SYSMESG_CODE_LOGINFAIL,
		    sndbuf
		);

		/* Log */
		if(sysparm.log_errors)
		{
		    char *s = (char *)malloc(
			40 + STRLEN(name) + 80
		    );
		    sprintf(
			s,
"Connection %i: Not enough memory to login as %s.",
			condescriptor,
			name
		    );
		    LogAppend(fname.log, s);
		    free(s);
		}

		/* Close the connection */
		NetCloseConnection(condescriptor);

		return(-1);   
	    }

	    obj = xsw_object[obj_num];


	    /* Set up guest object using OPM */
	    guest_opm_num = OPMGetByName(
		sysparm.guest_login_name,
		SW_OBJ_TYPE_PLAYER
	    );
	    if(OPMIsGarbage(guest_opm_num))
	    {
		/* Warn about missing OPM */
		char *s = (char *)malloc(
		    STRLEN(sysparm.guest_login_name) + 80
		);
		sprintf(
		    s,
"Warning: OPM `%s' not defined in the OPM file.",
		    sysparm.guest_login_name
		);
		NetSendLiveMessage(condescriptor, s);
		free(s);

		/* Log */
		if(sysparm.log_errors)
		{
		    char *s = (char *)malloc(
			STRLEN(sysparm.guest_login_name) + 128
		    );
		    sprintf(
			s,
"Warning: OPM `%s' used for new guest objects is not defined in the OPM file.",
			sysparm.guest_login_name
		    );
		    LogAppend(fname.log, s);
		    free(s);
		}
	    }
	    else
	    {
		/* Model new guest object according to OPM */
		OPMModelObject(obj_num, guest_opm_num);

		/* Move guest object to guest start location */
		obj->sect_x = unv_head.guest_start_sect_x;
		obj->sect_y = unv_head.guest_start_sect_y;
		obj->sect_z = unv_head.guest_start_sect_z;

		obj->x = unv_head.guest_start_x;
		obj->y = unv_head.guest_start_y;     
		obj->z = unv_head.guest_start_z;     

		/* Set guest object to guest start direction */
		obj->heading = unv_head.guest_start_heading;     
		obj->pitch = unv_head.guest_start_pitch;
		obj->bank = unv_head.guest_start_bank;

		/* Must set guest to own itself */
		obj->owner = obj_num;
	    }

	    /* Set guest name (post fix a number to it) */
	    for(i = 0; i < sysparm.max_guests; i++)
	    {
		sprintf(
		    guest_name,
		    "%s %i",
		    sysparm.guest_login_name,
		    i + 1
		);
		/* See if another object has this exact same name */
	        for(n = 0; n < total_objects; n++)
	        {
		    if(xsw_object[n] == NULL)
		        continue;
		    if(xsw_object[n]->type <= SW_OBJ_TYPE_GARBAGE)
			continue;

		    if(!strcasecmp(xsw_object[n]->name, guest_name))
			break;
		}
		if(n >= total_objects)
		    break;	/* No other objects have this name */
	    }
	    strncpy(obj->name, guest_name, sizeof(obj->name));
	    obj->name[sizeof(obj->name) - 1] = '\0';

	    /* Remove password for guest objects */
	    free(obj->password);
	    obj->password = NULL;


	    /* Set client type and IS guest */
	    con->client_type = client_type;
	    con->flags |= CON_FLAG_IS_GUEST;
	}
	/* Check if guest login, but guest login not allowed */
	else if(!sysparm.allow_guest &&
		!strcasecmp(name, sysparm.guest_login_name)
	)
	{
	    /* Notify that guest connections are not allowed */
	    char	*s = STRDUP(sysparm.mesg_no_guests),
			*s2 = strsub(s, "%title", unv_head.title),
	    		*mesg;
	    free(s);
	    mesg = strsub(s2, "%name", sysparm.guest_login_name);
	    free(s2);
	    NetSendSysMessage(
		condescriptor,
		CS_SYSMESG_CODE_LOGINFAIL,
		mesg
	    );
	    free(mesg);

	    /* Close the connection */
	    NetCloseConnection(condescriptor);

	    return(-3);
	}
	/* Get object number for regular login */
	else
	{
	    obj_num = MatchObjectByName(
		xsw_object, total_objects,
		name, SW_OBJ_TYPE_PLAYER
	    );
	    if(DBIsObjectGarbage(obj_num))
	    {
		/* Send bad login message */
		NetSendSysMessage(
		    condescriptor,
		    CS_SYSMESG_CODE_LOGINFAIL,
		    sysparm.mesg_wrong_login
		);

		/* Resend request for login */
		sprintf(sndbuf, "%i\n", CS_CODE_LOGIN);
		NetDoSend(condescriptor, sndbuf);

		/* Log unmatched object name login as an error */
		if(sysparm.log_errors)
		{
		    char *s = (char *)malloc(40 + STRLEN(name) + 80);
		    sprintf(
			s,
"Connection %i: No player object for login name: \"%s\"",
			condescriptor,
			name
		    );
		    LogAppend(fname.log, s);
		    free(s);
		}

		con->badlogins++;	/* Count this as a bad login */

		return(-1);
	    }

	    obj = xsw_object[obj_num];


	    /* Set client type and NOT guest */  
	    con->client_type = client_type;
	    con->flags &= ~CON_FLAG_IS_GUEST;
	}


	/* Get and check password (for non guest logins) */
	if(!(con->flags & CON_FLAG_IS_GUEST))
	{
	    if(CryptHandleVerify(password, obj->password) < 1)
	    {
		/* Send bad login message */
		NetSendSysMessage(
		    condescriptor,
		    CS_SYSMESG_CODE_LOGINFAIL,
		    sysparm.mesg_wrong_login
		);

		/* Resend a request for login */
		sprintf(sndbuf, "%i\n", CS_CODE_LOGIN);
		NetSendDataToConnection(condescriptor, sndbuf, 1);

		/* Log */
		if(sysparm.log_errors)
		{
		    char *s = (char *)malloc(
			40 + SW_OBJ_NAME_MAX + STRLEN(password) + 80
		    );
		    sprintf(
			s,
"Connection %i: %s: Bad password: \"%s\"",
			condescriptor,
			DBGetFormalNameStr(obj_num),
			password
		    );
		    LogAppend(fname.log, s);
		    free(s);
		}

		/* Record a bad login */
		con->badlogins++;

		return(-1);
	    }
	}


	/* All login checks passed! */

	/* Set the connection's object number */
	con->object_num = obj_num;



	/* Allow only one connection per object? */
	if(sysparm.single_connection)
	{
	    /* Disconnect extra connections of obj_num */
	    for(i = 0; i < total_connections; i++)
	    {
		if(connection[i] == NULL)
		    continue;
		if(connection[i]->object_num < 0)
		    continue;

		/* Skip current connection */
		if(i == condescriptor)
		    continue;

		if(connection[i]->object_num == obj_num)
		{
		    sprintf(
			sndbuf,
"Disconnecting due to new login of the same object as this connection."
		    );
		    NetSendLiveMessage(i, sndbuf);

		    if(sysparm.log_errors)
		    {
			char *s = (char *)malloc(40 + 40 + 80);
			sprintf(
			    s,
"Connection %i: Disconnected due to new login of the same object #%i",
			    i,
			    obj_num
			);
			LogAppend(fname.log, s);
			free(s);
		    }

		    NetCloseConnection(i);
		}
	    }
	}

	/* Mark the object as connected and unhide it
	 *
	 * This needs to be done after the above force single
	 * connections check to ensure that the object is marked
	 * properly
	 */
	if(obj != NULL)
	{
	    obj->ext_flags |= SWS_OBJ_FLAG_CONNECTED;
	    obj->ext_flags &= ~SWS_OBJ_FLAG_HIDEFROMCON;
	    obj->last_connected = cur_systime;
	}

	/* Notify all connections about new connection */
	if(sysparm.con_notify && (obj != NULL))
	{
	    char	*s = STRDUP("%name has connected."),
			*mesg = strsub(
			    s, "%name",
			    DBIsObjectGarbage(obj_num) ?
				"*unknown*" : obj->name
			);
	    free(s);

	    for(i = 0; i < total_connections; i++)
	    {
		if(connection[i] == NULL)
		    continue;
		if(connection[i]->object_num < 0)
		    continue;
	 
		/* Skip current connection */
		if(i == condescriptor)
		    continue;

		NetSendLiveMessage(i, mesg);
	    }

	    free(mesg);
	}



	/* Log successful connection */
	if(sysparm.log_net)
	{
	    const char *name = DBGetFormalNameStr(obj_num);
	    char *s = (char *)malloc(40 + STRLEN(name) + 80);
	    sprintf(
		s,
"Connection %i: Successfully logged in as: \"%s\"",
		condescriptor,
		name
	    );
	    LogAppend(fname.log, s);
	    free(s);
	}


	/* Print welcome message */
	if(obj != NULL)
	{
	    char	*s = STRDUP(sysparm.mesg_welcome),
			*s2 = strsub(s, "%title", unv_head.title),
			*mesg;
	    free(s);
	    mesg = strsub(
		s2, "%name",
		DBIsObjectGarbage(obj_num) ? "*unknown*" : obj->name
	    );
	    free(s2);
	    NetSendLiveMessage(condescriptor, mesg);
	    free(mesg);
	}

#if 0
	sprintf(
	    sndbuf,
	    "You are now connected as %s.",
	    DBGetFormalNameStr(obj_num)
	);
	NetSendLiveMessage(condescriptor, sndbuf);
#endif

	/* Refresh connection for the first time */
	NetSendRefresh(condescriptor);

	/* Schedual stats export to now in order to export the
	 * connections list now
	 */
	next.stats_export = cur_systime;

	return(0);
}


int NetSendLogin(int condescriptor)
{
	char sndbuf[CS_DATA_MAX_LEN];

	/* CS_CODE_LOGIN format:
	 *
	 */
	sprintf(
	    sndbuf,
	    "%i\n",
	    CS_CODE_LOGIN
	);
	NetSendDataToConnection(condescriptor, sndbuf, 1);

	return(0);
}
