#include <stdio.h>

#include "../include/unvmatch.h"
#include "../include/unvmath.h"
#include "../include/swnetcodes.h"

#include "net.h"
#include "swserv.h"
 
 
int NetHandleObjectPose(int condescriptor, char *arg)
{
	int obj_num;
	int type;
	int imgref_num;
	long size;
	float x, y, z;
	float heading, pitch, bank;
	float velocity;
	float velocity_heading;
	float velocity_pitch;
	float velocity_bank;
	int current_frame;
	xsw_object_struct *obj;
	sws_connection_struct *con = connection[condescriptor];

	/* CS_CODE_POSEOBJ format:
	 *
	 *      obj_num
	 *      type imgref_num size
	 *      x y z
	 *      heading pitch
	 *      velocity velocity_heading
	 *      throttle frame
	 */
	sscanf(
	    arg,
"%i\
 %i %i %ld\
 %f %f %f\
 %f %f %f\
 %f\
 %f %f %f\
 %i",
	    &obj_num,
	    &type, &imgref_num, &size,
	    &x, &y, &z,
	    &heading, &pitch, &bank,
	    &velocity,
	    &velocity_heading, &velocity_pitch, &velocity_bank,
	    &current_frame
	);

	/* Connection must own object */         
	if(con->object_num != obj_num)
	    return(-3);
 
	if(DBIsObjectGarbage(obj_num))
	    return(-1);

	obj = xsw_object[obj_num];

#if 0
/* Do not set type, imgref, or size */
	obj->type = type;
	obj->imageset = imgref_num;
	obj->size = size;
#endif

	/* Set these only if object has enough antimatter */
	if(obj->antimatter > 0.0)
	{
	    obj->x = x;
	    obj->y = y;
	    obj->z = z;

	    obj->velocity = velocity;

	    obj->velocity_heading = velocity_heading;
	    obj->velocity_pitch = velocity_pitch;
	    obj->velocity_bank = velocity_bank;
	    MuSetUnitVector2D(
		&obj->momentum_vector_compoent,
		obj->velocity_heading
	    );
	}

	obj->heading = heading;
	obj->pitch = pitch;
	obj->bank = bank;
	MuSetUnitVector2D(
	    &obj->attitude_vector_compoent,
	    obj->heading
	);

/*
	obj->animation.current_frame = current_frame;
 */

	return(0);
}


int NetSendObjectPose(int condescriptor, int obj_num)
{
	char sndbuf[CS_DATA_MAX_LEN];
	xsw_object_struct *obj;  

	if(DBIsObjectGarbage(obj_num))
	    return(-1);

	obj = xsw_object[obj_num];

	/* CS_CODE_POSEOBJ format:
	 *
	 *      obj_num
	 *      type imgref_num size
	 *      x y z
	 *      heading pitch bank
	 *      velocity
	 *      velocity_heading velocity_pitch velocity_bank
	 *      current_frame
	 */
	sprintf(
	    sndbuf,
"%i %i\
 %i %i %ld\
 %.4f %.4f %.4f\
 %.4f %.4f %.4f\
 %.4f\
 %.4f %.4f %.4f\
 %i\n",
	    CS_CODE_POSEOBJ,
	    obj_num,
	    obj->type, obj->imageset, obj->size,
	    obj->x, obj->y, obj->z,
	    obj->heading, obj->pitch, obj->bank,
	    obj->velocity,
	    obj->velocity_heading, 0.0 * PI, 0.0 * PI,
	    obj->animation.current_frame
	);
	NetDoSend(condescriptor, sndbuf);

	return(0); 
}
