#include <stdio.h>

#include "../include/unvmatch.h"
#include "../include/swnetcodes.h"

#include "net.h"
#include "swserv.h"


int NetHandleReqObjectSect(int condescriptor, char *arg)
{
	int obj_num;

	/* SWEXTCMD_REQSECT format:
	 *
	 *	obj_num
	 */
	sscanf(
	    arg,
	    "%i",
	    &obj_num
	);

	if(DBIsObjectGarbage(obj_num))
	    return(-1);

	NetSendObjectSect(condescriptor, obj_num);

	return(0);
}


int NetHandleObjectSect(int condescriptor, char *arg)
{
	int obj_num, con_obj_num;  
	long sect_x, sect_y, sect_z;
	sws_connection_struct *con = connection[condescriptor];
	xsw_object_struct *obj;

	con_obj_num = con->object_num;

	/* SWEXTCMD_SETOBJSECT format:
	 *
	 *	obj_num
	 *	sect_x sect_y sect_z
	 */
	sscanf(
	    arg,
	    "%i %ld %ld %ld",
	    &obj_num,
	    &sect_x, &sect_y, &sect_z
	);

	if(DBIsObjectGarbage(obj_num))
	    return(-1);

	obj = xsw_object[obj_num];

	/* Connection must own object */
	if(con_obj_num != obj_num)
	    return(-3);

	/* Change the object's sector and update related resources */
	DBObjectDoSetSector(
	    obj_num,
	    sect_x, sect_y, sect_z,
	    1			/* Allow wrapping */
	);

	return(0);
}       



int NetSendObjectSect(int condescriptor, int obj_num)
{
	char sndbuf[CS_DATA_MAX_LEN];
	xsw_object_struct *obj;


	if(DBIsObjectGarbage(obj_num))
	    return(-1);
	else
	    obj = xsw_object[obj_num];


	/*
	 *   SWEXTCMD_SETOBJSECT format:
	 *      
	 *      obj_num
	 *      sect_x sect_y sect_z
	 */
	sprintf(sndbuf,
"%i %i\
 %i %ld %ld %ld\n",
		CS_CODE_EXT,
		SWEXTCMD_SETOBJSECT,

		obj_num,
		obj->sect_x,
		obj->sect_y,
		obj->sect_z
	);
	NetDoSend(condescriptor, sndbuf);


	return(0);
}


int NetSendFObjectSect(int condescriptor, int obj_num)
{
	char sndbuf[CS_DATA_MAX_LEN];
	xsw_object_struct *obj;


	if(DBIsObjectGarbage(obj_num))
	    return(-1);
	else
	    obj = xsw_object[obj_num];
	   
	  
	/*
	 *      SWEXTCMD_SETFOBJSECT format:
	 *      
	 *      obj_num
	 *      sect_x sect_y sect_z
	 */
	sprintf(sndbuf,
"%i %i\
 %i %ld %ld %ld\n",
		CS_CODE_EXT,
		SWEXTCMD_SETFOBJSECT,

		obj_num,
		obj->sect_x,
		obj->sect_y,
		obj->sect_z
	);
	NetDoSend(condescriptor, sndbuf);


	return(0);
}
