#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../include/unvmatch.h"
#include "../include/unvmath.h"
#include "../include/swnetcodes.h"

#include "net.h"
#include "sysparm.h"
#include "swserv.h"


int NetHandleRefresh(int condescriptor, char *arg) 
{
	NetSendRefresh(condescriptor);
	return(0);
}


int NetSendRefresh(int condescriptor)
{
	int i, wep_num, obj_num;
	xsw_object_struct *obj, *tar_obj;
	sws_connection_struct *con;

	/* Connection must be logged in */
	if(!ConIsLoggedIn(condescriptor))  
	    return(-1);

	con = connection[condescriptor];

	/* Get obj_num and obj */
	obj_num = con->object_num;
	if(DBIsObjectGarbage(obj_num))
	    return(-1);

	obj = xsw_object[obj_num];


	/* Send units */
	NetSendUnits(condescriptor);

	/* Send title */
	NetSendTitle(condescriptor);

	/* Send data file names */
	NetSendSetImageSet(condescriptor, unv_head.imgrefs);
	NetSendSetSoundSet(condescriptor, unv_head.sndrefs);
	NetSendSetOCSN(condescriptor, unv_head.ocsn);

	/* Send who am I */
	NetSendWhoAmI(condescriptor);

	/* Send player description */
	NetSendObjectDescription(condescriptor, obj_num);


	/* Iterate through objects list */
	for(i = 0; i < total_objects; i++)
	{
	    tar_obj = xsw_object[i];
	    if(tar_obj == NULL)
		continue;

	    /* Check objects are valid and within range */
	    if(!Mu3DInRangePtr(
		obj, tar_obj,
		obj->scanner_range
	    ))
		continue;

	    /* If object is not owned by the connection, then check
	     * the visibility of the object
	     */
	    if(tar_obj->owner != obj_num)
	    {
		if(DBGetObjectVisibilityPtr(tar_obj) <= 0.0)
		    continue;
	    }

	    /* Check if marked hidden from connection */
	    if(tar_obj->ext_flags & SWS_OBJ_FLAG_HIDEFROMCON)
		continue;

	    /* Send create object */
	    NetSendCreateObject(condescriptor, i);

	    /* Send object maximums */
	    NetSendObjectMaximums(condescriptor, i);

	    /* Send object name */
	    NetSendObjectName(condescriptor, i);

	    /* Engine state */
	    NetSendSetEngine(condescriptor, i);

	    /* Com channel */
	    NetSendSetChannel(condescriptor, i, tar_obj->com_channel);

	    /* Send tractor beam lock */
	    if(tar_obj->total_tractored_objects > 0)
	    {
		int i, m = tar_obj->total_tractored_objects;
		for(i = 0; i < m; i++)
		    NetSendTractorBeamLock(
			condescriptor, i,
			tar_obj->tractored_object[i]
		    );
	    }

	    /* If the connection's object is locked on to this object,
	     * then send weapons and current values of this object
	     */
	    if(obj->locked_on == i)
	    {
		/* Send values */
		NetSendObjectValues(condescriptor, i);
	
		/* Send weapon values */
		for(wep_num = 0; wep_num < tar_obj->total_weapons; wep_num++)
		    NetSendWeaponValues(condescriptor, i, wep_num);
	    }
	}

	/* Send weapon values of connection's object */
	for(wep_num = 0; wep_num < obj->total_weapons; wep_num++)
	    NetSendWeaponValues(condescriptor, obj_num, wep_num);

	/* Send score */
	NetSendScore(condescriptor, obj_num);

	/* Send entire star chart listing */
	if(sysparm.send_starchart)
	    NetSendEntireStarChart(condescriptor);

	return(0);
}
