#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../include/unvmatch.h"
#include "../include/swnetcodes.h"

#include "net.h"
#include "swserv.h"


int NetSendStarChartAddObject(int condescriptor, int obj_num);
int NetSendStarChartSetObjectName(int condescriptor, int obj_num);
int NetSendStarChartSetObjectEmpire(int condescriptor, int obj_num);
int NetSendStarChartRecycleObject(int condescriptor, int obj_num);
int NetSendEntireStarChart(int condescriptor);


#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)	(((s) != NULL) ? strdup(s) : NULL)

#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : 1)


/* 
 *	Sends an add object to the star chart.
 */
int NetSendStarChartAddObject(int condescriptor, int obj_num)
{
	char sndbuf[CS_DATA_MAX_LEN];
	xsw_object_struct *obj;

	if(DBIsObjectGarbage(obj_num))
	    return(-1);

	obj = xsw_object[obj_num];

	/* SWEXTCMD_STARCHART_ADD_OBJECT format:
	 *
	 *	obj_num
	 *	type imgref_num size
	 *	sect_x sect_y sect_z
	 *	x, y, z
	 *	heading, pitch, bank
	 */
	sprintf(
	    sndbuf,
"%i %i %i\
 %i %i %ld\
 %ld %ld %ld\
 %.4f %.4f %.4f\
 %.4f %.4f %.4f\n",
	    CS_CODE_EXT,
	    SWEXTCMD_STARCHART_ADD_OBJECT,
	    obj_num,
	    obj->type, obj->imageset, obj->size,
	    obj->sect_x, obj->sect_y, obj->sect_z,
	    obj->x, obj->y, obj->z,
	    obj->heading, obj->pitch, obj->bank
	);
	NetDoSend(condescriptor, sndbuf);

	return(0);
}

/*
 *	Sends a set object name on star chart.
 */
int NetSendStarChartSetObjectName(int condescriptor, int obj_num)
{
	char sndbuf[CS_DATA_MAX_LEN];
	xsw_object_struct *obj;

	if(DBIsObjectGarbage(obj_num))
	    return(-1);

	obj = xsw_object[obj_num]; 

	/* SWEXTCMD_STARCHART_SET_OBJECT_NAME format:
	 *
	 *	obj_num;name
	 */
	sprintf(
	    sndbuf,
	    "%i %i %i;%s\n",
	    CS_CODE_EXT,
	    SWEXTCMD_STARCHART_SET_OBJECT_NAME,
	    obj_num,
	    obj->name
	);
	NetDoSend(condescriptor, sndbuf);

	return(0);
}

/*
 *	Sends a set object empire on star chart.
 */
int NetSendStarChartSetObjectEmpire(int condescriptor, int obj_num)
{
	char sndbuf[CS_DATA_MAX_LEN];
	xsw_object_struct *obj;

	if(DBIsObjectGarbage(obj_num))
	    return(-1);

	obj = xsw_object[obj_num];
 
	/* SWEXTCMD_STARCHART_SET_OBJECT_EMPIRE format:
	 *
	 *	obj_num;empire
	 */     
	sprintf(
	    sndbuf,
	    "%i %i %i;%s\n",
	    CS_CODE_EXT,
	    SWEXTCMD_STARCHART_SET_OBJECT_EMPIRE,
	    obj_num, obj->empire
	);
	NetDoSend(condescriptor, sndbuf);

	return(0);
}

/*
 *	Sends a recycle object on star chart.
 */
int NetSendStarChartRecycleObject(int condescriptor, int obj_num)
{
	char sndbuf[CS_DATA_MAX_LEN];

	/* SWEXTCMD_STARCHART_SET_OBJECT_RECYCLE format:
	 *
	 *	obj_num
	 */
	sprintf(
	    sndbuf,
	    "%i %i %i\n",
	    CS_CODE_EXT,
	    SWEXTCMD_STARCHART_SET_OBJECT_RECYCLE,
	    obj_num
	);
	NetDoSend(condescriptor, sndbuf);

	return(0);
}


/*
 *	Sends the entire star chart to the specified connection.
 */
int NetSendEntireStarChart(int condescriptor)
{
	int i;
	xsw_object_struct *obj;

	for(i = 0; i < total_objects; i++)
	{
	    obj = xsw_object[i];
	    if(obj == NULL)
		continue;

	    if((obj->type != SW_OBJ_TYPE_HOME) &&
	       (obj->type != SW_OBJ_TYPE_AREA) &&
	       (obj->type != SW_OBJ_TYPE_WORMHOLE) &&
	       (obj->type != SW_OBJ_TYPE_ELINK)
	    )
		continue;

	    NetSendStarChartAddObject(condescriptor, i);
	    NetSendStarChartSetObjectName(condescriptor, i);
	    NetSendStarChartSetObjectEmpire(condescriptor, i);
	}

	return(0);
}
