#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../include/unvmatch.h"
#include "../include/swnetcodes.h"

#include "net.h"
#include "swserv.h"


int NetHandleTractorBeamLock(int condescriptor, char *arg);
int NetSendTractorBeamLock(
	int condescriptor, int obj_num, int tar_obj_num
);


#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)	(((s) != NULL) ? strdup(s) : NULL)

#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : 1)


int NetHandleTractorBeamLock(int condescriptor, char *arg)
{
	int obj_num, tar_obj_num;
	xsw_object_struct *obj, *tar_obj;
	sws_connection_struct *con = connection[condescriptor];

	/* SWEXTCMD_TRACTORBEAMLOCK format:
	 *
	 *	obj_num tar_obj_num
	 */
	sscanf(
	    arg,
	    "%i %i",
	    &obj_num, &tar_obj_num
	);
	if(DBIsObjectGarbage(obj_num) || (obj_num != con->object_num))
	    return(-1);

	obj = xsw_object[obj_num];


	if(DBIsObjectGarbage(tar_obj_num))
	{
	    /* Remove target object from tractor beam lock (unlock
	     * all for now)
	     */
	    DBObjectTractor(obj_num, -1);
	}
	else
	{
	    tar_obj = xsw_object[tar_obj_num];

	    /* Target object is valid, check if is tractorable */
	    if(!DBIsObjectTractorablePtr(obj, tar_obj))
	    {
		char *s = (char *)malloc(
		    STRLEN(tar_obj->name) + 80
		);
		sprintf(
		    s,
"Unable to engage tractor beam on object `%s'.",
		    tar_obj->name
		);
		NetSendLiveMessage(condescriptor, s);
		free(s);
		return(0);
	    }
	    DBObjectTractor(obj_num, tar_obj_num);
	}

	/* Update all connections about tractor beam lock */
	NetSendTractorBeamLock(-1, obj_num, tar_obj_num);

	return(0);
}


int NetSendTractorBeamLock(
	int condescriptor, int obj_num, int tar_obj_num
)
{
	char sndbuf[CS_DATA_MAX_LEN];
	xsw_object_struct *obj;  

	if(DBIsObjectGarbage(obj_num))
	    return(-1);

	obj = xsw_object[obj_num];

	/* SWEXTCMD_TRACTORBEAMLOCK format:
	 *
	 *	obj_num tar_obj_num
	 */
	sprintf(
	    sndbuf,
"%i %i\
 %i %i\n",
	    CS_CODE_EXT,
	    SWEXTCMD_TRACTORBEAMLOCK,
	    obj_num, tar_obj_num
	);
	NetDoSend(condescriptor, sndbuf);

	return(0);
}
