#include <stdio.h>

#include "../include/unvmatch.h"
#include "../include/unvmath.h"
#include "../include/swnetcodes.h"

#include "net.h"
#include "swserv.h"


int NetHandleSetWeaponsLock(int condescriptor, char *arg)
{
	int	obj_num, con_obj_num,
		tar_obj_num, prev_obj_num;
	double scanner_range;
	xsw_object_struct	*con_obj,
				*new_locked_obj = NULL;

	con_obj_num = connection[condescriptor]->object_num;
	if(DBIsObjectGarbage(con_obj_num))
	    return(-1);

	con_obj = xsw_object[con_obj_num];

	/* Record previously locked on object */
	prev_obj_num = con_obj->locked_on;
	if(DBIsObjectGarbage(prev_obj_num))
	{
	    /* Was not locked on to anything, reset values */
	    con_obj->locked_on = -1;
	    prev_obj_num = -1;
	}

	/* Format NET_CMD_WEAPONSLOCK:
	 *
	 *	obj_num, tar_obj_num
	 */
	sscanf(
	    arg,
	    "%i %i",
	    &obj_num, &tar_obj_num
	);

	/* Connection must own the object */
	if(obj_num != con_obj_num)
	    return(-3);

	/* Get the object's current effective scanner range */
	if(con_obj->loc_type == SW_LOC_TYPE_NEBULA)
	    scanner_range = con_obj->scanner_range *
		VISIBILITY_NEBULA;
	else
	    scanner_range = con_obj->scanner_range;

	/* Handle parsed target object value as an argument for
	 * operation
	 */
	switch(tar_obj_num)
	{
	  case -1:	/* Unlock */
	    con_obj->locked_on = -1;
	    break;

	  case -2:	/* Lock on the next object */
	    while(1)
	    {
		/* Match next object */
		con_obj->locked_on = UNVMatchWeaponsLock(
		    xsw_object, total_objects,
		    con_obj_num,
		    prev_obj_num,
		    scanner_range
		);
		/* Unable to match next object? */
		if(DBIsObjectGarbage(con_obj->locked_on))
		    break;

		/* Get the next object */
		new_locked_obj = xsw_object[con_obj->locked_on];

		/* Check if matched object is marked hidden */
		if(new_locked_obj->ext_flags & SWS_OBJ_FLAG_HIDEFROMCON)
		{
		    /* Set previous object to current and try again */
		    prev_obj_num = con_obj->locked_on;
		    new_locked_obj = NULL;
		    continue;
		}

		/* Match successful, break */
		break;
	    }
	    break;

	  case -3:	/* Lock on the nearest object */
	    if(1)
	    {
		int i;
		double closest_d = -1.0, d;
		xsw_object_struct *tar_obj;

		for(i = 0; i < total_objects; i++)
		{
		    if(DBIsObjectGarbage(i))
			continue;

		    tar_obj = xsw_object[i];

		    /* Target object not within the current effective
		     * scanner range?
		     */
		    if(!Mu3DInRangePtr(
			con_obj, tar_obj,
			scanner_range * DBGetObjectVisibilityPtr(tar_obj)
		    ))
			continue;

		    if(closest_d > 0.0)
		    {
			d = Mu3DDistance(
			    tar_obj->x - con_obj->x,
			    tar_obj->y - con_obj->y,
			    tar_obj->z - con_obj->z
			);
			if(d < closest_d)
			{
			    closest_d = d;
			    con_obj->locked_on = i;
			    new_locked_obj = tar_obj;
			}
		    }
		    else
		    {
			closest_d = Mu3DDistance( 
			    tar_obj->x - con_obj->x,
			    tar_obj->y - con_obj->y,
			    tar_obj->z - con_obj->z
			);
			con_obj->locked_on = i;
			new_locked_obj = tar_obj;
		    }
		}
	    }
	    break;

	  default:	/* Lock explicitly on the specified object */
	    if(!DBIsObjectGarbage(tar_obj_num))
	    {
		new_locked_obj = xsw_object[tar_obj_num];

		/* Check if new locked on object is within scanner
		 * range
		 */
		if(Mu3DInRangePtr(
		    con_obj, new_locked_obj,
		    scanner_range *
		       DBGetObjectVisibilityPtr(new_locked_obj)
		))
		{
		    con_obj->locked_on = tar_obj_num;
		}
		else
		{
		    new_locked_obj = NULL;
		    con_obj->locked_on = -1;
		}
	    }
	    break;
	}


	/* Check if locked on a new valid object */
	if(new_locked_obj != NULL)
	{
	    const int locked_obj_num = con_obj->locked_on;
	    int wep_num;

	    /* Need to send set values of target object first */
	    NetSendObjectMaximums(condescriptor, locked_obj_num);

	    /* Do not send object standard values just yet, the
	     * schedualing for the next object values update will be
	     * set to now just below
	     */

	    NetSendObjectName(condescriptor, locked_obj_num);
	    NetSendObjectDescription(condescriptor, locked_obj_num);
	    NetSendSetEngine(condescriptor, locked_obj_num);
	    NetSendSetChannel(
		condescriptor,
		locked_obj_num,
		new_locked_obj->com_channel
	    );

	    /* Send weapon values */
	    for(wep_num = 0;
		wep_num < new_locked_obj->total_weapons;
		wep_num++
	    )
		NetSendWeaponValues(
		    condescriptor, locked_obj_num, wep_num
		);
	}

	/* Set next object values update to now so it gets updated now */
	next.object_values = cur_millitime;

	return(0);
}


int NetSendSetWeaponsLock(int condescriptor)
{
	return(0);
}
