#include <stdio.h>

#include "../include/unvmatch.h"
#include "../include/swnetcodes.h"

#include "net.h"
#include "swserv.h"


int NetHandleWormHoleEnter(int condescriptor, char *arg);


int NetHandleWormHoleEnter(int condescriptor, char *arg)
{
	double d;
	int obj_num, tar_obj_num, dest_obj_num;
	xsw_object_struct *obj, *tar_obj, *dest_obj;
	xsw_vector_compoent_struct *src_attitude;
	sws_connection_struct *con = connection[condescriptor];

	/* SWEXTCMD_WORMHOLEENTER format:
	 *
	 *	obj_num tar_obj_num
	 */
	sscanf(
	    arg,
	    "%i %i",
	    &obj_num, &tar_obj_num
	);

	/* Connection must own the source object */
	if(obj_num != con->object_num)
	    return(-3);

	/* Get pointers to source and target objects */
	if(DBIsObjectGarbage(obj_num))
	    return(-1);

	obj = xsw_object[obj_num];

	if(DBIsObjectGarbage(tar_obj_num))
	    return(-1);

	tar_obj = xsw_object[tar_obj_num];

	/* Target is not a wormhole? */
	if(tar_obj->type != SW_OBJ_TYPE_WORMHOLE)
	    return(-1);

	/* Get linked destination (intercepting object) of target
	 * object (the other end of the wormhole)
	 */
	dest_obj_num = tar_obj->intercepting_object;
	if(DBIsObjectGarbage(dest_obj_num))
	    return(-1);

	dest_obj = xsw_object[dest_obj_num];

	/* Linked to destination need not be a worm hole */


	/* Move source object to destination object's position with
	 * some offsets
	 */
	src_attitude = &obj->attitude_vector_compoent;

	/* Move source object in its heading direction to
	 * away from the destination object based on both object's
	 * size (in Real units)
	 */
	d = (double)dest_obj->size / 1000.0;
	   
	obj->x = dest_obj->x + (d * src_attitude->i);
	obj->y = dest_obj->y + (d * src_attitude->j);
	obj->z = dest_obj->z;

	/* Move the object to a position that is away from the
	 * destination object
	 */
	if((obj->sect_x != dest_obj->sect_x) ||
	   (obj->sect_y != dest_obj->sect_y) ||
	   (obj->sect_z != dest_obj->sect_z)
	)
	{
	    /* Change the object's sector and update related resources */
	    DBObjectDoSetSector(
		obj_num,
		dest_obj->sect_x,
		dest_obj->sect_y,
		dest_obj->sect_z,
		0			/* No position wrapping */
	    );
	    NetSendFObjectSect(condescriptor, obj_num);
	}
	NetSendObjectForcePose(condescriptor, obj_num);

	/* Send create for the wormhole's destination */
	NetSendCreateObject(condescriptor, dest_obj_num);

	return(0);
}
