#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "siteban.h"


int SiteBanIsAllocated(int n);

int SiteBanAdd(
	siteban_ip_union *ip,
	int restrict
);
int SiteBanRemoveIP(siteban_ip_union *ip);

void SiteBanDelete(int n);
void SiteBanDeleteAll(void);


siteban_struct	**siteban;
int		total_sitebans;


/*
 *	Checks if the site ban entry number is allocated.
 */
int SiteBanIsAllocated(int n)
{
	if((siteban == NULL) ||
           (n < 0) ||
           (n >= total_sitebans)
	)
	    return(0);
	else if(siteban[n] == NULL)
	    return(0);
	else
	    return(1);
}


/*
 *	Checks if the specified IP is banned in accordance with the
 *	restrict level.
 */
int SiteBanIsBanned(siteban_ip_union *ip, int restrict)
{
	int i;
        siteban_struct *sb;

        for(i = 0; i < total_sitebans; i++)
        {
	    sb = siteban[i];
            if(sb == NULL)
                continue;

/* Just check if ip matches, do not check restrict level for now.. */
            if(sb->ip.whole == ip->whole)
                return(1);
        }

	return(0);
}

/*
 *	Adds the specified IP to the site ban list.
 */
int SiteBanAdd(
	siteban_ip_union *ip,
	int restrict		/* Just 0 for now */
)
{
	int i, n;
	siteban_struct *sb;

	if(ip == NULL)
	    return(-1);

	/* Sanitize total */
	if(total_sitebans < 0)
	    total_sitebans = 0;

	/* Check if IP is already in the list */
	for(i = 0; i < total_sitebans; i++)
	{
	    sb = siteban[i];
	    if(sb == NULL)
		continue;

	    if(sb->ip.whole == ip->whole)
	    {
		/* Just update entry */
		sb->restrict = restrict;
		return(0);
	    }
	}

        /* Check for available entry */
        for(i = 0; i < total_sitebans; i++)
        {
	    sb = siteban[i];
            if(sb == NULL)
                break;
 	}
	if(i < total_sitebans)
	{
	    n = i;
	}
	else
	{
	    n = total_sitebans;
	    total_sitebans = n + 1;
	    siteban = (siteban_struct **)realloc(
		siteban,
		total_sitebans * sizeof(siteban_struct *)
	    );
	    if(siteban == NULL)
	    {
		total_sitebans = 0;
		return(-3);
	    }
	    siteban[n] = NULL;
	}

	/* Allocate new structure */
	siteban[n] = sb = (siteban_struct *)calloc(
	    1, sizeof(siteban_struct)
	);
	if(sb == NULL)
	    return(-3);

	sb->ip.whole = ip->whole;
	sb->restrict = restrict;

	return(0);
}


/*
 *	Removes ip from site ban list. Returns 0 if successfully
 *	removed or -1 if there was no such ip or error.
 *
 *	Warning: siteban pointer array will be reallocated
 *	in this function!
 */
int SiteBanRemoveIP(siteban_ip_union *ip)
{
        int i, n, d = 0;
	siteban_struct *sb;

	if(ip == NULL)
	    return(-1);

	if(total_sitebans < 0)
	    total_sitebans = 0;

        for(i = 0; i < total_sitebans; i++)
        {
	    sb = siteban[i];
            if(sb == NULL)
                continue;

	    if(sb->ip.whole == ip->whole)
	    {
		SiteBanDelete(i);
		d++;

		/* Shift entries and reallocate */
		total_sitebans--;

		for(n = i; n < total_sitebans; n++)
		    siteban[n] = siteban[n + 1];

		if(total_sitebans > 0)
		{
		    siteban = (siteban_struct **)realloc(
			siteban,
			total_sitebans * sizeof(siteban_struct *)
		    );
		    if(siteban == NULL)
		    {
			total_sitebans = 0;
			return(-3);
		    }
		}
		else
		{
		    free(siteban);
		    siteban = NULL;
		    total_sitebans = 0;
		}
	    }
        }

	return((d > 0) ? 0 : -1);
}


void SiteBanDelete(int n)
{
	if(!SiteBanIsAllocated(n))
	    return;

	free(siteban[n]);
	siteban[n] = NULL;
}

void SiteBanDeleteAll(void)
{
	int i;

	for(i = 0; i < total_sitebans; i++)
	    SiteBanDelete(i);

	free(siteban);
	siteban = NULL;
	total_sitebans = 0;
}
