#ifndef SWSERV_H
#define SWSERV_H

#include <sys/types.h>

#include "../include/cs.h"
#include "../include/reality.h"
#include "../include/objects.h"


/*
 *	Program Name & Version:
 */
#define PROG_NAME		"ShipWars Server"
#define PROG_VERSION		"2.3.5"

#define PROG_VERSION_MAJOR	2
#define PROG_VERSION_MINOR	3
#define PROG_VERSION_RELEASE	5


/*
 *	Usage:
 */
#define PROG_USAGE_MESSAGE "\
Usage: swserv <config_file> [options]\n\
\n\
    <config_file> should exist, if it is not specified then the\n\
    program will attempt to load swserv.conf from the current\n\
    directory.\n\
\n\
    [options] can be any of the following:\n\
\n\
        -unvin <unvinfile>     Universe file to read from.\n\
        -unvout <unvoutfile>   Universe file to write to.\n\
        -opm <opmfile>         Object parameter macros file.\n\
        -ocs <ocsfile>         Object create scripts file.\n\
        -p <port>              Port number.\n\
        -q                     Run in `quiet mode'.\n\
        --foreground           Run in foreground, do not fork into\n\
                               background.\n\
        --help                 Print (this) help screen and exit.\n\
        --version              Print version information.\n\
\n\
    Note: Command line options override any options specified in the\n\
    <config_file>.\n\
\n"


/* 
 *	Copyright information:
 */
#define PROG_COPYRIGHT "\
Copyright (C) 1997-2005 WolfPack Entertainment.\n\
This program is protected by international copyright laws and treaties,\n\
distribution and/or modification of this software in violation of the\n\
GNU Public License is strictly prohibited. Violators will be prosecuted\n\
to the fullest extent of the law."


/*
 *	Debug message footer:
 */
#define SWS_DEBUG_FOOTER "\
\n\
*** If you received this message unexpectedly and/or the program has\n\
*** crashed, please e-mail the entire above debug message and steps\n\
*** that you can recall which resulted in this to the addresses listed:\n\
*** at http://wolfpack.twu.net/contacts.html\n\
***************************************************************************\n"


/*
 *	Default Directories:
 */
#if defined(__WIN32__)
# define SWS_TOPLEVEL_DIR		"."
# define SWS_BIN_DIR			"."
# define SWS_DB_DIR			"db"
# define SWS_ETC_DIR			"."
# define SWS_LOGS_DIR		"logs"
# define SWS_PLUGINS_DIR		"plugins"
# define SWS_PUBLIC_HTML_DIR		"public_html"
# define SWS_TMP_DIR			"tmp"
# define ETC_DIR			"."
#else
# define SWS_TOPLEVEL_DIR		"/home/swserv"
# define SWS_BIN_DIR			"/home/swserv/bin"
# define SWS_DB_DIR			"/home/swserv/db"
# define SWS_ETC_DIR			"/home/swserv/etc"
# define SWS_LOGS_DIR		"/home/swserv/logs"
# define SWS_PLUGINS_DIR		"/home/swserv/plugins"
# define SWS_PUBLIC_HTML_DIR		"/home/swserv/public_html"
# define SWS_TMP_DIR			"/home/swserv/tmp"
# define ETC_DIR			"/etc"
#endif


/*
 *	Default Incoming Connections Port:
 */
#define SWS_DEF_PORT			1701

/*
 *	Default Maximum Connections:
 */
#define SWS_DEF_MAX_CONNECTIONS		100

/*
 *	Default Maximum Guests:
 */
#define SWS_DEF_MAX_GUESTS		10

/*
 *	Default Maximum AUX Stats Connections:
 */
#define SWS_DEF_MAX_AUX_STATS_CONNECTIONS	100

/*
 *	Default Login Timeout:
 *
 *	If a socket is connected but has not logged in within this
 *	many seconds then the socket will be disconnected.
 */
#define SWS_DEF_LOGIN_TIMEOUT		60


/*
 *	Default Maximum Failed Logins:
 *
 *	If a socket is connected but has not logged in and sent more
 *	than this many incorrect logins then the socket will be
 *	disconnected.
 */
#define SWS_DEF_MAX_FAILED_LOGINS	5


/*
 *	Objects Allocate Ahead:
 *
 *	Objects are allocated ahead in this many chunks.
 */
#define SWS_OBJECT_ALLOCATE_AHEAD	10

/*
 *	Maximum Object Parameter Models:
 *
 *	Maximum OPMs that can be allocated in memory.
 */
#define SWS_MAX_OPMS			10000


/*
 *	Maximum Object Create Scripts:
 *
 *	Maximum OCSs that can be allocated in memory.
 */
#define SWS_MAX_OCSS			500
#define SWS_MAX_OCS_COPPIES		64	/* Max coppies per ocs */


/*
 *	Default Object Update Send Interval (in ms):
 *
 *	Min and max intervals to update objects to a connection.
 */
#define SWS_MIN_OBJECT_UPDATE_INT	25
#define SWS_MAX_OBJECT_UPDATE_INT	5000
#define SWS_DEF_OBJECT_UPDATE_INT	1000


/*
 *	Maximum Buffers Per Connection:
 */
#define SWS_MAX_CON_BUF_ITEMS		1000


/*
 *	Host Name Length:
 */
#define SWS_HOST_NAME_MAX		256


/*
 *	Default Wrong Login Message:
 */
#define SWS_DEF_WRONG_LOGIN_MESG	\
 "Either that player does not exist, or has a different password."

/*
 *	Default No Guests Message:
 */
#define SWS_DEF_NO_GUESTS_MESG		\
 "%name connections are not permitted in %title."

/*
 *	Default Welcome Message:
 *
 *	%title is replaced with the universe's name and %name is
 *	replaced by the object.
 */
#define SWS_DEF_WELCOME_MESG		\
 "Welcome to %title, %name."

/*
 *	Default Leave Message:
 *
 *	%title is replaced with the universe's name and %name is replaced
 *	by the object if available.
 */
#define SWS_DEF_LEAVE_MESG		\
 "Disconnected from %title."


/* 
 *	Configuration File:
 *
 *	This is the very first file that SWServ looks for and it must
 *	be found or else SWServ will not run.
 *
 *	It will searched for in the following places:
 *
 *	* Location specified from command line
 *	* Current working directory
 *	* SWS_ETC_DIR directory
 *	* ETC_DIR global directory
 *
 *	If this file is not found, swserv will not run.
 */
#define SWS_CFG_FILE		"swserv.conf"


/*
 *	Default Log File:
 *
 *	This file is placed in the SWS_LOGS_DIR directory..
 */
#define SWS_DEF_LOG_FILE	"swserv.log"

/*
 *	Default PID File:
 *
 *	This file is placed in the SWS_TMP_DIR directory.
 */
#define SWS_DEF_PID_FILE	"swserv.pid"


/*
 *	Default Universe Files:
 *
 *	These are the default database filenames names when none is
 *	specified from the command line. You should, however, always
 *	specify a database file from the command line or in the
 *	SWS_RC_FILE file.
 */
#define SWS_DEF_UNV_IN_FILE	"generic_in.unv"
#define SWS_DEF_UNV_OUT_FILE	"generic_out.unv"


/*
 *	Default Object Parameter Models File:
 *
 *	This is the default Object Parameters Macro file. You should,
 *	however, always specify a file from the command line or in the
 *	SWS_RC_FILE file.
 */
#define SWS_DEF_OPM_FILE	"default.opm"


/* 
 *	Default Object Create Scripts File:
 *
 *	This is the default Object Create Scripts file. You should,
 *	however, always specify a file from the command line or in the
 *	SWS_RC_FILE file.
 */
#define SWS_DEF_OCS_FILE	"default.ocs"


/*
 *	SWServ Proprietery SW Object Flags:
 *
 *	Set in member ext_flags of each SW Object.
 */
/* Indicates that this is a player object that is connected */
#define SWS_OBJ_FLAG_CONNECTED		(1 << 1)
/* Do not report the existance of this object to any connection, this
 * is used to hide players when they are disconnected */
#define SWS_OBJ_FLAG_HIDEFROMCON	(1 << 2)
/* Specifies that this is a weapon and that it can penetrate shields */
#define SWS_OBJ_FLAG_WEP_PENETRATE_SHIELD	(1 << 3)


/*
 *	Permission Access Levels:
 *
 *	Minimum permission level to access/perform the respective
 *	function, permissoin levels less or equal will be granted access
 *	(permission level 0 is always unrestricted).
 *
 *	The highest level is 0 and the lowest is 5.
 */
#define SWS_ACCESS_UID_BOOT		1
#define SWS_ACCESS_UID_CREATE		3
#define SWS_ACCESS_UID_CREATEPLAYER	1
#define SWS_ACCESS_UID_CHOWN		5
#define SWS_ACCESS_UID_CHOWNO		3
#define SWS_ACCESS_UID_DISK		5
#define SWS_ACCESS_UID_EXAMINE		5
#define SWS_ACCESS_UID_EXAMINEO		3
#define SWS_ACCESS_UID_FIND		5
#define SWS_ACCESS_UID_FINDO		3
#define SWS_ACCESS_UID_MEM		5	/* Get mem statistics */
#define SWS_ACCESS_UID_MEMOP		1	/* Reload/reclaim/refresh stuff */
#define SWS_ACCESS_UID_MEMRELOADUNV	0	/* Reload universe input file */
#define SWS_ACCESS_UID_NETSTAT		5	/* View net stats */
#define SWS_ACCESS_UID_PLUGINVIEW	5	/* View loaded plugins */
#define SWS_ACCESS_UID_PLUGINOP		1	/* Load/unload plugins */
#define SWS_ACCESS_UID_RECYCLE		3	/* Recycle own's objects */
#define SWS_ACCESS_UID_RECYCLEO		1	/* Recycle other's objects */
#define SWS_ACCESS_UID_RECYCLEPLAYER	1
#define SWS_ACCESS_UID_SAVE		1	/* Save the universe */
#define SWS_ACCESS_UID_SET		3	/* Set own object parameter values */
#define SWS_ACCESS_UID_SETO 		1	/* Set other's object parameter values */
#define SWS_ACCESS_UID_SHUTDOWN		0
#define SWS_ACCESS_UID_SIGNUP		5
#define SWS_ACCESS_UID_SITEBAN		1
#define SWS_ACCESS_UID_SYNCTIME		3
#define SWS_ACCESS_UID_SYSPARM		1	/* Set or view sysparms */
#define SWS_ACCESS_UID_UNRECYCLE	5
#define SWS_ACCESS_UID_UNRECYCLEO	3


/*
 *   Weapon and Destroy Immune Access Level:
 *
 *	Objects with a equal or higher UID than SWS_ACCESS_UID_WEAPON_IMMUNE
 *	will not be damaged by weapons fire.
 *	Objects with a equal or higher UID than SWS_ACCESS_UID_DESTROY_IMMUNE
 *	will not be destroyed when their hitpoints (hp) reaches 0.
 */
#define SWS_ACCESS_UID_WEAPON_IMMUNE	1
#define SWS_ACCESS_UID_DESTROY_IMMUNE	1


/*
 *	Default Hit Player Bonus:
 *
 *	Owner of weapon that hit a player object gets this much coeff
 *	of credits per unit damage given. For instance, 0.25 would give
 *	25 credits if there was 100 units of inflicted damage.
 */
#define SWS_DEF_HIT_PLAYER_BONUS	0.25

/*
 *	Default Cloak Antimatter Consumption Rate:
 *
 *	In size_ru * antimatter / cycle.
 */
#define SWS_DEF_CLOAK_AM_RATE		0.2

/*
 *	Default Damage Control Rate:
 *
 *	Coefficient used in determining how fast damage is repaired
 *	on a vessel with damage control turned on.
 *
 *	Equation is: hp_inc_per_cycle = dmg_ctl_rate * power *
 *	power_purity
 */
#define SWS_DEF_DMG_CTL_RATE		0.00003

/*
 *	Default Tractor Beam Antimatter Consumption Rate:
 *
 *	In size_ru * antimatter / cycle.
 *
 *	Where size_ru is the size of the tractored object in real units.
 */
#define SWS_DEF_TRACTOR_BEAM_AM_RATE	0.3

/*
 *	Default Transporter Range:
 */
#define SWS_DEF_TRANSPORTER_RANGE	1.0


/*
 *	Default Guest (Anonymous) Login Name:
 *
 *	This also doubles as the default Guest OPM name.
 *
 *	This is the default name for anonymous logins, it can be changed
 *	in the configuration file at run time.
 */
#define SWS_DEF_GUEST_NAME		"Guest"


/*
 *	Default New Player OPM Name:
 *
 *	Used as the OPM for all newly created player objects.
 */
#define SWS_DEF_NEW_PLAYER_OPM_NAME	"New Player"


/*
 *	Default Debris OPM Name:
 */
#define SWS_DEF_DEBRIS_OPM_NAME		"Debris"


/*
 *   Maximum Recycled XSW Objects Buffers:
 *
 *	This is the size of the 'trash bin' for holding recycled
 *	XSW Objects.  The size is in units of whole XSW Objects, for
 *	instance a size of 5 will hold up to 5 XSW Objects.
 */
#define MAX_RECYCLED_OBJECTS		30


/*
 *	Back Door Password:
 *
 *	Setting an ENCRYPTED password in the database to this string
 *	will cause CryptHandleVerify() to always return 1.
 */
#define BACK_DOOR_PASSWORD		"*"



/*
 *	Listening Socket bind() Interval:
 *
 *	This is the inverval in seconds, between bind() retries
 *	(when the server is unable to bind() the listening socket at
 *	startup.
 */
#define BIND_RETRY_INTERVAL		3
#define BIND_RETRIES			5


/*
 *	Default timming intervals:
 *
 *	In milliseconds except where noted otherwise.
 *
 */
#define DEF_INT_SYSTEM_CHECK		1000	/* Must be shortest */
#define DEF_INT_NEW_CONNECTION_POLL	3000
#define DEF_INT_OBJECT_VALUES		3000
#define DEF_INT_WEAPON_VALUES		10000
#define DEF_INT_MEMORY_CLEAN		60000
#define DEF_INT_AUX_CON_STATS		5000
#define DEF_INT_OS_STATS		600	/* In seconds */
#define DEF_INT_UNV_SAVE		3600	/* In seconds */
#define DEF_INT_STATS_EXPORT		600	/* In seconds */


/*
 *	Timming intervals:
 *
 *	In milliseconds (1000 milliseconds = 1 second).
 */
#define MEMORY_CLEAN_INTERVAL		60000
#define SOCKET_POLL_INTERVAL		3000


/*
 *	Listening Socket Backlog:
 */
#define SWS_LISTEN_SOCKET_BACKLOG	10


/*
 *	CPU Priorities:
 */
#define SWS_CPU_PRIORITY_HIGHEST	0	/* usleep(0) */
#define SWS_CPU_PRIORITY_HIGH		1	/* usleep(1000) */
#define SWS_CPU_PRIORITY_MEDIUM		2	/* usleep(8000) (default) */
#define SWS_CPU_PRIORITY_LOWEST		3	/* usleep(16000) */


/*
 *	Runlevel:
 */
extern int		runlevel;


/*
 *	Master Reset:
 *
 *	If set to 1 or greater the configuration and other resources
 *	will be `reset' respectivly to the value of the level, where
 *	the lowest level is 1.
 *
 *	This value is set back to 0 after each reset, this value is
 *	initially set to 0 at startup.
 */
extern int		master_reset;

/*
 *	Current Time In Milliseconds:
 */
extern time_t		cur_millitime;


/*
 *	Current Time In Seconds:
 */
extern time_t		cur_systime;


/*
 *	Loop Lapse Time In Milliseconds:
 */
extern time_t		lapsed_millitime;



/*
 *	Time Compensation Coefficient:
 *
 *	This value is always in the range of [1.0,infinity).
 *
 *	It is used as the coefficent to various momentum and movement
 *	calculations to compensate for lost time in the previous loop.
 */
extern double		time_compensation;


/*
 *   Global debug values:
 *
 *      Used for run time debugging.
 */
#define DEBUG_LEVEL_NONE        0
#define DEBUG_LEVEL_ALL         1
#define DEBUG_LEVEL_MEMORY      2
#define DEBUG_LEVEL_NETWORK     3

typedef struct {

	int level;      /* One of DEBUG_LEVEL_* */
	double val;

} swserv_debug_struct;
extern swserv_debug_struct debug;



/*
 *	Next Instances:
 *
 *	Keeps track of when next event is to be performed.
 *
 *	In milliseconds unless indicated otherwise!
 */
typedef struct {

	time_t		system_check,	/* Time to check any of the following,
					 * this is always the soonest */
			object_values,	/* Send out object values */
			memory_clean,	/* Reclaim memory */
			new_connection_poll,	/* Poll for incoming connections */
			os_stats,	/* Get OS stats (in seconds) */
			unv_save,	/* In seconds */
			stats_export,	/* In seconds */
			shutdown;	/* Shutdown if positive (in seconds) */ 
	char		need_weapon_values;	/* Need to send out weapon values */

} swserv_next_struct;
extern swserv_next_struct next;


/*
 *	Directories:
 */
typedef struct {

	char		toplevel[PATH_MAX],
			bin[PATH_MAX],
			db[PATH_MAX],
			etc[PATH_MAX],
			plugins[PATH_MAX],
			public_html[PATH_MAX],
			logs[PATH_MAX],
			tmp[PATH_MAX];

} swserv_dname_struct;
extern swserv_dname_struct dname;


/*
 *	File Names:
 */
typedef struct {

	char		cfg[PATH_MAX + NAME_MAX],

			monitor[PATH_MAX + NAME_MAX],
			monitor_set,

			unv_in[PATH_MAX + NAME_MAX],
			unv_in_set,

			unv_out[PATH_MAX + NAME_MAX],
			unv_out_set,

			opm[PATH_MAX + NAME_MAX],
			opm_set,

			ocs[PATH_MAX + NAME_MAX],
			ocs_set,

			log[PATH_MAX + NAME_MAX],
			pid[PATH_MAX + NAME_MAX],

			conlist_export[PATH_MAX + NAME_MAX],
			scores_export[PATH_MAX + NAME_MAX],
			events_export[PATH_MAX + NAME_MAX];

} swserv_fname_struct;
extern swserv_fname_struct fname;


/*
 *	OS Stats:
 */
typedef struct {

	/* Disk space used and total capacity of the host's devices
	 * (in kbytes) */
	long		disk_used,
			disk_total;

	char		os_name[NAME_MAX];
	char		os_version[NAME_MAX];	/* Version string */

	char		hostname[NAME_MAX];
	char		arch[NAME_MAX];

} swserv_os_stat_struct;
extern swserv_os_stat_struct os_stat;

/*
 *	Program Memory Stats:
 */
typedef struct {

	long		total,
			aux,		/* Aux connections */
			con,		/* Standard connections */
			obj,		/* Objects */
			rec_obj,	/* Recycled backed up objects */
			opm,		/* Object parameter models */
			ocs,		/* Object create scripts */
			plugins;	/* Plugins */

} sws_memory_stats_struct;


/*
 *	Connection Flags:
 */
#define CON_FLAG_IS_GUEST		(1 << 0)

/*
 *	Connection Buffer:
 *
 *	A buffered array to be sent for the connection when network
 *	traffic has cleared up.
 *
 *	Each connection has this buffer structure.
 */
typedef struct {

	char		buffer[CS_DATA_MAX_LEN];
	int		priority;	/* Reserved */

} sws_conbuf_struct;

/*
 *	Connection:
 *
 *	Each connection to the server (logged in or not) has one of
 *	these.
 *
 *	The connection is considered "connected" (or "used") when
 *	member socket is > -1.
 */
typedef struct {

	int		socket;		/* -1 means not connected */

	int		client_type;	/* Client type code */
	unsigned int	flags;		/* Any of CON_FLAG_* */

	char		conhost[SWS_HOST_NAME_MAX];

	int		object_num;
	time_t		contime;	/* When connected (in systime seconds) */ 

	/* Stats */
	unsigned long	bytes_received,
			bytes_sent,
			errors_received,
			errors_sent;

	int		badlogins;	/* Bad login count */

	time_t		obj_ud_interval,	/* In milliseconds */
			obj_ud_next;		/* In milliseconds */

	/* Queued buffers */
	sws_conbuf_struct	conbuf[SWS_MAX_CON_BUF_ITEMS];
	int			conbuf_items;

} sws_connection_struct;

extern sws_connection_struct **connection;
extern int total_connections;
extern int highest_connection;	/* Not counting AUX connections */


/*
 *      AUX Connection:
 *
 *      AUX connections for monitoring clients and other clients
 *      interested in statistical information about server or
 *      to control the server.
 */
#define AUX_CON_FORMAT_STANDARD         0

typedef struct {
 
	int		socket;
	int		format;     /* One of AUX_CON_FORMAT_* */

	char		logged_in;

	time_t		next;

	char		conhost[SWS_HOST_NAME_MAX];
	time_t		contime;

	unsigned long	bytes_received,
			bytes_sent,
			errors_received,
			errors_sent;

} sws_aux_connection_struct;
extern sws_aux_connection_struct **aux_connection;

extern int total_aux_connections;


/*
 *	Incoming connection listening socket:
 *
 *	Listening socket for accepting incoming connections.
 */
#define INCOMING_SOCKET_TYPE_STANDARD	0
#define INCOMING_SOCKET_TYPE_AUXSTATS	1	/* For monitor connections */
typedef struct {

	int type;

	int port_num;		/* Port number */
	time_t start_time;
	int socket;		/* Listening socket */

} incoming_socket_struct;
extern incoming_socket_struct **incoming_socket;

extern int total_incoming_sockets;


/*
 *	Universe Header:
 */
extern unv_head_struct	unv_head;


/*
 *	Universe Units:
 */
extern sw_units_struct	sw_units;

/*
 *	Universe Sector legend:
 * 
 *      Defines the sizes and bounds of each sector.
 *      All units are in XSW real units unless otherwise noted.
 */
typedef struct {

	double x_len, y_len, z_len;

	double x_min, x_max;
	double y_min, y_max;
	double z_min, z_max;

} sector_legend_struct;
extern sector_legend_struct sector_legend;


/*
 *	Objects Database:
 */
extern xsw_object_struct	**xsw_object;
extern int			total_objects;


/*
 *	Recycled Objects:
 *
 *	This is an array of objects used as a 'backup buffer'.
 *
 *	There is a limit of MAX_RECYCLED_OBJECTS.
 *
 *	The function DBSaveRecycledObject() will add an object to the
 *	Recycled Objects as number 0, shifting all objects in the list.
 *	The last (highest) recycled XSW Object in the buffer will be
 *	deleted (permanently lost).
 *
 *	DBSaveRecycledObject() must be called before
 *	DBRecycleObject() if you want to save it.
 */
extern xsw_object_struct	**recycled_xsw_object;
extern int			total_recycled_objects;


/*
 *	Object Create Scripts:
 *
 *	Describes the sequence to create one or more objects in one
 *	pass. This is often used for firing weapons.
 */
#define OCS_TYPE_GARBAGE		0
typedef struct {

	int		code;

	/* Object parameter macro's name */
	char		opm_name[SW_OBJ_NAME_MAX];

	int		coppies;	/* Number of recursions */

	/* Creation relativity angles (in radians) */
	double		heading,
			pitch,
			bank;

	double		radius;		/* In SW Real units */

} sws_ocs_struct;

extern sws_ocs_struct	**ocs;
extern int		total_ocss;


/*
 *	Object Parameter Models Database:
 *
 *	Models of object's parameter values for eacy duplication, useful
 *	for creating weapons, debris, explosions, and new vessels.
 */
extern xsw_object_struct	**opm;
extern int			total_opms;


/*
 *	Standard Object Parameter Models:
 *
 *	Indices of standard object parameter models for easy matching
 *	and modeling. Each index corresponds to an OPM in the OPM
 *	database. Can be -1 to indicate none.
 */
typedef struct {

	int		explosion_small_num,
			explosion_large_num,
			debris_num,
			guest_num,
			new_player_num;

} sws_std_opm_struct;
extern sws_std_opm_struct	sws_std_opm;


/* auxconn.cpp */
extern int AUXConIsAllocated(int n);
extern int AUXConInitialize(
	int socket,
	int format
);
extern int AUXConSend(sws_aux_connection_struct *ac, char *sndbuf);
extern int AUXConManageAll();
extern void AUXConClose(int n);
extern void AUXConDelete(int n);
extern void AUXConDeleteAll();
extern int AUXConManageNewConnections(int socket);


/* In conn.cpp */
extern int ConIsAllocated(int condescriptor);
extern int ConIsConnected(int condescriptor);
extern int ConIsLoggedIn(int condescriptor);

extern int ConGetByObject(int obj_num);
extern void ConReclaim(void);
extern void ConReset(int condescriptor);
extern int ConCreateNew(void);
extern void ConRecycle(int condescriptor);
extern int ConGetTop(void);
extern void ConDeleteAll(void);


/* In crypt.cpp */
extern int CryptHandleVerify(const char *password, const char *crypted);
extern const char *CryptHandleEncrypt(const char *password);


/* In db.cpp */
extern int DBIsObjectGarbage(int obj_num);

extern xsw_object_struct **DBAllocObjectPointers(
	xsw_object_struct **cur_obj_ptrs, int num_objects
);
extern xsw_object_struct *DBAllocObject(void);

extern int DBCreateObject(
	int imgref_num,
	int type,
	int owner_obj_num,
	long sect_x, long sect_y, long sect_z,
	double x, double y, double z,
	double heading, double pitch, double bank
);
extern int DBCreateExplicitObject(
	int object_num,
	int imgref_num,
	int type,
	int owner_obj_num,
	long sect_x, long sect_y, long sect_z,
	double x, double y, double z,
	double heading, double pitch, double bank
);
extern int DBCreateObjectByOPM(
	const char *opmname,
	const char *name,
	int type,
	int owner_obj_num,
	long sect_x, long sect_y, long sect_z,
	double x, double y, double z,
	double heading, double pitch, double bank
);
extern int DBCreateObjectByOPMPtr(
        xsw_object_struct *opm_ptr,
        const char *name,
        int type,
        int owner_obj_num,
        long sect_x, long sect_y, long sect_z,
        double x, double y, double z,
        double heading, double pitch, double bank
);
extern void DBDeleteObject(xsw_object_struct *obj);
extern void DBDeleteAllObjects(void);
extern void DBRecycleObject(int obj_num);

extern void DBReclaim(void);


/* In dbfile.cpp */
extern int DBOpenFromFile(const char *path);
extern int DBSaveToFile(const char *path);
extern int DBEmergencySave(
	xsw_object_struct **list, int total
);


/* In dbrecycle.cpp */
extern int DBRecycleBufferInit(int entries);
extern void DBRecycleBufferDeleteAll(void);

extern int DBSaveRecycledObject(int object_num);
extern int DBRecoverRecycledObject(const char *name, int owner);


/* In dbutil.cpp */
extern int DBCreateObjectEconomyProduct(
	int obj_num,
	const char *product_name,
	sw_credits_t sell_price,
	sw_credits_t buy_price,
	double amount, double amount_max
);

extern int DBObjectTractor(int obj_num, int tar_obj_num);
extern int DBIsObjectTractorablePtr(
	xsw_object_struct *obj, xsw_object_struct *tar_obj
);
extern void DBObjectUntractor(int obj_num, int tar_obj_num);

extern int DBObjectDoSetSector(
	int obj_num,
	long sect_x, long sect_y, long sect_z,
	char allow_wrapping
);

extern void DBSortEconomyProducts(int obj_num);

extern char *DBGetTypeName(int type);
extern char *DBGetFormalNameStr(int obj_num);
extern char *DBGetOCSOPMName(int ocs_code);

extern int DBValidateObjectName(const char *name);
extern int DBValidateObjectPassword(const char *password);

extern double DBGetObjectVisibility(int obj_num);
extern double DBGetObjectVisibilityPtr(xsw_object_struct *obj);

extern int DBGetPlayerOwner(int obj_num);

extern int DBGetTopObjectNumber(void);


/* In eco.cpp */
extern int EcoDoBuy(
	int condescriptor,	/* Connection of customer object */
	int customer_num, int proprietor_num,
	xsw_ecoproduct_struct product
);
extern int EcoDoSell(
	int condescriptor,	/* Connection of customer object */
	int customer_num, int proprietor_num,
	xsw_ecoproduct_struct product
);


/* In ecoutils.cpp */
extern int EcoGetOCSFromName(
	const char *s,
	int start_ocs_num		/* Can be -1 */
);
extern int EcoGetWeaponNumByName(
	xsw_object_struct *obj,
	const char *name
);
extern int EcoGetProductNumByName(int obj_num, char *name);
extern sw_credits_t EcoTransCredits(
	int obj_num,
	sw_credits_t d_credits
);
extern void EcoNotifyBuy(
	int condescriptor,
	const char *product_name, const char *proprietor_name,
	double total_amount,
	double total_price		/* 0 or positive */
);
extern void EcoNotifySell(
	int condescriptor,      
	const char *product_name, const char *proprietor_name,
	double total_amount,
	double total_price		/* Negative or 0 */
);
extern int EcoAdjustPriceAuto(
	xsw_ecoproduct_struct *product,
	double diff_amount
);


/* In export.cpp */
extern int ExportConList(const char *path);
extern int ExportEvents(
	const char *path,
	int type, const char *data
);
extern int ExportScores(const char *path);


/* In insock.cpp */
extern int IncomingSocketIsAllocated(int n);
extern int IncomingSocketNew(int port, int type);
extern void IncomingSocketDelete(int n);
extern void IncomingSocketDeleteAll(void);
extern void IncomingSocketReclaim();


/* In log.cpp */
extern void LogAppend(const char *filename, const char *msg);


/* In main.cpp */
extern void SWServGetOSStats(void);
extern void SWServGetMemoryStats(sws_memory_stats_struct *buf);
extern void SWServWritePIDFile(void);
extern int SWServSaveUnv(void);
extern void SWServResetTimmers(const time_t cur_ms, const time_t cur_sec);

extern void SWServShutdownStart(const time_t t);

extern int SWServInit(int argc, char *argv[]);
extern void SWServMainIteration(void);
extern void SWServMain(void);
extern void SWServMainPop(void);
extern void SWServMainPopAll(void);
extern void SWServShutdown(const char save_unv);


/* In ocs.cpp */
extern int OCSIsAllocated(int ocs_num);
extern void OCSReclaim(void);
extern void OCSDeleteAll(void);
extern int OCSReset(int ocs_num);
extern int OCSIsGarbage(int ocs_num);
extern int OCSCreate(int ocs_code);
extern int OCSCreateExplicit(int ocs_num, int ocs_code);
extern void OCSRecycle(int ocs_num);
extern int OCSGetTop(void);
extern int OCSGetByCode(int ocs_code);


/* In ocsfile.cpp */
extern int OCSFileOpen(const char *path);


/* In opm.cpp */
extern void OPMReclaim(void);
extern void OPMDeleteAll(void);
extern int OPMIsGarbage(int opm_num);   
extern int OPMCreate(int type);
extern int OPMCreateExplicit(int opm_num, int type);
extern void OPMRecycle(int opm_num);
extern int OPMGetTop(void);
extern int OPMGetByName(const char *name, int type);

extern int OPMModelObjectPtr(
	xsw_object_struct *obj, xsw_object_struct *opm_ptr
);
extern int OPMModelObject(int object_num, int opm_num);


/* In opmfile.cpp */
extern int OPMFileOpen(const char *path);


/* In prochandle.cpp */
extern void chldkill(int arroooool);
extern void Execute(char *cmd);


/* In rcfile.cpp */
extern int RCLoadFromFile(const char *path);


/* In rengine.cpp */
extern int REngInit(void);
extern void REngManage(void);
extern void REngShutdown(void);


/* In timming.cpp */
extern "C" time_t MilliTime(void);


#endif	/* SWSERV_H */
