#ifndef SYSPARM_H
#define SYSPARM_H

#include <sys/types.h>

#include "../include/cs.h"
#include "../include/objects.h"


typedef struct _swserv_sysparm_struct	swserv_sysparm_struct;


/*
 *	Sysparm:
 */
struct _swserv_sysparm_struct {

	char		allow_guest,	/* Allow guest connections? */
			allow_guest_signup,	/* Allow guests to signup? */
			cease_fire;	/* No one may fire a weapon? */

	double		cloak_am_rate;	/* Cloak antimatter consumption rate
					 * in size_ru * antimatter / cycle */

	double		com_range;	/* Max range for hail and com messages
					 * in real units */

	char		con_notify,	/* Notify everyone of new logins? */
			console_quiet;	/* Do not print non-error and
					 * non-warning messages? (can only
					 * be from the command line) */

	int		cpu_priority;	/* One of SWS_CPU_PRIORITY_* */

	/* Damage control applied repair (in hp per cycle) */
	double          dmg_ctl_rate;

	/* Name for anonymous logins and the Guest OPM */
	char		guest_login_name[SW_OBJ_NAME_MAX];

	char		hide_players_home;	/* Hide disconnected players
						 * at home objects? */

	/* Credits coeff per unit damage given to a player object */
	double          hit_player_bonus;

	char		homes_destroyable;	/* Allow home objects to
						 * be destroyable? */

	/* Intervals (in ms, unless noted otherwise) */
	time_t          int_system_check,       /* Must be the shortest
						 * of all the intervals */   
			int_new_connection_poll,
			int_object_values,
			int_weapon_values,
			int_memory_clean,
			int_aux_con_stats,
			int_os_stats,		/* In seconds */
			int_stats_export,	/* In seconds */
			int_unv_save;		/* In seconds */

	char		killer_gets_credits,	/* Killer gets the credits of
						 * destroyed object? */
			log_general,	/* Log general stuff? */
			log_events,	/* Log events? */
			log_net,	/* Log network events? (not errors) */
			log_errors;	/* Log errors? */

	time_t		login_timeout;	/* Login timeout (in seconds) */

	int		max_aux_connections,
			max_connections,
			max_failed_logins,
			max_guests;

	char            mesg_wrong_login[CS_MESG_MAX],
			mesg_no_guests[CS_MESG_MAX],
			mesg_welcome[CS_MESG_MAX],
			mesg_leave[CS_MESG_MAX];

	char		report_destroyed_weapons,	/* Report weapon objects
							 * being destroyed? */
			respawn_players,	/* Respawn players instead if
						 * recycling them when they
						 * are destroyed? */
			send_starchart,	/* Send starchart after logins
					 * and refreshes? */
	  		single_connection;	/* Limit one connection per
						 * player? */

	double		tractor_beam_range,	/* Max range in real units */
			tractor_beam_am_rate;	/* Tractor beam antimatter
						 * consumption rate in
						 * size_ru * antimatter / cycle */

	double		transporter_range;	/* Max range in real units */

	char		use_server_ai;	/* Use the server's default AI
					 * simulation? */

	time_t		weapon_disarm_lifespan;	/* 0 = infinate
						 * 1 = no change
						 * else = life span in ms
						 */
};

extern swserv_sysparm_struct	sysparm;


#endif	/* SYSPARM_H */
