/*
			Universe Editor: Lists

	Functions:

	void UEWUpdateObjectList(uew_struct *uew_ptr)

	void UEWObjectListScrollToItem(
	        uew_struct *uew_ptr,
	        int obj_num
	)

	void UEWUpdateObjectListItemName(
	        uew_struct *uew_ptr,
	        int obj_num
	)


	---

 */

#include <stdlib.h>
#include <string.h>

#include "../include/string.h"
#include "../include/osw-x.h"
#include "../include/widget.h"

#include "../include/reality.h"
#include "../include/objects.h"
#include "../include/unvmatch.h"
   
#include "ue.h"
#include "uew.h"


/*
 *	Updates the objects list to match values of objects pointers.
 *	All items in colum list will be unselected.
 */
void UEWUpdateObjectList(uew_struct *uew_ptr)
{
	int i, n;
	char string[128];
	xsw_object_struct **obj_ptr;
	clist_struct *list_ptr;
	clist_row_struct *row_ptr;


	if(uew_ptr == NULL)
	    return;

	list_ptr = &uew_ptr->objects_list;


	/*   Delete all existing entries in colum list and
	 *   unselect all.
	 */
	CListUnselectAllRows(list_ptr);
	CListClear(list_ptr);


	/* Add objects list to colum list */
	for(i = 0,	/* Object number */
	    n = 0,	/* List row position */
	    obj_ptr = uew_ptr->object;
	    i < uew_ptr->total_objects;
	    i++, obj_ptr++
	)
	{
	    if(*obj_ptr == NULL)
		continue;

	    if((*obj_ptr)->type <= SW_OBJ_TYPE_GARBAGE)
		continue;


	    if(CListRowInsert(list_ptr, -1) < 0)
		continue;

	    /* Get pointer to newly allocater row */
	    row_ptr = list_ptr->row[n];

	    sprintf(string, "#%i", i);
	    if(
	        CListAddItem(
		    list_ptr,
		    string,
		    OSWQueryCurrentFont(),
		    widget_global.editable_text_pix,
		    0,
		    n
	        ) < 0
	    )
	    {
		n++;
		continue;
	    }

	    /* Set row data as the index of its associated object */
	    row_ptr->data = (void *)i;

	    CListAddItem(
		list_ptr,
		(*obj_ptr)->name,
		OSWQueryCurrentFont(),
		widget_global.editable_text_pix,
		0,
		n
	    );

	    n++;	/* Increment row number */
	}	

	if(uew_ptr->map_state)
	     CListDraw(&uew_ptr->objects_list, CL_DRAW_AMOUNT_COMPLETE);
}


/*
 *	Scrolls object list to obj_num.
 */
void UEWObjectListScrollToItem(
	uew_struct *uew_ptr, int obj_num
)
{
	int i, m, n;
	unsigned int width, height;
	scroll_bar_struct *sb;
	clist_struct *clist;
	clist_row_struct *row;

	if(uew_ptr == NULL)
	    return;

	clist = &uew_ptr->objects_list;

	n = -1;
	for(i = 0; i < clist->total_rows; i++)
	{
	    row = clist->row[i];
	    if(row == NULL)
		continue;

	    if((int)row->data == obj_num)
	    {
		n = i;
		break;
	    }
	}

	/* Could not find row? */
	if(n == -1)
	    return;

	OSWGetWindowGeometry(
	    clist->list, NULL, NULL, &width, &height, NULL
	);
	sb = &clist->sb;
	sb->y_win_pos = (n * (int)clist->row_height) -
	    (int)height;

	m = (clist->total_rows * (int)clist->row_height) -
	    (int)height;
	if(sb->y_win_pos > m)
	    sb->y_win_pos = m;
	if(sb->y_win_pos < 0)
	    sb->y_win_pos = 0;
}


/*
 *	Updates the name of the object on the objects list.
 */
void UEWUpdateObjectListItemName(
	uew_struct *uew_ptr,
	int obj_num
)
{
	int i;
	clist_struct *clist;
	clist_row_struct *row;

	if(uew_ptr == NULL)
	    return;

	clist = &uew_ptr->objects_list;

	if(UEWIsObjectGarbage(uew_ptr, obj_num))
	    return;

	for(i = 0; i < clist->total_rows; i++)
	{
	    row = clist->row[i];
	    if(row == NULL)
		continue;

	    if(((int)row->data == obj_num) &&
	       (row->total_items > 1)
	    )
	    {
		clist_item_struct *item = row->item[1];
		if(item == NULL)
		    continue;

		free(item->label);
		item->label = strdup(
		    uew_ptr->object[obj_num]->name
		);
	    }
	}

}




