#include <stdlib.h>
#include <string.h>

#include "../include/string.h"
#include "../include/osw-x.h"
#include "../include/widget.h"
#include "../include/reality.h"
#include "../include/objects.h"
#include "../include/unvutil.h"

#include "uew.h"
#include "ue.h"
#include "wepw.h"


int WepWGetLastSelected(wepw_struct *wepw_ptr);
xsw_object_struct *WepWGetObjectPtr(wepw_struct *wepw_ptr);
xsw_weapon_struct *WepWGetWeaponPtr(
	wepw_struct *wepw_ptr, int wep_num
);
int WepWIsAllocated(int n);

int WepWAllocate(void);
void WepWDelete(int n);
void WepWDeleteAll(void);

int WepWGetAllValues(wepw_struct *wepw_ptr);
int WepWGetValues(wepw_struct *wepw_ptr, int wep_num);
int WepWSetValues(wepw_struct *wepw_ptr, int wep_num);

static void WepWCListActivateCB(
	clist_struct *clist, void *data
);
static int WepWCreateCB(void *ptr);
static int WepWDeleteCB(void *ptr);
static int WepWCloseCB(void *ptr);

int WepWInit(int n, void *src_ptr);
int WepWDraw(int n, int amount);
int WepWManage(int n, event_t *event);
int WepWManageAll(event_t *event);
void WepWMap(int n);
void WepWDoMapValues(int n, void *src, int obj_num);
void WepWUnmap(int n);
void WepWDestroy(int n);


namespace static_wepw {
	wepw_struct	*delete_wepw;
}

#ifndef MAX
#define MIN(a,b)        (((a) < (b)) ? (a) : (b))
#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#endif


/*
 *	Gets the last selected row number or -1 on error nor
 *	no rows selected.
 */
int WepWGetLastSelected(wepw_struct *wepw_ptr)
{
	int n;
	clist_struct *list;


	if(wepw_ptr == NULL)
	    return(-1);

	list = &wepw_ptr->list;
	if(list->total_sel_rows < 1)
	    return(-1);

	n = list->sel_row[list->total_sel_rows - 1];
	if((n < 0) || (n >= list->total_rows))
	    return(-1);

	if(list->row[n] == NULL)
	    return(-1);

	return(n);
}


/*
 *	Returns the pointer to the object referanced in the
 *	weapons window wepw_ptr.  Returns NULL on error.
 */
xsw_object_struct *WepWGetObjectPtr(
	wepw_struct *wepw_ptr
)
{
	int i, obj_num;
	uew_struct *uew_ptr; 


	if(wepw_ptr == NULL)
	    return(NULL);


	uew_ptr = (uew_struct *)wepw_ptr->src;
	obj_num = wepw_ptr->src_obj_num;

	/* Check if uew pointer is valid */
	for(i = 0; i < total_uews; i++)
	{
	    if(uew[i] == NULL)
		continue;

	    if(uew[i] == uew_ptr)
		break;
	}
	if(i >= total_uews)
	    return(NULL);


	if(UEWIsObjectGarbage(
	    uew_ptr,
	    obj_num
	))  
	    return(NULL);
	else
	    return(uew_ptr->object[obj_num]);
}


/*
 *	Get pointer to weapon wep_num.  Returns NULL on error or
 *	failed match.
 */
xsw_weapon_struct *WepWGetWeaponPtr(
	wepw_struct *wepw_ptr,
	int wep_num
)
{
	xsw_object_struct *obj_ptr;


	/* Get pointer to object */
	obj_ptr = WepWGetObjectPtr(wepw_ptr);

	/* Is the object valid and non-garbage? */
	if(obj_ptr == NULL)
	    return(NULL);


	/* Is weapon wep_num allocated on object? */
	if((wep_num < 0) || (wep_num >= obj_ptr->total_weapons))
	    return(NULL);

	return(
	    obj_ptr->weapons[wep_num]
	);
}


/*
 *	Checks if wepon window n is allocated.
 */
int WepWIsAllocated(int n)
{
	if((wepw == NULL) ||
	   (n < 0) || (n >= total_wepws)
	)
	    return(0);
	else if(wepw[n] == NULL)
	    return(0);
	else
	    return(1);
}          


/*
 *      Allocates a new universe header window, returning its
 *      index number.
 */      
int WepWAllocate()
{
	int i, n;

	 
	if(total_wepws < 0)
	    total_wepws = 0;

	for(i = 0; i < total_wepws; i++)
	{   
	    if(wepw[i] == NULL)
		break;
	}
	if(i < total_wepws)
	{
	    n = i;
	}
	else
	{
	    n = total_wepws;
	    total_wepws++;
	    wepw = (wepw_struct **)realloc(
		wepw,  
		total_wepws * sizeof(wepw_struct *)
	    );
	    if(wepw == NULL)
	    {
		total_wepws = 0;
		return(-1);
	    }
	}
 
	wepw[n] = (wepw_struct *)calloc(
	    1,   
	    sizeof(wepw_struct)
	);
	if(wepw[n] == NULL)
	{
	    return(-1);   
	}


	return(n);
}


/*
 *	 Deletes weapon window.
 */     
void WepWDelete(int n)
{
	if(!WepWIsAllocated(n))
	    return;
 
	
	/* Deallocate resources */
	WepWDestroy(n);
	
	    
	/* Free structure itself */
	free(wepw[n]);
	wepw[n] = NULL;


	return;
}

/*
 *	Deletes all weapon windows.
 */
void WepWDeleteAll()
{
	int i;


	for(i = 0; i < total_wepws; i++)
	    WepWDelete(i);

	free(wepw);
	wepw = NULL;

	total_wepws = 0;


	return;
}


/*
 *	Procedure to fetch all values from object specified in
 *	in weapon window wepw_ptr and put them into the prompts
 *	and list window.  Prompts will be redrawn if mapped.
 */
int WepWGetAllValues(wepw_struct *wepw_ptr)
{
	int i;
	clist_struct *list;
	xsw_object_struct *obj_ptr;
	char text[1024];


	if(wepw_ptr == NULL)
	    return(-1);

	list = &wepw_ptr->list;


	/* Delete all rows in list */
	CListUnselectAllRows(list);
	CListClear(list);


	/* Get pointer to object referanced in weapon window */
	obj_ptr = WepWGetObjectPtr(wepw_ptr);

	if(obj_ptr == NULL)
	{
	    /* No valid object */

	    /* Reset referance to object */
	    wepw_ptr->src_obj_num = -1;
	    wepw_ptr->src_obj_ptr = NULL;

	    /* Do not update list */

	    /* Redraw list as needed */
	    if(list->map_state)
		CListDraw(list, CL_DRAW_AMOUNT_COMPLETE);
	}
	else
	{
	    /* Add list rows */
	    for(i = 0; i < obj_ptr->total_weapons; i++)
	    {
		if(obj_ptr->weapons[i] == NULL)
		    continue;

		if(CListRowInsert(list, i) < 0)
		    break;

		sprintf(text, "Weapon #%i", i + 1);
		CListAddItem(
		    list,
		    text,
		    OSWQueryCurrentFont(),
		    widget_global.editable_text_pix,
		    0,		/* Attributes */
		    i		/* Row number */
		);
	    }
	    /* Redraw list as needed */
	    if(list->map_state)
		CListDraw(list, CL_DRAW_AMOUNT_COMPLETE);
	}


	/* Reset prompts */
	PromptSetS(&wepw_ptr->flags_prompt, "");
	PromptSetS(&wepw_ptr->ocs_code_prompt, "");
	PromptSetS(&wepw_ptr->emission_type_prompt, "");
	PromptSetS(&wepw_ptr->amount_prompt, "");
	PromptSetS(&wepw_ptr->max_prompt, "");
	PromptSetS(&wepw_ptr->power_prompt, "");
	PromptSetS(&wepw_ptr->range_prompt, "");
	PromptSetS(&wepw_ptr->create_power_prompt, "");
	PromptSetS(&wepw_ptr->delay_prompt, "");
	PromptSetS(&wepw_ptr->use_sound_code_prompt, "");
	PromptSetS(&wepw_ptr->fire_sound_code_prompt, "");
	PromptSetS(&wepw_ptr->hit_sound_code_prompt, "");
	PromptSetS(&wepw_ptr->recover_sound_code_prompt, "");

	return(0);
}

/*
 *	Fetches values from weapon number wep_num and puts
 *	them into the prompts.  (does not update the list window).
 */
int WepWGetValues(wepw_struct *wepw_ptr, int wep_num)
{
	const char *s;
	const int len = 256;
	char text[len];
	xsw_weapon_struct *wep;

	if(wepw_ptr == NULL)
	    return(-1);

	wep = WepWGetWeaponPtr(wepw_ptr, wep_num);
	if(wep == NULL)
	    return(-1);


	/* Get values from weapon and put into prompts */

	*text = '\0';
        if(wep->flags & SW_WEP_FLAG_NO_LOCK)
            straddflag(text, SW_WEP_FLAG_STR_NO_LOCK, '|', len);           
	if(wep->flags & SW_WEP_FLAG_FIRE_CLOAKED)
	    straddflag(text, SW_WEP_FLAG_STR_FIRE_CLOAKED, '|', len);
	if(wep->flags & SW_WEP_FLAG_NO_FIRE_SOUND)
	    straddflag(text, SW_WEP_FLAG_STR_NO_FIRE_SOUND, '|', len);
	if(wep->flags & SW_WEP_FLAG_FIXED)
	    straddflag(text, SW_WEP_FLAG_STR_FIXED, '|', len);
        if(wep->flags & SW_WEP_FLAG_PENETRATE_SHIELD)
            straddflag(text, SW_WEP_FLAG_STR_PENETRATE_SHIELD, '|', len);
	PromptSetS(&wepw_ptr->flags_prompt, text);

	PromptSetI(&wepw_ptr->ocs_code_prompt, wep->ocs_code);

	switch(wep->emission_type)
	{
	  case SW_WEP_EMISSION_STREAM:
	    s = SW_WEP_EMISSION_STR_STREAM;
	    break;

	  case SW_WEP_EMISSION_PROJECTILE:
	    s = SW_WEP_EMISSION_STR_PROJECTILE;
	    break;

	  case SW_WEP_EMISSION_PULSE:
	    s = SW_WEP_EMISSION_STR_PULSE;
	    break;

	  default:	/* Default to stream */
	    s = SW_WEP_EMISSION_STR_STREAM;
	    break;
	}
	PromptSetS(&wepw_ptr->emission_type_prompt, s);

	PromptSetI(&wepw_ptr->amount_prompt, wep->amount);
	PromptSetI(&wepw_ptr->max_prompt, wep->max);
	PromptSetF(&wepw_ptr->power_prompt, wep->power);
	PromptSetI(&wepw_ptr->range_prompt, wep->range);
	PromptSetF(&wepw_ptr->create_power_prompt, wep->create_power);
	PromptSetI(&wepw_ptr->delay_prompt, wep->delay);
	PromptSetI(&wepw_ptr->range_prompt, wep->range);
	PromptSetI(&wepw_ptr->use_sound_code_prompt, wep->use_sound_code);
	PromptSetI(&wepw_ptr->fire_sound_code_prompt, wep->fire_sound_code);
	PromptSetI(&wepw_ptr->hit_sound_code_prompt, wep->hit_sound_code);
	PromptSetI(&wepw_ptr->recover_sound_code_prompt, wep->recover_sound_code);

	return(0);
}

/*
 *	Sets the values in the prompts of the weapon window wepw_ptr
 *	into the object's weapon wep_num.
 *
 *	If wep_num does not exist on object it will *not* be created.
 */
int WepWSetValues(wepw_struct *wepw_ptr, int wep_num)
{
	int i;
	const char *s;
	xsw_weapon_struct *wep;


	if(wepw_ptr == NULL)
	    return(-1); 

	wep = WepWGetWeaponPtr(wepw_ptr, wep_num);
	if(wep == NULL)
	    return(-1);


	/* Get values from prompts and set to weapon */

	s = PromptGetS(&wepw_ptr->flags_prompt);
	wep->flags = 0;
        if(strcasestr(s, SW_WEP_FLAG_STR_NO_LOCK))
            wep->flags |= SW_WEP_FLAG_NO_LOCK;
	if(strcasestr(s, SW_WEP_FLAG_STR_FIRE_CLOAKED))
	    wep->flags |= SW_WEP_FLAG_FIRE_CLOAKED;
	if(strcasestr(s, SW_WEP_FLAG_STR_NO_FIRE_SOUND))
	    wep->flags |= SW_WEP_FLAG_NO_FIRE_SOUND;
	if(strcasestr(s, SW_WEP_FLAG_STR_FIXED))
	    wep->flags |= SW_WEP_FLAG_FIXED;
        if(strcasestr(s, SW_WEP_FLAG_STR_PENETRATE_SHIELD))
            wep->flags |= SW_WEP_FLAG_PENETRATE_SHIELD;

	wep->ocs_code = PromptGetI(&wepw_ptr->ocs_code_prompt);

	s = PromptGetS(&wepw_ptr->emission_type_prompt);
	if(s == NULL)
	{
	    i = SW_WEP_EMISSION_STREAM;
	}
	else
	{
	    if(!strcasecmp(s, SW_WEP_EMISSION_STR_STREAM))
		i = SW_WEP_EMISSION_STREAM;
	    else if(!strcasecmp(s, SW_WEP_EMISSION_STR_PROJECTILE))
		i = SW_WEP_EMISSION_PROJECTILE;
	    else if(!strcasecmp(s, SW_WEP_EMISSION_STR_PULSE))
		i = SW_WEP_EMISSION_PULSE;
	    else
		i = SW_WEP_EMISSION_STREAM;
	}
	wep->emission_type = i;

	wep->amount = PromptGetI(&wepw_ptr->amount_prompt);
	wep->max = PromptGetI(&wepw_ptr->max_prompt);
	wep->power = PromptGetF(&wepw_ptr->power_prompt);
	wep->range = PromptGetL(&wepw_ptr->range_prompt);
	wep->create_power = PromptGetF(&wepw_ptr->create_power_prompt);
	wep->delay = PromptGetL(&wepw_ptr->delay_prompt);
	wep->use_sound_code = PromptGetL(&wepw_ptr->use_sound_code_prompt);
	wep->fire_sound_code = PromptGetL(&wepw_ptr->fire_sound_code_prompt);
	wep->hit_sound_code = PromptGetL(&wepw_ptr->hit_sound_code_prompt);
	wep->recover_sound_code = PromptGetL(&wepw_ptr->recover_sound_code_prompt);

	return(0);
}


/*
 *	Weapons list callback.
 */
static void WepWCListActivateCB(
	clist_struct *clist, void *data
)
{
	wepw_struct *wepw_ptr = (wepw_struct *)data;
	if(wepw_ptr == NULL)
	    return;


}

/*
 *	Create button callback.
 */
static int WepWCreateCB(void *ptr)
{
	int i, wepw_num;
	wepw_struct *wepw_ptr = NULL;
	clist_struct *list;
	xsw_object_struct *obj_ptr;
	char text[256];


	if(ptr == NULL)
	    return(-1);


	for(i = 0; i < total_wepws; i++)
	{
	    if(wepw[i] == NULL)
		continue;

	    if(wepw[i] == (wepw_struct *)ptr)
	    {
		wepw_ptr = (wepw_struct *)ptr;
		break;
	    }
	}
	if(wepw_ptr == NULL)
	    return(-1);
	else
	    wepw_num = i;

	list = &wepw_ptr->list;


	/* Apply values to currently selected weapon */
	if(list->total_sel_rows > 0)
	{
	    WepWSetValues(
		wepw_ptr,
		list->sel_row[list->total_sel_rows - 1]
	    );
	}


	/* Allocate a new weapon on object */
	obj_ptr = WepWGetObjectPtr(wepw_ptr);
	if(obj_ptr != NULL)
	{
	    if(obj_ptr->total_weapons >= SW_OBJ_WEAPONS_MAX)
	    {
		sprintf(text,
"Maximum of %i weapons exists for this object, cannot\n\
create additional weapon.",
		    SW_OBJ_WEAPONS_MAX
		);
		printdw(&dialog, text, wepw_ptr->toplevel);
	    }
	    else
	    {
		/* Allocate a new weapon */
		i = obj_ptr->total_weapons;
		obj_ptr->total_weapons++;
		obj_ptr->weapons = (xsw_weapon_struct **)realloc(
		    obj_ptr->weapons,
		    obj_ptr->total_weapons * sizeof(xsw_weapon_struct *)
		);
		if(obj_ptr->weapons == NULL)
		{
		    obj_ptr->total_weapons = 0;
		    return(-1);
		}

		obj_ptr->weapons[i] = (xsw_weapon_struct *)calloc(
		    1,
		    sizeof(xsw_weapon_struct)
		);

		/*   Redget all values, update list, and unselect
		 *   rows.
		 */
		WepWGetAllValues(wepw_ptr);
	    }
	}

	/* Update has changes marker */
	if(!wepw_ptr->has_changes)
	    wepw_ptr->has_changes = True;


	/* Redraw list as needed */
	if(list->map_state)
	    CListDraw(list, CL_DRAW_AMOUNT_COMPLETE);


	return(0);
}

/*      
 *      Delete button callback.
 */
static int WepWDeleteCB(void *ptr)
{
	int i, n, sel_wep_num;

	int wepw_num;
	wepw_struct *wepw_ptr = NULL;

	clist_struct *list;
	xsw_object_struct *obj_ptr;


	if(ptr == NULL)  
	    return(-1);


	for(i = 0; i < total_wepws; i++)
	{
	    if(wepw[i] == NULL)
		continue;

	    if(wepw[i] == (wepw_struct *)ptr)
	    {
		wepw_ptr = (wepw_struct *)ptr;
		break;
	    }   
	}       
	if(wepw_ptr == NULL)
	    return(-1);
	else
	    wepw_num = i;

	/* Get pointer to list */
	list = &wepw_ptr->list;


	/* Nothing selected? */
	if(list->total_sel_rows <= 0)
	    return(0);


	/* Get pointer to object (must be successful) */
	obj_ptr = WepWGetObjectPtr(wepw_ptr);
	if(obj_ptr == NULL)
	    return(-1);


	/*   Do not apply values to currently selected weapon,
	 *   since it will be deleted.
	 */

	sel_wep_num = list->sel_row[list->total_sel_rows - 1];
	if((sel_wep_num < 0) || (sel_wep_num >= obj_ptr->total_weapons))
	    return(-1);

	/* Deallocate selected weapon on object */
	if(obj_ptr->weapons != NULL)
	{
	    free(obj_ptr->weapons[sel_wep_num]);

	    /* Shift pointer array on object */
	    for(i = sel_wep_num, n = obj_ptr->total_weapons - 1;
		i < n;
		i++
	    )
		obj_ptr->weapons[i] = obj_ptr->weapons[i + 1];


	    /* Reallocate weapon pointers */
	    obj_ptr->total_weapons--;

	    if(obj_ptr->total_weapons > 0)
	    {
		obj_ptr->weapons = (xsw_weapon_struct **)realloc(
		    obj_ptr->weapons,
		    obj_ptr->total_weapons * sizeof(xsw_weapon_struct *)
		);
		if(obj_ptr->weapons == NULL)
		{
		    obj_ptr->total_weapons = 0;
		}
	    }
	    else
	    {
		free(obj_ptr->weapons);
		obj_ptr->weapons = NULL;

		obj_ptr->total_weapons = 0;
	    }
	}
	if(obj_ptr->total_weapons < 0)
	    obj_ptr->total_weapons = 0;


	/* Update has changes marker */
	if(!wepw_ptr->has_changes)
	    wepw_ptr->has_changes = True;


	/*   Redget all values, update list, and unselect
	 *   rows. 
	 */
	WepWGetAllValues(wepw_ptr);


	/* Redraw list as needed */
	if(list->map_state)
	    CListDraw(list, CL_DRAW_AMOUNT_COMPLETE);




	return(0);
}

/*
 *	Close button callback.
 */
static int WepWCloseCB(void *ptr)
{ 
	int i, wepw_num;
	wepw_struct *wepw_ptr = NULL;
	uew_struct *uew_ptr;


	if(ptr == NULL)
	    return(-1);


	for(i = 0; i < total_wepws; i++)
	{    
	    if(wepw[i] == NULL)
		continue;
	
	    if(wepw[i] == (wepw_struct *)ptr)
	    {
		wepw_ptr = (wepw_struct *)ptr;
		break;
	    }
	}
	if(wepw_ptr == NULL)
	    return(-1);
	else
	    wepw_num = i;


	/* Apply changes on currently selected weapon */
	if(wepw_ptr->list.total_sel_rows > 0)
	{
	    i = wepw_ptr->list.sel_row[
		wepw_ptr->list.total_sel_rows - 1
	    ];
	    WepWSetValues(
		wepw_ptr,
		i
	    );
	}

	/* Unmap */
	WepWUnmap(wepw_num);


	/* Update has changes marker on corresponding uew */
	if(wepw_ptr->has_changes)
	{
	    /* Mark uew to have changes too */
	    for(i = 0, uew_ptr = NULL; i < total_uews; i++)
	    {
		if(uew[i] == NULL)
		    continue;

		if(uew[i] == (uew_struct *)wepw_ptr->src)
		    uew_ptr = uew[i];
	    }
	    if(uew_ptr != NULL)
	    {
		if(!uew_ptr->has_changes)
		    uew_ptr->has_changes = True;
	    }

	    wepw_ptr->has_changes = False;
	}


	return(0);
} 


/*
 *      Initializes a weapons window.
 */
int WepWInit(int n, void *src_ptr)
{
	int x, y;
	char hotkey[PBTN_MAX_HOTKEYS];
	win_t w, parent = osw_gui[0].root_win;
	win_attr_t wattr;
	clist_struct *clist;
	wepw_struct *wepw_ptr;

	if(!WepWIsAllocated(n))
	    return(-1);

	wepw_ptr = wepw[n];

	wepw_ptr->map_state = False;
	wepw_ptr->is_in_focus = False;
	wepw_ptr->x = 0;
	wepw_ptr->y = 0;
	wepw_ptr->width = WEPW_WIDTH;
	wepw_ptr->height = WEPW_HEIGHT;
	    
	wepw_ptr->has_changes = False;
	wepw_ptr->src = src_ptr;

	/* Toplevel */
	OSWCreateWindow(
	    &w, parent,
	    wepw_ptr->x, wepw_ptr->y,
	    wepw_ptr->width, wepw_ptr->height
	);
	wepw_ptr->toplevel = w;
	OSWSetWindowWMProperties(
	    w,
	    WEPW_DEF_TITLE,
	    WEPW_DEF_ICON_TITLE,
	    ((ue_image.unvedit_icon_pm == 0) ?
		widget_global.std_icon_pm : ue_image.unvedit_icon_pm),
	    False,                      /* Let WM set coordinates? */
	    wepw_ptr->x, wepw_ptr->y,
	    wepw_ptr->width, wepw_ptr->height,
	    wepw_ptr->width, wepw_ptr->height,
	    OSW_WINDOW_FRAME_DIALOG,
	    0, NULL
	);
	OSWSetWindowInput(
	    w,
	    OSW_EVENTMASK_TOPLEVEL   
	);
	WidgetCenterWindow(w, WidgetCenterWindowToRoot);
	OSWGetWindowAttributes(w, &wattr);
	wepw_ptr->x = wattr.x;
	wepw_ptr->y = wattr.y; 
	wepw_ptr->width = wattr.width;
	wepw_ptr->height = wattr.height;

	parent = w;

	/* Weapons list */
	clist = &wepw_ptr->list;
	CListInit(
	    clist, parent,
	    10, 10,
	    MAX(
		(int)wepw_ptr->width - WEPW_BUTTON_WIDTH - 30,
		50
	    ),
	    100
	);
	clist->activate_cb = WepWCListActivateCB;
	clist->activate_data = wepw_ptr;
	clist->option = 0;
	CListColumnAppend(
	    clist,
	    "Weapon",
	    OSWQueryCurrentFont(),
	    widget_global.normal_text_pix,
	    0,
	    0
	);

	/* Create button */
	PBtnInit(
	    &wepw_ptr->create_wep_btn, parent,
	    (int)wepw_ptr->width - WEPW_BUTTON_WIDTH - 10,
	    (10 * 1) + (WEPW_BUTTON_HEIGHT * 0),
	    WEPW_BUTTON_WIDTH, WEPW_BUTTON_HEIGHT,
	    "Create",
	    PBTN_TALIGN_CENTER,
	    NULL,
	    wepw_ptr, WepWCreateCB
	);

	/* Delete button */
	PBtnInit(
	    &wepw_ptr->delete_wep_btn, parent,
	    (int)wepw_ptr->width - WEPW_BUTTON_WIDTH - 10,  
	    (10 * 2) + (WEPW_BUTTON_HEIGHT * 1),
	    WEPW_BUTTON_WIDTH, WEPW_BUTTON_HEIGHT,
	    "Delete",
	    PBTN_TALIGN_CENTER,
	    NULL,
	    wepw_ptr, WepWDeleteCB
	);

	/* Flags prompt */
	x = WEPW_MARGIN;
	y = 120;

	PromptInit(
	    &wepw_ptr->flags_prompt, parent,
	    x, y,
	    (int)wepw_ptr->width - (2 * WEPW_MARGIN),
	    WEPW_PROMPT_HEIGHT,
	    PROMPT_STYLE_FLUSHED,
	    "Flags:",
	    1024, 0
	);

	/* Object create script code */
	y += WEPW_PROMPT_HEIGHT;
	PromptInit(
	    &wepw_ptr->ocs_code_prompt, parent,
	    x, y,
	    (int)wepw_ptr->width - (2 * WEPW_MARGIN),
	    WEPW_PROMPT_HEIGHT,
	    PROMPT_STYLE_FLUSHED,
	    "Object Create Script Code:",
	    80, 0
	);

	/* Emission type */
	y += WEPW_PROMPT_HEIGHT;
	PromptInit(
	    &wepw_ptr->emission_type_prompt, parent,
	    x, y,
	    (int)wepw_ptr->width - (2 * WEPW_MARGIN),
	    WEPW_PROMPT_HEIGHT,
	    PROMPT_STYLE_FLUSHED,
	    "Emission Type:",
	    256, 0
	);

	/* Amount */
	y += WEPW_PROMPT_HEIGHT;
	PromptInit(
	    &wepw_ptr->amount_prompt, parent,
	    x, y,
	    static_cast<unsigned int>(((int)wepw_ptr->width / 2) - (1.5 * WEPW_MARGIN)),
	    WEPW_PROMPT_HEIGHT,  
	    PROMPT_STYLE_FLUSHED,
	    "Amount:",
	    80, 0
	);

	/* Max */
	PromptInit(
	    &wepw_ptr->max_prompt, parent,
	    static_cast<int>(x + ((int)wepw_ptr->width / 2) + (0.5 * WEPW_MARGIN)),
	    y,
	    ((int)wepw_ptr->width / 2) - (2 * WEPW_MARGIN),
	    WEPW_PROMPT_HEIGHT,
	    PROMPT_STYLE_FLUSHED,
	    "Max:",
	    80, 0
	);

	/* Power */
	y += WEPW_PROMPT_HEIGHT;
	PromptInit(
	    &wepw_ptr->power_prompt, parent,
	    x, y,
	    static_cast<unsigned int>(((int)wepw_ptr->width / 2) - (1.5 * WEPW_MARGIN)),
	    WEPW_PROMPT_HEIGHT,
	    PROMPT_STYLE_FLUSHED,
	    "Power:",
	    80, 0
	);

	/* Create power */
	PromptInit(
	    &wepw_ptr->create_power_prompt, parent,
	    static_cast<int>(x + ((int)wepw_ptr->width / 2) + (0.5 * WEPW_MARGIN)),
	    y,
	    ((int)wepw_ptr->width / 2) - (2 * WEPW_MARGIN),
	    WEPW_PROMPT_HEIGHT,
	    PROMPT_STYLE_FLUSHED,
	    "Create Power:",
	    80, 0
	);

	/* Range (for stream and sphere weapons) */
	y += WEPW_PROMPT_HEIGHT;
	PromptInit(
	    &wepw_ptr->range_prompt, parent,
	    x, y,
	    static_cast<unsigned int>(((int)wepw_ptr->width / 2) - (1.5 * WEPW_MARGIN)),
	    WEPW_PROMPT_HEIGHT,
	    PROMPT_STYLE_FLUSHED,
	    "Range (su):",
	    80, 0
	);

	/* Fire sound code */
	PromptInit(
	    &wepw_ptr->fire_sound_code_prompt, parent,
	    static_cast<int>(x + ((int)wepw_ptr->width / 2) + (0.5 * WEPW_MARGIN)),
	    y,
	    ((int)wepw_ptr->width / 2) - (2 * WEPW_MARGIN),
	    WEPW_PROMPT_HEIGHT,
	    PROMPT_STYLE_FLUSHED,
	    "Fire Sound Code:",
	    80, 0
	);

	/* Delay */
	y += WEPW_PROMPT_HEIGHT;
	PromptInit(
	    &wepw_ptr->delay_prompt, parent,
	    x, y,
	    static_cast<unsigned int>(((int)wepw_ptr->width / 2) - (1.5 * WEPW_MARGIN)),
	    WEPW_PROMPT_HEIGHT,
	    PROMPT_STYLE_FLUSHED,
	    "Delay (ms):",
	    80, 0
	);

	/* Link prompts togeather */
	wepw_ptr->flags_prompt.next = &wepw_ptr->ocs_code_prompt;
	wepw_ptr->flags_prompt.prev = &wepw_ptr->delay_prompt;

	wepw_ptr->ocs_code_prompt.next = &wepw_ptr->emission_type_prompt;
	wepw_ptr->ocs_code_prompt.prev = &wepw_ptr->delay_prompt;

	wepw_ptr->emission_type_prompt.next = &wepw_ptr->amount_prompt;
	wepw_ptr->emission_type_prompt.prev = &wepw_ptr->ocs_code_prompt;

	wepw_ptr->amount_prompt.next = &wepw_ptr->max_prompt;
	wepw_ptr->amount_prompt.prev = &wepw_ptr->emission_type_prompt;

	wepw_ptr->max_prompt.next = &wepw_ptr->power_prompt;
	wepw_ptr->max_prompt.prev = &wepw_ptr->amount_prompt;

	wepw_ptr->power_prompt.next = &wepw_ptr->create_power_prompt;
	wepw_ptr->power_prompt.prev = &wepw_ptr->max_prompt;

	wepw_ptr->create_power_prompt.next = &wepw_ptr->range_prompt;
	wepw_ptr->create_power_prompt.prev = &wepw_ptr->power_prompt;

	wepw_ptr->range_prompt.next = &wepw_ptr->fire_sound_code_prompt;
	wepw_ptr->range_prompt.prev = &wepw_ptr->create_power_prompt;

	wepw_ptr->fire_sound_code_prompt.next = &wepw_ptr->delay_prompt;
	wepw_ptr->fire_sound_code_prompt.prev = &wepw_ptr->range_prompt;

	wepw_ptr->delay_prompt.next = &wepw_ptr->flags_prompt;
	wepw_ptr->delay_prompt.prev = &wepw_ptr->fire_sound_code_prompt;


	/* Close button */
	PBtnInit(
	    &wepw_ptr->close_btn, parent,
	    (int)wepw_ptr->width - WEPW_BUTTON_WIDTH - 10,
	    (int)wepw_ptr->height - 10 - WEPW_BUTTON_HEIGHT,
	    WEPW_BUTTON_WIDTH, WEPW_BUTTON_HEIGHT,
	    "Close",
	    PBTN_TALIGN_CENTER,
	    NULL,
	    wepw_ptr, WepWCloseCB
	);
	hotkey[0] = '\n';
	hotkey[1] = 0x1b;
	hotkey[2] = '\0';
	PBtnSetHotKeys(&wepw_ptr->close_btn, hotkey);

	/* Need to update window menus on uews */
	UEWDoUpdateWindowMenus();

	return(0);
}


int WepWDraw(int n, int amount)
{
	win_t w;
	pixmap_t pixmap;
	wepw_struct *wepw_ptr;
	win_attr_t wattr;


	if(WepWIsAllocated(n))
	    wepw_ptr = wepw[n];
	else
	    return(-1);  
		
		
	/* Map as needed */
	if(!wepw_ptr->map_state)
	{
	    OSWMapRaised(wepw_ptr->toplevel);

	    CListMap(&wepw_ptr->list);

	    PBtnMap(&wepw_ptr->create_wep_btn);
	    PBtnMap(&wepw_ptr->delete_wep_btn); 

	    PromptMap(&wepw_ptr->flags_prompt);
	    PromptMap(&wepw_ptr->ocs_code_prompt);
	    PromptMap(&wepw_ptr->emission_type_prompt);
	    PromptMap(&wepw_ptr->amount_prompt);
	    PromptMap(&wepw_ptr->max_prompt);
	    PromptMap(&wepw_ptr->power_prompt);
	    PromptMap(&wepw_ptr->range_prompt);
	    PromptMap(&wepw_ptr->create_power_prompt);
	    PromptMap(&wepw_ptr->delay_prompt);
	    PromptMap(&wepw_ptr->use_sound_code_prompt);
	    PromptMap(&wepw_ptr->fire_sound_code_prompt);
	    PromptMap(&wepw_ptr->hit_sound_code_prompt);
	    PromptMap(&wepw_ptr->recover_sound_code_prompt);

	    PBtnMap(&wepw_ptr->close_btn);

	    wepw_ptr->map_state = 1;
	}


	if(wepw_ptr->toplevel_buf == 0)
	{
	    OSWGetWindowAttributes(wepw_ptr->toplevel, &wattr);
	    if(OSWCreatePixmap(
		&wepw_ptr->toplevel_buf,
		wattr.width, wattr.height
	    ))
		return(-1);
	}


	if(1)
	{
	    w = wepw_ptr->toplevel;
	    pixmap = wepw_ptr->toplevel_buf;

	    OSWGetWindowAttributes(w, &wattr);

	    /* Redraw background */
	    if(widget_global.force_mono)
	    {
		OSWClearPixmap(
		    pixmap,
		    wattr.width, wattr.height,
		    osw_gui[0].black_pix
		);
	    }
	    else
	    {
		WidgetPutImageTile(
		    pixmap,
		    widget_global.std_bkg_img,
		    wattr.width, wattr.height
		);

		OSWSetFgPix(widget_global.surface_shadow_pix);
		OSWDrawLine(
		    pixmap,
		    0,
		    (int)wattr.height - 25 - WEPW_BUTTON_HEIGHT,
		    wattr.width,
		    (int)wattr.height - 25 - WEPW_BUTTON_HEIGHT
		);
		OSWSetFgPix(widget_global.surface_highlight_pix);
		OSWDrawLine(
		    pixmap,
		    0,
		    (int)wattr.height - 24 - WEPW_BUTTON_HEIGHT,
		    wattr.width,
		    (int)wattr.height - 24 - WEPW_BUTTON_HEIGHT
		);  

	    }



	    OSWPutBufferToWindow(w, pixmap);
	}


	return(0);
}


int WepWManage(int n, event_t *event)
{
	int i, p;
	keycode_t keycode;
	int events_handled = 0;
	wepw_struct *wepw_ptr;
	clist_struct *list;


	static_wepw::delete_wepw = NULL;


	if(WepWIsAllocated(n))
	    wepw_ptr = wepw[n];
	else
	    return(events_handled);


	if(event == NULL)
	    return(events_handled);

	if(!wepw_ptr->map_state &&
	   (event->type != MapNotify)
	)
	    return(events_handled);


	switch(event->type)
	{
	  /* ******************************************************* */
	  case KeyPress:
	    if(!wepw_ptr->is_in_focus)
		break;

	    keycode = event->xkey.keycode;

	    /* Enter */
	    if((keycode == osw_keycode.enter) ||
	       (keycode == osw_keycode.np_enter)
	    )
	    {
		WepWCloseCB((void *)wepw_ptr);

		/* Delete this wepw as needed */
		if(static_wepw::delete_wepw == wepw_ptr)
		{
		    WepWDelete(n);
		    UEWDoUpdateWindowMenus();
		}

		events_handled++;
		return(events_handled);
	    }
	    break;

	  /* ******************************************************* */
	  case KeyRelease:
	    if(!wepw_ptr->is_in_focus) 
		break;

	    break;

	  /* ******************************************************* */
	  case Expose:
	    if(event->xany.window == wepw_ptr->toplevel)
	    {
		events_handled++;
	    }
	    break;

	  /* ******************************************************* */
	  case FocusIn:
	    if(event->xany.window == wepw_ptr->toplevel)
	    {
		wepw_ptr->is_in_focus = 1;

		events_handled++;
		return(events_handled);
	    }
	    break;

	  /* ******************************************************* */
	  case FocusOut:
	    if(event->xany.window == wepw_ptr->toplevel)
	    {
		wepw_ptr->is_in_focus = 0;

		events_handled++;
		return(events_handled);
	    }   
	    break;
	 
	  /* ******************************************************* */
	  case ClientMessage:
	    if(OSWIsEventDestroyWindow(wepw_ptr->toplevel, event))
	    {
		WepWCloseCB((void *)wepw_ptr);

		/* Delete this wepw as needed */
		if(static_wepw::delete_wepw == wepw_ptr)
		{
		    WepWDelete(n);
		    UEWDoUpdateWindowMenus();
		}

		events_handled++;
		return(events_handled);
	    }
	    break;
	}

	if(events_handled > 0)
	{
	    WepWDraw(n, 0);
	}


	/* Manage widgets */

	/* Weapons list */
	if(events_handled == 0)
	{
	    list = &wepw_ptr->list;

	    if(list->total_sel_rows > 0)
		p = list->sel_row[list->total_sel_rows - 1];
	    else
		p = -1;

	    events_handled += CListManage(
		&wepw_ptr->list,
		event
	    );
	    if(events_handled > 0)
	    {
		if(list->total_sel_rows > 0)
		    i = list->sel_row[list->total_sel_rows - 1]; 
		else
		    i = -1;

		if(p != i)
		{
		    WepWSetValues(wepw_ptr, p);
		    WepWGetValues(wepw_ptr, i);
		}
	    }
	}            

	if(events_handled == 0)
	    events_handled += PBtnManage(&wepw_ptr->create_wep_btn, event);
	if(events_handled == 0)
	    events_handled += PBtnManage(&wepw_ptr->delete_wep_btn, event);
	if(events_handled == 0)
	    events_handled += PBtnManage(&wepw_ptr->close_btn, event);

	if(events_handled == 0)
	    events_handled += PromptManage(&wepw_ptr->flags_prompt, event);
	if(events_handled == 0)
	    events_handled += PromptManage(&wepw_ptr->ocs_code_prompt, event);
	if(events_handled == 0)
	    events_handled += PromptManage(&wepw_ptr->emission_type_prompt, event);
	if(events_handled == 0)
	    events_handled += PromptManage(&wepw_ptr->amount_prompt, event);
	if(events_handled == 0)
	    events_handled += PromptManage(&wepw_ptr->max_prompt, event);
	if(events_handled == 0)
	    events_handled += PromptManage(&wepw_ptr->power_prompt, event);
	if(events_handled == 0)
	    events_handled += PromptManage(&wepw_ptr->range_prompt, event);
	if(events_handled == 0)
	    events_handled += PromptManage(&wepw_ptr->create_power_prompt, event);
	if(events_handled == 0)
	    events_handled += PromptManage(&wepw_ptr->delay_prompt, event);
	if(events_handled == 0)
	    events_handled += PromptManage(&wepw_ptr->use_sound_code_prompt, event);
	if(events_handled == 0)
	    events_handled += PromptManage(&wepw_ptr->fire_sound_code_prompt, event);
	if(events_handled == 0)
	    events_handled += PromptManage(&wepw_ptr->hit_sound_code_prompt, event);
	if(events_handled == 0)
	    events_handled += PromptManage(&wepw_ptr->recover_sound_code_prompt, event);


	/* Delete this wepw as needed */
	if(static_wepw::delete_wepw == wepw_ptr)
	{
	    WepWDelete(n);
	    UEWDoUpdateWindowMenus();
	}

	return(events_handled);
}


int WepWManageAll(event_t *event)
{
	int i;
	int events_handled = 0;


	if(event == NULL)
	    return(events_handled);


	for(i = 0; i < total_wepws; i++)
	{
	    events_handled += WepWManage(i, event);
	}


	return(events_handled);
}


void WepWMap(int n)
{
	wepw_struct *wepw_ptr;


	if(WepWIsAllocated(n))
	    wepw_ptr = wepw[n];
	else
	    return;


	wepw_ptr->map_state = 0;
	WepWDraw(n, 0);


	return;
}

/*
 *	Procedure to map weapon window and fetch values.
 */
void WepWDoMapValues(int n, void *src, int obj_num)
{
	int i;
	wepw_struct *wepw_ptr;
	uew_struct *uew_ptr;
	char text[SW_OBJ_NAME_MAX + 80];


	if(WepWIsAllocated(n))
	    wepw_ptr = wepw[n];
	else
	    return;


	/* Reset wepw has changes marker */
	if(wepw_ptr->has_changes)
	    wepw_ptr->has_changes = False;


	/* Look for uew */
	for(i = 0; i < total_uews; i++)
	{
	    if(uew[i] == NULL)
		continue;
	    if(uew[i] == (uew_struct *)src)
		break;
	}
	if(i < total_uews)
	{
	    uew_ptr = (uew_struct *)src;

	    wepw_ptr->src = src;
	    if(UEWIsObjectGarbage(uew_ptr, obj_num))
	    {
		wepw_ptr->src_obj_num = -1;
		wepw_ptr->src_obj_ptr = NULL;

		OSWSetWindowTitle(
		    wepw_ptr->toplevel,
		    WEPW_DEF_TITLE
		);
	    }
	    else
	    {
		wepw_ptr->src_obj_num = obj_num;
		wepw_ptr->src_obj_ptr = uew_ptr->object[obj_num];

		sprintf(
		    text,
		    "%s Weapons",
		    UNVGetObjectFormalName(
			wepw_ptr->src_obj_ptr,
			wepw_ptr->src_obj_num
		    )
		);
		OSWSetWindowTitle(
		    wepw_ptr->toplevel,
		    text
		);
	    }
	}
	else
	{
	    wepw_ptr->src = NULL;

	    wepw_ptr->src_obj_num = -1;
	    wepw_ptr->src_obj_ptr = NULL;
	}


	/* Update list and reset prompts */
	WepWGetAllValues(wepw_ptr);


	/* Need to update all uew window menus */
	UEWDoUpdateWindowMenus();


	/* Map window */
	WepWMap(n);


	return;
}


void WepWUnmap(int n)
{
	wepw_struct *wepw_ptr;
 
 
	if(WepWIsAllocated(n))
	    wepw_ptr = wepw[n];
	else
	    return;


	OSWUnmapWindow(wepw_ptr->toplevel);
	wepw_ptr->map_state = 0;

	OSWDestroyPixmap(&wepw_ptr->toplevel_buf);


	/* Schedual this wepw for deletion */
	static_wepw::delete_wepw = wepw_ptr;


	return;
}


/*
 *      Destroys all allocated resources for weapon
 *      window n.
 */
void WepWDestroy(int n)
{
	wepw_struct *wepw_ptr;
	
	    
	if(WepWIsAllocated(n))
	    wepw_ptr = wepw[n];
	else
	    return;
	  
	
	wepw_ptr->map_state = 0;
	wepw_ptr->is_in_focus = 0;
	    
	wepw_ptr->has_changes = False;
	wepw_ptr->src = NULL;
	wepw_ptr->src_obj_num = -1;
	wepw_ptr->src_obj_ptr = NULL;
	
	if(IDC())
	{
	    PBtnDestroy(&wepw_ptr->close_btn);

	    PromptDestroy(&wepw_ptr->flags_prompt);
	    PromptDestroy(&wepw_ptr->ocs_code_prompt);
	    PromptDestroy(&wepw_ptr->emission_type_prompt);
	    PromptDestroy(&wepw_ptr->amount_prompt);
	    PromptDestroy(&wepw_ptr->max_prompt);
	    PromptDestroy(&wepw_ptr->power_prompt);
	    PromptDestroy(&wepw_ptr->range_prompt);
	    PromptDestroy(&wepw_ptr->create_power_prompt);
	    PromptDestroy(&wepw_ptr->delay_prompt);
	    PromptDestroy(&wepw_ptr->use_sound_code_prompt);
	    PromptDestroy(&wepw_ptr->fire_sound_code_prompt);
	    PromptDestroy(&wepw_ptr->hit_sound_code_prompt);
	    PromptDestroy(&wepw_ptr->recover_sound_code_prompt);

	    PBtnDestroy(&wepw_ptr->create_wep_btn);
	    PBtnDestroy(&wepw_ptr->delete_wep_btn);
 
	    CListDestroy(&wepw_ptr->list);


	    OSWDestroyPixmap(&wepw_ptr->toplevel_buf);
	    OSWDestroyWindow(&wepw_ptr->toplevel);
	}
	
	
	return;
}           




